@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright (C) 2014-2019 GEM Foundation
+# Copyright (C) 2014-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -21,7 +21,6 @@ Utility functions of general interest.
 """
 import os
 import sys
-import imp
 import copy
 import math
 import socket
@@ -35,15 +34,16 @@ import tempfile
 import importlib
 import itertools
 import subprocess
-import collections
-from collections.abc import Mapping, Container
-
+from collections.abc import Mapping, Container, MutableSequence
 import numpy
 from decorator import decorator
 from openquake.baselib.python3compat import decode
 
+U16 = numpy.uint16
 F32 = numpy.float32
 F64 = numpy.float64
+TWO16 = 2 ** 16
+BASE64 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-'
 
 
 def duplicated(items):
@@ -79,7 +79,7 @@ def nokey(item):
     return 'Unspecified'
 
 
-class WeightedSequence(collections.MutableSequence):
+class WeightedSequence(MutableSequence):
     """
     A wrapper over a sequence of weighted items with a total weight attribute.
     Adding items automatically increases the weight.
@@ -271,6 +271,22 @@ def split_in_slices(number, num_slices):
     return slices
 
 
+def gen_slices(start, stop, blocksize):
+    """
+    Yields slices of lenght at most block_size.
+
+    >>> list(gen_slices(1, 6, 2))
+    [slice(1, 3, None), slice(3, 5, None), slice(5, 6, None)]
+    """
+    assert start <= stop, (start, stop)
+    assert blocksize > 0, blocksize
+    while True:
+        yield slice(start, min(start + blocksize, stop))
+        start += blocksize
+        if start >= stop:
+            break
+
+
 def split_in_blocks(sequence, hint, weight=lambda item: 1, key=nokey):
     """
     Split the `sequence` in a number of WeightedSequences close to `hint`.
@@ -323,13 +339,8 @@ def assert_close(a, b, rtol=1e-07, atol=0, context=None):
         # another shortcut
         assert a == b, (a, b)
         return
-    if hasattr(a, '_slots_'):  # record-like objects
-        assert a._slots_ == b._slots_
-        for x in a._slots_:
-            assert_close(getattr(a, x), getattr(b, x), rtol, atol, x)
-        return
     if hasattr(a, 'keys'):  # dict-like objects
-        assert a.keys() == b.keys()
+        assert a.keys() == b.keys(), set(a).symmetric_difference(set(b))
         for x in a:
             if x != '__geom__':
                 assert_close(a[x], b[x], rtol, atol, x)
@@ -356,7 +367,7 @@ _tmp_paths = []
 def gettemp(content=None, dir=None, prefix="tmp", suffix="tmp"):
     """Create temporary file with the given content.
 
-    Please note: the temporary file must be deleted by the caller.
+    Please note: the temporary file can be deleted by the caller or not.
 
     :param string content: the content to write to the temporary file.
     :param string dir: directory where the file should be created
@@ -369,12 +380,11 @@ def gettemp(content=None, dir=None, prefix="tmp", suffix="tmp"):
             os.makedirs(dir)
     fh, path = tempfile.mkstemp(dir=dir, prefix=prefix, suffix=suffix)
     _tmp_paths.append(path)
-    if content:
-        fh = os.fdopen(fh, "wb")
-        if hasattr(content, 'encode'):
-            content = content.encode('utf8')
-        fh.write(content)
-        fh.close()
+    with os.fdopen(fh, "wb") as fh:
+        if content:
+            if hasattr(content, 'encode'):
+                content = content.encode('utf8')
+            fh.write(content)
     return path
 
 
@@ -437,6 +447,10 @@ def run_in_process(code, *args):
         print(exc.cmd[-1], file=sys.stderr)
         raise
     if out:
+        out = out.rstrip(b'\x1b[?1034h')
+        # this is absurd, but it happens: just importing a module can
+        # produce escape sequences in stdout, see for instance
+        # https://bugs.python.org/issue19884
         return eval(out, {}, {})
 
 
@@ -489,25 +503,6 @@ def assert_independent(package, *packages):
                 raise CodeDependencyError('%s depends on %s' % (package, pkg))
 
 
-def search_module(module, syspath=sys.path):
-    """
-    Given a module name (possibly with dots) returns the corresponding
-    filepath, or None, if the module cannot be found.
-
-    :param module: (dotted) name of the Python module to look for
-    :param syspath: a list of directories to search (default sys.path)
-    """
-    lst = module.split(".")
-    pkg, submodule = lst[0], ".".join(lst[1:])
-    try:
-        fileobj, filepath, descr = imp.find_module(pkg, syspath)
-    except ImportError:
-        return
-    if submodule:  # recursive search
-        return search_module(submodule, [filepath])
-    return filepath
-
-
 class CallableDict(dict):
     r"""
     A callable object built on top of a dictionary of functions, used
@@ -600,8 +595,8 @@ class AccumDict(dict):
 
     The multiplication has been defined::
 
-     >>> prob1 = AccumDict(a=0.4, b=0.5)
-     >>> prob2 = AccumDict(b=0.5)
+     >>> prob1 = AccumDict(dict(a=0.4, b=0.5))
+     >>> prob2 = AccumDict(dict(b=0.5))
      >>> prob1 * prob2
      {'a': 0.4, 'b': 0.25}
      >>> prob1 * 1.2
@@ -627,19 +622,23 @@ class AccumDict(dict):
     accumulator, therefore each key has a different accumulator, which
     initially is the empty list (in this case).
     """
-    def __init__(self, dic=None, accum=None, **kw):
+    def __init__(self, dic=None, accum=None, keys=()):
+        for key in keys:
+            self[key] = copy.deepcopy(accum)
         if dic:
             self.update(dic)
-        self.update(kw)
         self.accum = accum
 
     def __iadd__(self, other):
         if hasattr(other, 'items'):
             for k, v in other.items():
-                try:
-                    self[k] = self[k] + v
-                except KeyError:
+                if k not in self:
                     self[k] = v
+                elif isinstance(v, list):
+                    # specialized for speed
+                    self[k].extend(v)
+                else:
+                    self[k] = self[k] + v
         else:  # add other to all elements
             for k in self:
                 self[k] = self[k] + other
@@ -735,7 +734,7 @@ def _slicedict_n(imt_dt):
     return slicedic, n
 
 
-class DictArray(collections.Mapping):
+class DictArray(Mapping):
     """
     A small wrapper over a dictionary of arrays serializable to HDF5:
 
@@ -757,8 +756,17 @@ class DictArray(collections.Mapping):
              for imt, imls in sorted(imtls.items())])
         self.slicedic, num_levels = _slicedict_n(dt)
         self.array = numpy.zeros(num_levels, F64)
+        lenset = set()
         for imt, imls in imtls.items():
             self[imt] = imls
+            try:
+                lenset.add(len(imls))
+            except TypeError:
+                lenset.add(1)
+        if len(lenset) == 1:
+            self.L1 = lenset.pop()
+        else:
+            self.L1 = None
 
     def new(self, array):
         """
@@ -864,6 +872,45 @@ def groupby2(records, kfield, vfield):
     return list(dic.items())  # Python3 compatible
 
 
+def bin_idxs(values, nbins, key=None, minval=None, maxval=None):
+    """
+    :param values: an array of N floats (or arrays)
+    :returns: an array of N indices
+    """
+    assert len(values)
+    if key is not None:
+        values = numpy.array([key(val) for val in values])
+    if minval is None:
+        minval = values.min()
+    if maxval is None:
+        maxval = values.max()
+    if minval == maxval:
+        bins = [minval] * nbins
+    else:
+        bins = numpy.arange(minval, maxval, (maxval-minval) / nbins)
+    return numpy.searchsorted(bins, values, side='right')
+
+
+def groupby_bin(values, nbins, key=None, minval=None, maxval=None):
+    """
+    >>> values = numpy.arange(10)
+    >>> for group in groupby_bin(values, 3):
+    ...     print(group)
+    [0, 1, 2]
+    [3, 4, 5]
+    [6, 7, 8, 9]
+    """
+    if len(values) == 0:  # do nothing
+        return values
+    idxs = bin_idxs(values, nbins, key, minval, maxval)
+    acc = AccumDict(accum=[])
+    for idx, val in zip(idxs, values):
+        if isinstance(idx, numpy.ndarray):
+            idx = tuple(idx)  # make it hashable
+        acc[idx].append(val)
+    return acc.values()
+
+
 def _reducerecords(group):
     records = list(group)
     return numpy.array(records, records[0].dtype)
@@ -876,6 +923,102 @@ def group_array(array, *kfields):
     return groupby(array, operator.itemgetter(*kfields), _reducerecords)
 
 
+def multi_index(shape, axis=None):
+    """
+    :param shape: a shape of lenght L with P = S1 * S2 * ... * SL
+    :param axis: None or an integer in the range 0 .. L -1
+    :yields:
+        P tuples of indices with a slice(None) at the axis position (if any)
+    """
+    if any(s >= TWO16 for s in shape):
+        raise ValueError('Shape too big: ' + str(shape))
+    ranges = (range(s) for s in shape)
+    if axis is None:
+        yield from itertools.product(*ranges)
+    for tup in itertools.product(*ranges):
+        lst = list(tup)
+        lst.insert(axis, slice(None))
+        yield tuple(lst)
+
+
+def fast_agg(indices, values=None, axis=0, factor=None):
+    """
+    :param indices: N indices in the range 0 ... M - 1 with M < N
+    :param values: N values (can be arrays)
+    :returns: M aggregated values (can be arrays)
+
+    >>> values = numpy.array([[.1, .11], [.2, .22], [.3, .33], [.4, .44]])
+    >>> fast_agg([0, 1, 1, 0], values)
+    array([[0.5 , 0.55],
+           [0.5 , 0.55]])
+    """
+    if values is None:
+        values = numpy.ones_like(indices)
+    N = len(values)
+    if len(indices) != N:
+        raise ValueError('There are %d values but %d indices' %
+                         (N, len(indices)))
+    shp = values.shape[1:]
+    if not shp:
+        return numpy.bincount(indices, values)
+    M = max(indices) + 1
+    lst = list(shp)
+    lst.insert(axis, M)
+    res = numpy.zeros(lst, values.dtype)
+    for mi in multi_index(shp, axis):
+        vals = values[mi] if factor is None else values[mi] * factor
+        res[mi] = numpy.bincount(indices, vals)
+    return res
+
+
+def fast_agg2(tags, values=None, axis=0):
+    """
+    :param tags: N non-unique tags out of M
+    :param values: N values (can be arrays)
+    :returns: (M unique tags, M aggregated values)
+
+    >>> values = numpy.array([[.1, .11], [.2, .22], [.3, .33], [.4, .44]])
+    >>> fast_agg2(['A', 'B', 'B', 'A'], values)
+    (array(['A', 'B'], dtype='<U1'), array([[0.5 , 0.55],
+           [0.5 , 0.55]]))
+
+    It can also be used to count the number of tags:
+
+    >>> fast_agg2(['A', 'B', 'B', 'A', 'A'])
+    (array(['A', 'B'], dtype='<U1'), array([3., 2.]))
+    """
+    uniq, indices = numpy.unique(tags, return_inverse=True)
+    return uniq, fast_agg(indices, values, axis)
+
+
+def fast_agg3(structured_array, kfield, vfields, factor=None):
+    """
+    Aggregate a structured array with a key field (the kfield)
+    and some value fields (the vfields).
+
+    >>> data = numpy.array([(1, 2.4), (1, 1.6), (2, 2.5)],
+    ...                    [('aid', U16), ('val', F32)])
+    >>> fast_agg3(data, 'aid', ['val'])
+    array([(1, 4. ), (2, 2.5)], dtype=[('aid', '<u2'), ('val', '<f4')])
+    """
+    allnames = structured_array.dtype.names
+    assert kfield in allnames, kfield
+    for vfield in vfields:
+        assert vfield in allnames, vfield
+    tags = structured_array[kfield]
+    uniq, indices = numpy.unique(tags, return_inverse=True)
+    dic = {}
+    dtlist = [(kfield, structured_array.dtype[kfield])]
+    for name in vfields:
+        dic[name] = fast_agg(indices, structured_array[name], factor=factor)
+        dtlist.append((name, structured_array.dtype[name]))
+    res = numpy.zeros(len(uniq), dtlist)
+    res[kfield] = uniq
+    for name in dic:
+        res[name] = dic[name]
+    return res
+
+
 def count(groupiter):
     return sum(1 for row in groupiter)
 
@@ -1075,7 +1218,7 @@ def zipfiles(fnames, archive, mode='w', log=lambda msg: None, cleanup=False):
     Build a zip archive from the given file names.
 
     :param fnames: list of path names
-    :param archive: path of the archive
+    :param archive: path of the archive or BytesIO object
     """
     prefix = len(os.path.commonprefix([os.path.dirname(f) for f in fnames]))
     with zipfile.ZipFile(
@@ -1085,7 +1228,6 @@ def zipfiles(fnames, archive, mode='w', log=lambda msg: None, cleanup=False):
             z.write(f, f[prefix:])
             if cleanup:  # remove the zipped file
                 os.remove(f)
-    log('Generated %s' % archive)
     return archive
 
 
@@ -1159,3 +1301,108 @@ def getsizeof(o, ids=None):
         return nbytes + sum(getsizeof(x, ids) for x in o)
 
     return nbytes
+
+
+def add_defaults(array, **kw):
+    """
+    :param array: a structured array
+    :param kw: a dictionary field name -> default value
+    :returns: a new array with additional fields with default values
+    """
+    dtlist = [(name, array.dtype[name]) for name in array.dtype.names]
+    for k, v in kw.items():
+        if k not in array.dtype.names:
+            dtlist.append((k, type(v)))
+    new = numpy.zeros(array.shape, dtlist)
+    for name in array.dtype.names:
+        new[name] = array[name]
+    for k, v in kw.items():
+        if k not in array.dtype.names:
+            new[k] = v
+    return new
+
+
+def get_duplicates(array, *fields):
+    """
+    :returns: a dictionary {key: num_dupl} for duplicate records
+    """
+    uniq = numpy.unique(array[list(fields)])
+    if len(uniq) == len(array):  # no duplicates
+        return {}
+    return {k: len(g) for k, g in group_array(array, *fields).items()
+            if len(g) > 1}
+
+
+def add_columns(a, b, on, cols=None):
+    """
+    >>> a_dt = [('aid', int), ('eid', int), ('loss', float)]
+    >>> b_dt = [('ordinal', int), ('zipcode', int)]
+    >>> a = numpy.array([(1, 0, 2.4), (2, 0, 2.2),
+    ...                  (1, 1, 2.1), (2, 1, 2.3)], a_dt)
+    >>> b = numpy.array([(0, 20126), (1, 20127), (2, 20128)], b_dt)
+    >>> add_columns(a, b, 'aid', ['zipcode'])
+    array([(1, 0, 2.4, 20127), (2, 0, 2.2, 20128), (1, 1, 2.1, 20127),
+           (2, 1, 2.3, 20128)],
+          dtype=[('aid', '<i8'), ('eid', '<i8'), ('loss', '<f8'), ('zipcode', '<i8')])
+    """
+    if cols is None:
+        cols = b.dtype.names
+    dtlist = []
+    for name in a.dtype.names:
+        dtlist.append((name, a.dtype[name]))
+    for name in cols:
+        dtlist.append((name, b.dtype[name]))
+    new = numpy.zeros(len(a), dtlist)
+    for name in a.dtype.names:
+        new[name] = a[name]
+    idxs = a[on]
+    for name in cols:
+        new[name] = b[name][idxs]
+    return new
+
+
+def categorize(values, nchars=2):
+    """
+    Takes an array with duplicate values and categorize it, i.e. replace
+    the values with codes of length nchars in base64. With nchars=2 4096
+    unique values can be encoded, if there are more nchars must be increased
+    otherwise a ValueError will be raised.
+
+    :param values: an array of V non-unique values
+    :param nchars: number of characters in base64 for each code
+    :returns: an array of V non-unique codes
+
+    >>> categorize([1,2,2,3,4,1,1,2]) # 8 values, 4 unique ones
+    array([b'AA', b'AB', b'AB', b'AC', b'AD', b'AA', b'AA', b'AB'],
+          dtype='|S2')
+    """
+    uvalues = numpy.unique(values)
+    mvalues = 64 ** nchars  # maximum number of unique values
+    if len(uvalues) > mvalues:
+        raise ValueError(
+            f'There are too many unique values ({len(uvalues)} > {mvalues})')
+    prod = itertools.product(*[BASE64] * nchars)
+    dic = {uvalue: ''.join(chars) for uvalue, chars in zip(uvalues, prod)}
+    return numpy.array([dic[v] for v in values], (numpy.string_, nchars))
+
+
+def get_array_nbytes(sizedict):
+    """
+    :param sizedict: mapping name -> num_dimensions
+    :returns: (size of the array in bytes, descriptive message)
+
+    >>> get_array_nbytes(dict(nsites=2, nbins=5))
+    (80, '(nsites=2) * (nbins=5) * 8 bytes = 80 B')
+    """
+    nbytes = numpy.prod(list(sizedict.values())) * 8
+    prod = ' * '.join('(%s=%d)' % item for item in sizedict.items())
+    return nbytes, '%s * 8 bytes = %s' % (prod, humansize(nbytes))
+
+
+def gen_subclasses(cls):
+    """
+    :returns: the subclasses of `cls`, ordered by name
+    """
+    for subclass in sorted(cls.__subclasses__(), key=lambda cls: cls.__name__):
+        yield subclass
+        yield from gen_subclasses(subclass)
