@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright (C) 2015-2019 GEM Foundation
+# Copyright (C) 2015-2020 GEM Foundation
 
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -17,19 +17,20 @@
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-import sys
 import ast
+import csv
 import inspect
 import logging
-import operator
 import tempfile
 import importlib
 import itertools
 from numbers import Number
 from urllib.parse import quote_plus, unquote_plus
 import collections
+import toml
 import numpy
 import h5py
+from openquake.baselib import InvalidFile
 from openquake.baselib.python3compat import encode, decode
 
 vbytes = h5py.special_dtype(vlen=bytes)
@@ -45,8 +46,11 @@ def maybe_encode(value):
     """
     If value is a sequence of strings, encode it
     """
-    if isinstance(value, (list, tuple)) and isinstance(value[0], str):
-        return encode(value)
+    if isinstance(value, bytes):
+        return numpy.void(value)
+    if isinstance(value, (list, tuple)):
+        if value and isinstance(value[0], str):
+            return encode(value)
     return value
 
 
@@ -106,26 +110,6 @@ def extend(dset, array, **attrs):
     return newlength
 
 
-def extend3(filename, key, array, **attrs):
-    """
-    Extend an HDF5 file dataset with the given array
-    """
-    with h5py.File(filename) as h5:
-        try:
-            dset = h5[key]
-        except KeyError:
-            if array.dtype.name == 'object':  # vlen array
-                shape = (None,) + preshape(array[0])
-            else:
-                shape = (None,) + array.shape[1:]
-            dset = create(h5, key, array.dtype, shape)
-        length = extend(dset, array)
-        for key, val in attrs.items():
-            dset.attrs[key] = val
-        h5.flush()
-    return length
-
-
 class LiteralAttrs(object):
     """
     A class to serialize a set of parameters in HDF5 format. The goal is to
@@ -240,7 +224,7 @@ class ByteCounter(object):
             self.nbytes += nbytes
 
 
-class Group(collections.Mapping):
+class Group(collections.abc.Mapping):
     """
     A mock for a h5py group object
     """
@@ -270,12 +254,20 @@ class File(h5py.File):
     >>> f = File('/tmp/x.h5', 'w')
     >>> f['dic'] = dict(a=dict(x=1, y=2), b=3)
     >>> dic = f['dic']
-    >>> dic['a']['x'].value
+    >>> dic['a']['x'][()]
     1
-    >>> dic['b'].value
+    >>> dic['b'][()]
     3
     >>> f.close()
     """
+    def __init__(self, name, mode=None, driver=None, libver='latest',
+                 userblock_size=None, swmr=True, rdcc_nslots=None,
+                 rdcc_nbytes=None, rdcc_w0=None, track_order=None,
+                 **kwds):
+        super().__init__(name, mode, driver, libver,
+                         userblock_size, swmr, rdcc_nslots,
+                         rdcc_nbytes, rdcc_w0, track_order, **kwds)
+
     @classmethod
     def temporary(cls):
         """
@@ -289,7 +281,7 @@ class File(h5py.File):
         self.path = path
         return self
 
-    def save_vlen(self, key, data):
+    def save_vlen(self, key, data):  # used in SourceWriterTestCase
         """
         Save a sequence of variable-length arrays
 
@@ -302,17 +294,9 @@ class File(h5py.File):
         except KeyError:
             vdt = h5py.special_dtype(vlen=data[0].dtype)
             dset = create(self, key, vdt, shape, fillvalue=None)
-        nbytes = dset.attrs.get('nbytes', 0)
-        totlen = dset.attrs.get('totlen', 0)
-        for i, val in enumerate(data):
-            nbytes += val.nbytes
-            totlen += len(val)
         length = len(dset)
         dset.resize((length + len(data),) + shape[1:])
-        for i, arr in enumerate(data):
-            dset[length + i] = arr
-        dset.attrs['nbytes'] = nbytes
-        dset.attrs['totlen'] = totlen
+        dset[length:length + len(data)] = data
 
     def save_attrs(self, path, attrs, **kw):
         items = list(attrs.items()) + list(kw.items())
@@ -333,7 +317,11 @@ class File(h5py.File):
         else:
             pyclass = ''
         if isinstance(obj, (dict, Group)) and obj:
-            for k, v in sorted(obj.items()):
+            for k, v in obj.items():
+                # NB: there was a line sorted(obj.items()) here
+                # it was removed because it caused the absurd issue
+                # https://github.com/gem/oq-engine/issues/4761
+                # for an exposure with more than 65536 assets
                 if isinstance(k, tuple):  # multikey
                     k = '-'.join(k)
                 key = '%s/%s' % (path, quote_plus(k))
@@ -341,14 +329,29 @@ class File(h5py.File):
             if isinstance(obj, Group):
                 self.save_attrs(
                     path, obj.attrs, __pyclass__=cls2dotname(Group))
-        elif isinstance(obj, list) and isinstance(obj[0], numpy.ndarray):
+        elif (isinstance(obj, numpy.ndarray) and obj.shape and
+              len(obj) and isinstance(obj[0], str)):
+            self.create_dataset(path, obj.shape, vstr)[:] = obj
+        elif (isinstance(obj, numpy.ndarray) and not obj.shape and
+              obj.dtype.name.startswith('bytes')):
+            self._set(path, numpy.void(bytes(obj)))
+        elif isinstance(obj, list) and len(obj) and isinstance(
+                obj[0], numpy.ndarray):
             self.save_vlen(path, obj)
+        elif isinstance(obj, bytes):
+            self._set(path, numpy.void(obj))
         else:
-            super().__setitem__(path, obj)
+            self._set(path, obj)
         if pyclass:
             self.flush()  # make sure it is fully saved
             self.save_attrs(path, attrs, __pyclass__=pyclass)
 
+    def _set(self, path, obj):
+        try:
+            super().__setitem__(path, obj)
+        except Exception as exc:
+            raise exc.__class__('Could not set %s=%r' % (path, obj))
+
     def __getitem__(self, path):
         h5obj = super().__getitem__(path)
         h5attrs = h5obj.attrs
@@ -358,8 +361,8 @@ class File(h5py.File):
             if hasattr(h5obj, 'items'):  # is group
                 h5obj = {unquote_plus(k): self['%s/%s' % (path, k)]
                          for k, v in h5obj.items()}
-            elif hasattr(h5obj, 'value'):
-                h5obj = h5obj.value
+            elif hasattr(h5obj, 'shape'):
+                h5obj = h5obj[()]
             if hasattr(obj, '__fromh5__'):
                 obj.__fromh5__(h5obj, h5attrs)
             else:  # Group object
@@ -373,71 +376,12 @@ class File(h5py.File):
         # make the file pickleable
         return {'_id': 0}
 
-    def set_nbytes(self, key, nbytes=None):
-        """
-        Set the `nbytes` attribute on the HDF5 object identified by `key`.
-        """
-        obj = super().__getitem__(key)
-        if nbytes is not None:  # size set from outside
-            obj.attrs['nbytes'] = nbytes
-        else:  # recursively determine the size of the datagroup
-            obj.attrs['nbytes'] = nbytes = ByteCounter.get_nbytes(obj)
-        return nbytes
-
     def getitem(self, name):
         """
         Return a dataset by using h5py.File.__getitem__
         """
         return h5py.File.__getitem__(self, name)
 
-    def save(self, nodedict, root=''):
-        """
-        Save a node dictionary in the .hdf5 file, starting from the root
-        dataset. A common application is to convert XML files into .hdf5
-        files, see the usage in :mod:`openquake.commands.to_hdf5`.
-
-        :param nodedict:
-            a dictionary with keys 'tag', 'attrib', 'text', 'nodes'
-        """
-        setitem = super().__setitem__
-        getitem = super().__getitem__
-        tag = nodedict['tag']
-        text = nodedict.get('text', None)
-        if hasattr(text, 'strip'):
-            text = text.strip()
-        attrib = nodedict.get('attrib', {})
-        path = '/'.join([root, tag])
-        nodes = nodedict.get('nodes', [])
-        if text not in ('', None):  # text=0 is stored
-            try:
-                setitem(path, text)
-            except Exception as exc:
-                sys.stderr.write('%s: %s\n' % (path, exc))
-                raise
-        elif attrib and not nodes:
-            setitem(path, numpy.nan)
-        for subdict in _resolve_duplicates(nodes):
-            self.save(subdict, path)
-        if attrib:
-            dset = getitem(path)
-            for k, v in attrib.items():
-                dset.attrs[k] = maybe_encode(v)
-
-
-def _resolve_duplicates(dicts):
-    # when node dictionaries with duplicated tags are passed (for instance
-    # [{'tag': 'SourceGroup', ...}, {'tag': 'SourceGroup', ...}])
-    # add an incremental number to the tag, preceded by a semicolon:
-    # [{'tag': 'SourceGroup;1', ...}, {'tag': 'SourceGroup;2', ...}])
-    # in this way the dictionaries can be saved in HDF5 format without
-    # name conflicts and by respecting the ordering
-    for tag, grp in itertools.groupby(dicts, operator.itemgetter('tag')):
-        group = list(grp)
-        if len(group) > 1:  # there are duplicate tags
-            for i, dic in enumerate(group, 1):
-                dic['tag'] += ';%d' % i
-    return dicts
-
 
 def array_of_vstr(lst):
     """
@@ -456,21 +400,30 @@ def array_of_vstr(lst):
 class ArrayWrapper(object):
     """
     A pickleable and serializable wrapper over an array, HDF5 dataset or group
+
+    :param array: an array (or the empty tuple)
+    :param attrs: metadata of the array (or dictionary of arrays)
     """
     @classmethod
-    def from_(cls, obj):
+    def from_(cls, obj, extra='value'):
         if isinstance(obj, cls):  # it is already an ArrayWrapper
             return obj
         elif inspect.isgenerator(obj):
             array, attrs = (), dict(obj)
         elif hasattr(obj, '__toh5__'):
             return obj
+        elif hasattr(obj, 'attrs'):  # is a dataset
+            array, attrs = obj[()], dict(obj.attrs)
+            shape_descr = attrs.get('shape_descr', [])
+            for descr in map(decode, shape_descr):
+                attrs[descr] = list(attrs[descr])
         else:  # assume obj is an array
             array, attrs = obj, {}
-        return cls(array, attrs)
+        return cls(array, attrs, (extra,))
 
-    def __init__(self, array, attrs):
+    def __init__(self, array, attrs, extra=('value',)):
         vars(self).update(attrs)
+        self._extra = tuple(extra)
         if len(array):
             self.array = array
 
@@ -493,13 +446,20 @@ class ArrayWrapper(object):
 
     def __toh5__(self):
         arr = getattr(self, 'array', ())
-        return (arr, {k: v for k, v in vars(self).items()
-                      if k != 'array' and not k.startswith('_')})
+        if len(arr):
+            return arr, self.to_dict()
+        return self.to_dict(), {}
 
     def __fromh5__(self, array, attrs):
         self.__init__(array, attrs)
 
     def __repr__(self):
+        if hasattr(self, 'shape_descr'):
+            assert len(self.shape) == len(self.shape_descr), (
+                self.shape_descr, self.shape)
+            lst = ['%s=%d' % (descr, size)
+                   for descr, size in zip(self.shape_descr, self.shape)]
+            return '<%s(%s)>' % (self.__class__.__name__, ', '.join(lst))
         return '<%s%s>' % (self.__class__.__name__, self.shape)
 
     @property
@@ -512,6 +472,16 @@ class ArrayWrapper(object):
         """shape of the underlying array"""
         return self.array.shape if hasattr(self, 'array') else ()
 
+    def toml(self):
+        """
+        :returns: a TOML string representation of the ArrayWrapper
+        """
+        if self.shape:
+            return toml.dumps(self.array)
+        dic = {k: v for k, v in vars(self).items()
+               if not k.startswith('_')}
+        return toml.dumps(dic)
+
     def save(self, path, **extra):
         """
         :param path: an .hdf5 pathname
@@ -532,54 +502,60 @@ class ArrayWrapper(object):
         """
         Convert an ArrayWrapper with shape (D1, ..., DN) and attributes
         T1, ..., TN which are list of tags of lenghts D1, ... DN into
-        a table with rows (tag1, ... tagN, value) of maximum length
-        D1 * ... * DN. Zero values are discarded.
+        a table with rows (tag1, ... tagN, extra1, ... extraM) of maximum
+        length D1 * ... * DN. Zero values are discarded.
 
         >>> from pprint import pprint
-        >>> dic = dict(tagnames=['taxonomy', 'occupancy'],
-        ...            taxonomy=['?', 'RC', 'WOOD'],
-        ...            occupancy=['?', 'RES', 'IND', 'COM'])
+        >>> dic = dict(shape_descr=['taxonomy', 'occupancy'],
+        ...            taxonomy=['RC', 'WOOD'],
+        ...            occupancy=['RES', 'IND', 'COM'])
         >>> arr = numpy.zeros((2, 3))
         >>> arr[0, 0] = 2000
         >>> arr[0, 1] = 5000
         >>> arr[1, 0] = 500
-        >>> pprint(ArrayWrapper(arr, dic).to_table())
-        [['taxonomy', 'occupancy', 'value'],
-         ['RC', 'RES', 2000.0],
-         ['RC', 'IND', 5000.0],
-         ['WOOD', 'RES', 500.0]]
+        >>> aw = ArrayWrapper(arr, dic)
+        >>> pprint(aw.to_table())
+        [('taxonomy', 'occupancy', 'value'),
+         ('RC', 'RES', 2000.0),
+         ('RC', 'IND', 5000.0),
+         ('WOOD', 'RES', 500.0)]
         """
         shape = self.shape
-        # the tagnames are bytestrings so they must be decoded
-        tagnames = decode_array(self.tagnames)
-        if len(shape) == len(tagnames):
-            return [tagnames + ['value']] + self._to_table()
-        elif len(shape) == len(tagnames) + 1:  # there is an extra field
-            tbl = [tagnames + [self.extra[0], 'value']]
-            return tbl + self._to_table(self.extra[1:])
-        else:
-            raise TypeError(
-                'There are %d dimensions but only %d tagnames' %
-                (len(shape), len(tagnames)))
-
-    def _to_table(self, extra=()):
-        tags = []  # tag_idx -> tag_values
-        shape = self.shape
-        if extra:
-            assert shape[-1] == len(extra), (shape[-1], len(extra))
-        tagnames = decode_array(self.tagnames)
-        for i, tagname in enumerate(tagnames):
-            values = getattr(self, tagname)[1:]
-            assert len(values) == shape[i], (len(values), shape[i])
+        tup = len(self._extra) > 1
+        if tup:
+            if shape[-1] != len(self._extra):
+                raise ValueError(
+                    'There are %d extra-fields but %d dimensions in %s' %
+                    (len(self._extra), shape[-1], self))
+        shape_descr = tuple(decode(d) for d in self.shape_descr)
+        fields = shape_descr + self._extra
+        out = []
+        tags = []
+        idxs = []
+        for i, tagname in enumerate(shape_descr):
+            values = getattr(self, tagname)
+            if len(values) != shape[i]:
+                raise ValueError(
+                    'The tag %s with %d values is inconsistent with %s'
+                    % (tagname, len(values), self))
             tags.append(decode_array(values))
-        if extra:
-            tags.append(extra)
-        tbl = []
-        for idx, value in numpy.ndenumerate(self.array):
-            row = [tags[i][j] for i, j in enumerate(idx)] + [value]
-            if value > 0:
-                tbl.append(row)
-        return tbl
+            idxs.append(range(len(values)))
+        for idx, values in zip(itertools.product(*idxs),
+                               itertools.product(*tags)):
+            val = self.array[idx]
+            if tup:
+                if val.sum():
+                    out.append(values + tuple(val))
+            elif val:
+                out.append(values + (val,))
+        return [fields] + out
+
+    def to_dict(self):
+        """
+        Convert the public attributes into a dictionary
+        """
+        return {k: v for k, v in vars(self).items()
+                if k != 'array' and not k.startswith('_')}
 
 
 def decode_array(values):
@@ -641,3 +617,86 @@ def extract(dset, *d_slices):
                     for s in tup)
         array[aidx] = dset[sel]
     return array
+
+
+def parse_comment(comment):
+    """
+    Parse a comment of the form
+    `investigation_time=50.0, imt="PGA", ...`
+    and returns it as pairs of strings:
+
+    >>> parse_comment('''path=['b1'], time=50.0, imt="PGA"''')
+    [('path', ['b1']), ('time', 50.0), ('imt', 'PGA')]
+    """
+    if comment[0] == '"' and comment[-1] == '"':
+        comment = comment[1:-1]
+    dic = toml.loads('{%s}' % comment.replace('""', '"'))
+    return list(dic.items())
+
+
+def build_dt(dtypedict, names):
+    """
+    Build a composite dtype for a list of names and dictionary
+    name -> dtype with a None entry corresponding to the default dtype.
+    """
+    lst = []
+    for name in names:
+        try:
+            dt = dtypedict[name]
+        except KeyError:
+            dt = dtypedict[None]
+        lst.append((name, vstr if dt is str else dt))
+    return numpy.dtype(lst)
+
+
+def _read_csv(fileobj, compositedt):
+    itemsize = [0] * len(compositedt)
+    for i, name in enumerate(compositedt.names):
+        if compositedt[name].kind == 'S':  # limit of the length of byte-fields
+            itemsize[i] = compositedt[name].itemsize
+    rows = []
+    for lineno, row in enumerate(csv.reader(fileobj), 3):
+        cols = []
+        for i, col in enumerate(row):
+            if itemsize[i] and len(col) > itemsize[i]:
+                raise ValueError(
+                    'line %d: %s=%r has length %d > %d' %
+                    (lineno, compositedt.names[i], col, len(col), itemsize[i]))
+            cols.append(col)
+        rows.append(tuple(cols))
+    return numpy.array(rows, compositedt)
+
+
+# NB: it would be nice to use numpy.loadtxt(
+#  f, build_dt(dtypedict, header), delimiter=sep, ndmin=1, comments=None)
+# however numpy does not support quoting, and "foo,bar" would be split :-(
+def read_csv(fname, dtypedict={None: float}, renamedict={}, sep=','):
+    """
+    :param fname: a CSV file with an header and float fields
+    :param dtypedict: a dictionary fieldname -> dtype, None -> default
+    :param renamedict: aliases for the fields to rename
+    :param sep: separator (default comma)
+    :return: a structured array of floats
+    """
+    attrs = {}
+    with open(fname, encoding='utf-8-sig') as f:
+        while True:
+            first = next(f)
+            if first.startswith('#'):
+                attrs = dict(parse_comment(first.strip('#,\n ')))
+                continue
+            break
+        header = first.strip().split(sep)
+        try:
+            arr = _read_csv(f, build_dt(dtypedict, header))
+        except KeyError:
+            raise KeyError('Missing None -> default in dtypedict')
+        except Exception as exc:
+            raise InvalidFile('%s: %s' % (fname, exc))
+    if renamedict:
+        newnames = []
+        for name in arr.dtype.names:
+            new = renamedict.get(name, name)
+            newnames.append(new)
+        arr.dtype.names = newnames
+    return ArrayWrapper(arr, attrs)
