@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2010-2019 GEM Foundation
+# Copyright (C) 2010-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -19,29 +19,39 @@
 The Starmap API
 ====================================
 
-There are several good libraries to manage parallel programming, both
+There are several good libraries to manage parallel programming in Python, both
 in the standard library and in third party packages. Since we are not
 interested in reinventing the wheel, OpenQuake does not provide any new
 parallel library; however, it does offer some glue code so that you
-can use your library of choice. Currently threading, multiprocessing,
+can use over your library of choice. Currently threading, multiprocessing,
 zmq and celery are supported. Moreover,
 :mod:`openquake.baselib.parallel` offers some additional facilities
 that make it easier to parallelize scientific computations,
 i.e. embarrassingly parallel problems.
 
-Typically one wants to apply a callable to a list of arguments - in
-parallel rather then sequentially - and then combine together the
-results. This is known as a `MapReduce` problem. As a simple example,
-we will consider the problem of counting the letters in a text. Here is
-how you can solve the problem in parallel by using
+Typically one wants to apply a callable to a list of arguments in
+parallel, and then combine together the results. This is known as a
+`MapReduce` problem. As a simple example, we will consider the problem
+of counting the letters in a text, by using the following `count`
+function:
+
+.. python::
+
+  def count(word):
+      return collections.Counter(word)
+
+The `collections.Counter` class works sequentially, and can
+solve the problem in parallel by using
 :class:`openquake.baselib.parallel.Starmap`:
 
 >>> arglist = [('hello',), ('world',)]  # list of arguments
->>> res = Starmap(count, arglist).reduce()  # aggregated counts
->>> sorted(res.items())  # counts per letter
+>>> smap = Starmap(count, arglist)  # Starmap instance, nothing started yet
+>>> sorted(smap.reduce().items())  # build the counts per letter
 [('d', 1), ('e', 1), ('h', 1), ('l', 3), ('o', 2), ('r', 1), ('w', 1)]
 
-`Starmap` has a `reduce` method with sensible defaults:
+A `Starmap` object is an iterable: when iterating over it produces
+task results. It also has a `reduce` method similar to `functools.reduce`
+with sensible defaults:
 
 1. the default aggregation function is `add`, so there is no need to specify it
 2. the default accumulator is an empty accumulation dictionary (see
@@ -91,23 +101,42 @@ way your code is future-proof.
 Monitoring
 =============================
 
-The major feature of the Starmap API is the ability to pass to it a
-Monitor instance, an object which is able to measure the time and
-memory occupation in each spawned task. The monitor can keep that
-information in memory or even store it into an .hdf5 file. If you
-do not pass a Monitor instance to the Starmap, internally a do-nothing
-monitor is instantiated. Here is an in-memory monitor:
+A major feature of the Starmap API is the ability to monitor the time spent
+in each task and the memory allocated. Such information is written into an
+HDF5 file that can be provided by the user or autogenerated. To autogenerate
+the file you can use :func:`openquake.baselib.datastore.hdf5new` which
+will create a file named ``calc_XXX.hdf5`` in your $OQ_DATA directory
+(if the environment variable is not set, the engine will use $HOME/oqdata).
+Here is an example of usage:
 
->>> from openquake.baselib.performance import Monitor
->>> mon = Monitor('count')
+>>> from openquake.baselib.datastore import hdf5new
+>>> h5 = hdf5new()
+>>> smap = Starmap(count, [['hello'], ['world']], h5=h5)
+>>> print(sorted(smap.reduce().items()))
+[('d', 1), ('e', 1), ('h', 1), ('l', 3), ('o', 2), ('r', 1), ('w', 1)]
 
-Here is a monitoring writing on an .hdf5 file:
+After the calculation, or even while the calculation is running, you can
+open the calculation file for reading and extract the performance information
+for it. The engine provides a command to do that, `oq show performance`,
+but you can also get it manually, with a call to
+`openquake.baselib.performance.performance_view(h5)` which will return
+the performance information as a numpy array:
 
->>> from openquake.baselib import hdf5
->>> mon = Monitor('count', hdf5.File.temporary())
+>>> from openquake.baselib.performance import performance_view
+>>> performance_view(h5).dtype.names
+('operation', 'time_sec', 'memory_mb', 'counts')
+>>> h5.close()
 
-The engine provides a command `oq show performance` to print the performance
-information stored in the HDF5 datastore in a nice way.
+The four columns are as follows:
+
+operation:
+  the name of the function running in parallel (in this case 'count')
+time_sec:
+  the cumulative time in second spent running the function
+memory_mb:
+  the maximum allocated memory per core
+counts:
+  the number of times the function was called (in this case 2)
 
 The Starmap.apply API
 ====================================
@@ -120,7 +149,7 @@ letter counting example discussed before, `Starmap.apply` could
 be used as follows:
 
 >>> text = 'helloworld'  # sequence of characters
->>> res3 = Starmap.apply(count, (text, mon)).reduce()
+>>> res3 = Starmap.apply(count, (text,)).reduce()
 >>> assert res3 == res
 
 The API of `Starmap.apply` is designed to extend the one of `apply`,
@@ -149,8 +178,11 @@ having finished all the short tasks, but you have to wait for days for
 the single core processing the slow task). The OpenQuake engine does
 a great deal of work trying to split slow sources in more manageable
 fast sources.
+
 """
 import os
+import re
+import ast
 import sys
 import time
 import socket
@@ -159,9 +191,9 @@ import pickle
 import inspect
 import logging
 import operator
-import itertools
 import traceback
 import collections
+from unittest import mock
 import multiprocessing.dummy
 import psutil
 import numpy
@@ -171,30 +203,24 @@ except ImportError:
     def setproctitle(title):
         "Do nothing"
 
-from openquake.baselib import hdf5, config
+from openquake.baselib import config, hdf5, workerpool, __version__
 from openquake.baselib.zeromq import zmq, Socket
-from openquake.baselib.performance import Monitor, memory_rss, perf_dt
-
+from openquake.baselib.performance import (
+    Monitor, memory_rss, init_performance)
 from openquake.baselib.general import (
-    split_in_blocks, block_splitter, AccumDict, humansize, CallableDict)
-
-cpu_count = multiprocessing.cpu_count()
-GB = 1024 ** 3
-OQ_DISTRIBUTE = os.environ.get('OQ_DISTRIBUTE', 'processpool').lower()
-if OQ_DISTRIBUTE == 'futures':  # legacy name
-    print('Warning: OQ_DISTRIBUTE=futures is deprecated', file=sys.stderr)
-    OQ_DISTRIBUTE = os.environ['OQ_DISTRIBUTE'] = 'processpool'
-if OQ_DISTRIBUTE not in ('no', 'processpool', 'threadpool', 'celery', 'zmq',
-                         'dask'):
-    raise ValueError('Invalid oq_distribute=%s' % OQ_DISTRIBUTE)
-
-# data type for storing the performance information
-task_info_dt = numpy.dtype(
-    [('taskno', numpy.uint32), ('weight', numpy.float32),
-     ('duration', numpy.float32), ('received', numpy.int64),
-     ('mem_gb', numpy.float32)])
+    split_in_blocks, block_splitter, AccumDict, humansize, CallableDict,
+    gettemp)
 
+sys.setrecursionlimit(1200)  # raised a bit to make pickle happier
+# see https://github.com/gem/oq-engine/issues/5230
 submit = CallableDict()
+GB = 1024 ** 3
+# use only the "visible" cores, not the total system cores
+# if the underlying OS supports it (macOS does not)
+try:
+    CT = len(psutil.Process().cpu_affinity()) * 2
+except AttributeError:
+    CT = psutil.cpu_count() * 2
 
 
 @submit.add('no')
@@ -208,7 +234,10 @@ def processpool_submit(self, func, args, monitor):
         safely_call, (func, args, self.task_no, monitor))
 
 
-threadpool_submit = submit.add('threadpool')(processpool_submit)
+@submit.add('threadpool')
+def threadpool_submit(self, func, args, monitor):
+    return self.pool.apply_async(
+        safely_call, (func, args, self.task_no, monitor))
 
 
 @submit.add('celery')
@@ -219,9 +248,10 @@ def celery_submit(self, func, args, monitor):
 @submit.add('zmq')
 def zmq_submit(self, func, args, monitor):
     if not hasattr(self, 'sender'):
-        task_in_url = 'tcp://%s:%s' % (config.dbserver.host,
-                                       config.zworkers.task_in_port)
-        self.sender = Socket(task_in_url, zmq.PUSH, 'connect').__enter__()
+        port = int(config.zworkers.ctrl_port) + 2
+        task_input_url = 'tcp://127.0.0.1:%d' % port
+        self.sender = Socket(
+            task_input_url, zmq.PUSH, 'connect').__enter__()
     return self.sender.send((func, args, self.task_no, monitor))
 
 
@@ -234,7 +264,11 @@ def oq_distribute(task=None):
     """
     :returns: the value of OQ_DISTRIBUTE or 'processpool'
     """
-    return os.environ.get('OQ_DISTRIBUTE', 'processpool').lower()
+    dist = os.environ.get('OQ_DISTRIBUTE', 'processpool').lower()
+    if dist not in ('no', 'processpool', 'threadpool', 'celery', 'zmq',
+                    'dask'):
+        raise ValueError('Invalid oq_distribute=%s' % dist)
+    return dist
 
 
 class Pickled(object):
@@ -313,6 +347,17 @@ def pickle_sequence(objects):
     return out
 
 
+class FakePickle:
+    def __init__(self, sentbytes):
+        self.sentbytes = sentbytes
+
+    def unpickle(self):
+        pass
+
+    def __len__(self):
+        return self.sentbytes
+
+
 class Result(object):
     """
     :param val: value to return or exception instance
@@ -320,19 +365,25 @@ class Result(object):
     :param tb_str: traceback string (empty if there was no exception)
     :param msg: message string (default empty)
     """
-    func_args = ()
+    func = None
 
-    def __init__(self, val, mon, tb_str='', msg='', count=0):
+    def __init__(self, val, mon, tb_str='', msg=''):
         if isinstance(val, dict):
-            # store the size in bytes of the content
+            self.pik = Pickled(val)
             self.nbytes = {k: len(Pickled(v)) for k, v in val.items()}
         elif isinstance(val, tuple) and callable(val[0]):
-            self.func_args = val
-        self.pik = Pickled(val)
+            self.func = val[0]
+            self.pik = pickle_sequence(val[1:])
+            self.nbytes = {'args': sum(len(p) for p in self.pik)}
+        elif msg == 'TASK_ENDED':
+            self.pik = Pickled(None)
+            self.nbytes = {}
+        else:
+            self.pik = Pickled(val)
+            self.nbytes = {'tot': len(self.pik)}
         self.mon = mon
         self.tb_str = tb_str
         self.msg = msg
-        self.count = count
 
     def get(self):
         """
@@ -348,22 +399,32 @@ class Result(object):
                 raise etype(msg)
         return val
 
+    def __repr__(self):
+        nbytes = ['%s: %s' % (k, humansize(v)) for k, v in self.nbytes.items()]
+        return '<%s %s>' % (self.__class__.__name__, ' '.join(nbytes))
+
     @classmethod
-    def new(cls, func, args, mon, count=0):
+    def new(cls, func, args, mon, sentbytes=0):
         """
         :returns: a new Result instance
         """
         try:
+            if mon.version != __version__:
+                raise RuntimeError(
+                    'The master is at version %s while the worker %s is at '
+                    'version %s' % (mon.version, socket.gethostname(),
+                                    __version__))
             with mon:
                 val = func(*args)
         except StopIteration:
+            mon.counts -= 1  # StopIteration does not count
             res = Result(None, mon, msg='TASK_ENDED')
+            res.pik = FakePickle(sentbytes)
         except Exception:
             _etype, exc, tb = sys.exc_info()
-            res = Result(exc, mon, ''.join(traceback.format_tb(tb)),
-                         count=count)
+            res = Result(exc, mon, ''.join(traceback.format_tb(tb)))
         else:
-            res = Result(val, mon, count=count)
+            res = Result(val, mon)
         return res
 
 
@@ -384,6 +445,7 @@ def check_mem_usage(soft_percent=None, hard_percent=None):
 
 
 dummy_mon = Monitor()
+dummy_mon.version = __version__
 dummy_mon.backurl = None
 
 
@@ -407,40 +469,37 @@ def safely_call(func, args, task_no=0, mon=dummy_mon):
     if mon is dummy_mon:  # in the DbServer
         assert not isgenfunc, func
         return Result.new(func, args, mon)
-
     mon = mon.new(operation='total ' + func.__name__, measuremem=True)
     mon.weight = getattr(args[0], 'weight', 1.)  # used in task_info
     mon.task_no = task_no
     if mon.inject:
         args += (mon,)
+    sentbytes = 0
     with Socket(mon.backurl, zmq.PUSH, 'connect') as zsocket:
         msg = check_mem_usage()  # warn if too much memory is used
         if msg:
             zsocket.send(Result(None, mon, msg=msg))
         if inspect.isgeneratorfunction(func):
-            gfunc = func
+            it = func(*args)
         else:
-            def gfunc(*args):
+            def gen(*args):
                 yield func(*args)
-        gobj = gfunc(*args)
-        for count in itertools.count():
-            res = Result.new(next, (gobj,), mon, count=count)
+            it = gen(*args)
+        while True:
             # StopIteration -> TASK_ENDED
+            res = Result.new(next, (it,), mon, sentbytes)
             try:
                 zsocket.send(res)
             except Exception:  # like OverflowError
                 _etype, exc, tb = sys.exc_info()
-                err = Result(exc, mon, ''.join(traceback.format_tb(tb)),
-                             count=count)
+                err = Result(exc, mon, ''.join(traceback.format_tb(tb)))
                 zsocket.send(err)
-            mon.duration = 0
-            mon.counts = 0
-            mon.children.clear()
+            sentbytes += len(res.pik)
             if res.msg == 'TASK_ENDED':
                 break
 
 
-if OQ_DISTRIBUTE.startswith('celery'):
+if oq_distribute().startswith('celery'):
     from celery import Celery
     from celery.task import task
 
@@ -448,7 +507,7 @@ if OQ_DISTRIBUTE.startswith('celery'):
     app.config_from_object('openquake.engine.celeryconfig')
     safetask = task(safely_call, queue='celery')  # has to be global
 
-elif OQ_DISTRIBUTE == 'dask':
+elif oq_distribute() == 'dask':
     from dask.distributed import Client
 
 
@@ -461,30 +520,24 @@ class IterResult(object):
     :param done_total:
         a function returning the number of done tasks and the total
     :param sent:
-        the number of bytes sent (0 if OQ_DISTRIBUTE=no)
+        a nested dictionary name -> {argname: number of bytes sent}
     :param progress:
         a logging function for the progress report
-    :param hdf5:
-        if given, hdf5 file where to append the performance information
-    """
-    def __init__(self, iresults, taskname, argnames, sent, hdf5=None):
+    :param hdf5path:
+        a path where to store persistently the performance info
+     """
+    def __init__(self, iresults, taskname, argnames, sent, h5):
         self.iresults = iresults
         self.name = taskname
         self.argnames = ' '.join(argnames)
         self.sent = sent
-        self.hdf5 = hdf5
-        self.received = []
+        self.h5 = h5
 
-    def __iter__(self):
-        if self.iresults == ():
-            return ()
-        t0 = time.time()
-        self.received = []
+    def _iter(self):
         first_time = True
-        nbytes = AccumDict()
-        names = {self.name}
         for result in self.iresults:
-            msg = check_mem_usage()  # log a warning if too much memory is used
+            msg = check_mem_usage()
+            # log a warning if too much memory is used
             if msg and first_time:
                 logging.warning(msg)
                 first_time = False  # warn only once
@@ -493,13 +546,10 @@ class IterResult(object):
                 raise result
             elif isinstance(result, Result):
                 val = result.get()
-                names.add(result.mon.operation[6:])
-                self.received.append(len(result.pik))
-                if hasattr(result, 'nbytes'):
-                    nbytes += result.nbytes
+                self.nbytes += result.nbytes
             else:  # this should never happen
                 raise ValueError(result)
-            if OQ_DISTRIBUTE == 'processpool' and sys.platform != 'darwin':
+            if sys.platform != 'darwin':
                 # it normally works on macOS, but not in notebooks calling
                 # notebooks, which is the case relevant for Marco Pagani
                 mem_gb = (memory_rss(os.getpid()) + sum(
@@ -507,19 +557,31 @@ class IterResult(object):
             else:
                 # measure only the memory used by the main process
                 mem_gb = memory_rss(os.getpid()) / GB
-            if not result.func_args:  # not subtask
+            if result.msg == 'TASK_ENDED':
+                task_sent = ast.literal_eval(self.h5['task_sent'][()])
+                task_sent.update(self.sent)
+                del self.h5['task_sent']
+                self.h5['task_sent'] = str(task_sent)
+                name = result.mon.operation[6:]  # strip 'total '
+                result.mon.save_task_info(self.h5, result, name, mem_gb)
+                result.mon.flush(self.h5)
+                self.h5.flush()
+            elif not result.func:  # real output
                 yield val
-            save_task_info(self, result, mem_gb)
-        if self.received:
-            tot = sum(self.received)
-            max_per_output = max(self.received)
-            logging.info(
-                'Received %s from %d %s outputs in %d seconds, biggest '
-                'output=%s', humansize(tot), len(self.received),
-                '|'.join(names), time.time() - t0, humansize(max_per_output))
-            if nbytes:
-                logging.info('Received %s',
-                             {k: humansize(v) for k, v in nbytes.items()})
+
+    def __iter__(self):
+        if self.iresults == ():
+            return ()
+        t0 = time.time()
+        self.nbytes = AccumDict()
+        try:
+            yield from self._iter()
+        finally:
+            items = sorted(self.nbytes.items(), key=operator.itemgetter(1))
+            nb = {k: humansize(v) for k, v in reversed(items)}
+            msg = nb if len(nb) < 10 else {
+                'tot': humansize(sum(self.nbytes.values()))}
+            logging.info('Received %s in %d seconds', msg, time.time() - t0)
 
     def reduce(self, agg=operator.add, acc=None):
         if acc is None:
@@ -534,10 +596,8 @@ class IterResult(object):
         Sum the data transfer information of a set of results
         """
         res = object.__new__(cls)
-        res.received = []
         res.sent = 0
         for iresult in iresults:
-            res.received.extend(iresult.received)
             res.sent += iresult.sent
             name = iresult.name.split('#', 1)[0]
             if hasattr(res, 'name'):
@@ -547,23 +607,6 @@ class IterResult(object):
         return res
 
 
-def save_task_info(self, res, mem_gb=0):
-    """
-    :param self: an object with attributes .hdf5, .argnames, .sent
-    :parent res: a :class:`Result` object
-    :param mem_gb: memory consumption at the saving time (optional)
-    """
-    mon = res.mon
-    name = mon.operation[6:]  # strip 'total '
-    if self.hdf5:
-        mon.hdf5 = self.hdf5  # needed for the flush below
-        t = (mon.task_no, mon.weight, mon.duration, len(res.pik), mem_gb)
-        data = numpy.array([t], task_info_dt)
-        hdf5.extend3(self.hdf5.filename, 'task_info/' + name, data,
-                     argnames=self.argnames, sent=self.sent)
-    mon.flush()
-
-
 def init_workers():
     """Waiting function, used to wake up the process pool"""
     setproctitle('oq-worker')
@@ -578,15 +621,27 @@ def init_workers():
         prctl.set_pdeathsig(signal.SIGKILL)
 
 
+def getargnames(task_func):
+    # a task can be a function, a class or an instance with a __call__
+    if inspect.isfunction(task_func):
+        return inspect.getfullargspec(task_func).args
+    elif inspect.isclass(task_func):
+        return inspect.getfullargspec(task_func.__init__).args[1:]
+    else:  # instance with a __call__ method
+        return inspect.getfullargspec(task_func.__call__).args[1:]
+
+
 class Starmap(object):
-    calc_id = None
-    hdf5 = None
     pids = ()
     running_tasks = []  # currently running tasks
+    # use only the "visible" cores, not the total system cores
+    # if the underlying OS supports it (macOS does not)
+    num_cores = None
 
     @classmethod
-    def init(cls, poolsize=None, distribute=OQ_DISTRIBUTE):
-        if distribute == 'processpool' and not hasattr(cls, 'pool'):
+    def init(cls, poolsize=None, distribute=None):
+        cls.distribute = distribute or oq_distribute()
+        if cls.distribute == 'processpool' and not hasattr(cls, 'pool'):
             # unregister custom handlers before starting the processpool
             term_handler = signal.signal(signal.SIGTERM, signal.SIG_DFL)
             int_handler = signal.signal(signal.SIGINT, signal.SIG_IGN)
@@ -600,9 +655,9 @@ class Starmap(object):
             signal.signal(signal.SIGTERM, term_handler)
             signal.signal(signal.SIGINT, int_handler)
             cls.pids = [proc.pid for proc in cls.pool._pool]
-        elif distribute == 'threadpool' and not hasattr(cls, 'pool'):
+        elif cls.distribute == 'threadpool' and not hasattr(cls, 'pool'):
             cls.pool = multiprocessing.dummy.Pool(poolsize)
-        elif distribute == 'dask':
+        elif cls.distribute == 'dask':
             cls.dask_client = Client(config.distribution.dask_scheduler)
 
     @classmethod
@@ -619,10 +674,11 @@ class Starmap(object):
             del cls.dask_client
 
     @classmethod
-    def apply(cls, task, args, concurrent_tasks=cpu_count * 3,
+    def apply(cls, task, args, concurrent_tasks=None,
               maxweight=None, weight=lambda item: 1,
               key=lambda item: 'Unspecified',
-              distribute=None, progress=logging.info):
+              distribute=None, progress=logging.info, h5=None,
+              num_cores=None):
         r"""
         Apply a task to a tuple of the form (sequence, \*other_args)
         by first splitting the sequence in chunks, according to the weight
@@ -637,76 +693,82 @@ class Starmap(object):
         :param key: function to extract the kind of an item in arg0
         :param distribute: if not given, inferred from OQ_DISTRIBUTE
         :param progress: logging function to use (default logging.info)
+        :param h5: an open hdf5.File where to store the performance info
+        :param num_cores: the number of available cores
         :returns: an :class:`IterResult` object
         """
-        arg0 = args[0]  # this is assumed to be a sequence
-        mon = args[-1]
-        args = args[1:-1]
+        arg0, *args = args
         if maxweight:  # block_splitter is lazy
-            task_args = ((blk,) + args for blk in block_splitter(
+            taskargs = ([blk] + args for blk in block_splitter(
                 arg0, maxweight, weight, key))
         else:  # split_in_blocks is eager
-            task_args = [(blk,) + args for blk in split_in_blocks(
+            if concurrent_tasks is None:
+                concurrent_tasks = CT
+            taskargs = [[blk] + args for blk in split_in_blocks(
                 arg0, concurrent_tasks or 1, weight, key)]
-        return cls(task, task_args, mon, distribute, progress).submit_all()
+        return cls(
+            task, taskargs, distribute, progress, h5, num_cores
+        ).submit_all()
 
-    def __init__(self, task_func, task_args=(), monitor=None, distribute=None,
-                 progress=logging.info):
-        self.__class__.init(distribute=distribute or OQ_DISTRIBUTE)
+    def __init__(self, task_func, task_args=(), distribute=None,
+                 progress=logging.info, h5=None, num_cores=None):
+        self.__class__.init(distribute=distribute)
         self.task_func = task_func
-        self.monitor = monitor or Monitor(task_func.__name__)
-        self.calc_id = getattr(self.monitor, 'calc_id', None)
+        if h5:
+            match = re.search(r'(\d+)', os.path.basename(h5.filename))
+            self.calc_id = int(match.group(1))
+        else:
+            self.calc_id = None
+            h5 = hdf5.File(gettemp(suffix='.hdf5'), 'w')
+            init_performance(h5)
+        self.monitor = Monitor(task_func.__name__)
+        self.monitor.calc_id = self.calc_id
         self.name = self.monitor.operation or task_func.__name__
         self.task_args = task_args
-        self.distribute = distribute or oq_distribute(task_func)
         self.progress = progress
+        self.h5 = h5
+        self.num_cores = num_cores
+        self.task_queue = []
         try:
             self.num_tasks = len(self.task_args)
         except TypeError:  # generators have no len
             self.num_tasks = None
-        # a task can be a function, a class or an instance with a __call__
-        if inspect.isfunction(task_func):
-            self.argnames = inspect.getfullargspec(task_func).args
-        elif inspect.isclass(task_func):
-            self.argnames = inspect.getfullargspec(task_func.__init__).args[1:]
-        else:  # instance with a __call__ method
-            self.argnames = inspect.getfullargspec(task_func.__call__).args[1:]
+        self.argnames = getargnames(task_func)
+        self.sent = AccumDict(accum=AccumDict())  # fname -> argname -> nbytes
         self.monitor.inject = (self.argnames[-1].startswith('mon') or
                                self.argnames[-1].endswith('mon'))
         self.receiver = 'tcp://%s:%s' % (
             config.dbserver.listen, config.dbserver.receiver_ports)
-        self.sent = numpy.zeros(len(self.argnames) - 1)
         self.monitor.backurl = None  # overridden later
         self.tasks = []  # populated by .submit
-        h5 = self.monitor.hdf5
-        task_info = 'task_info/' + self.name
-        if h5 and task_info not in h5:  # first time
-            # task_info and performance_data should be generated in advance
-            hdf5.create(h5, task_info, task_info_dt)
-        if h5 and 'performance_data' not in h5:
-            hdf5.create(h5, 'performance_data', perf_dt)
-
-    @property
-    def hdf5(self):
-        return self.monitor.hdf5
+        self.task_no = 0
+        if self.distribute == 'zmq':  # add a check
+            err = workerpool.check_status()
+            if err:
+                raise RuntimeError(err)
 
     def log_percent(self):
         """
         Log the progress of the computation in percentage
         """
-        done = self.total - self.todo
-        percent = int(float(done) / self.total * 100)
+        submitted = len(self.tasks)
+        queued = len(self.task_queue)
+        total = submitted + queued
+        done = submitted - self.todo
+        percent = int(float(done) / total * 100)
+        fname = self.task_func.__name__
         if not hasattr(self, 'prev_percent'):  # first time
             self.prev_percent = 0
-            self.progress('Sent %s of data in %d %s task(s)',
-                          humansize(self.sent.sum()), self.total, self.name)
+            nbytes = sum(self.sent[fname].values())
+            self.progress('%s %s sent, %d submitted, %d queued',
+                          self.name, humansize(nbytes), submitted, queued)
         elif percent > self.prev_percent:
-            self.progress('%s %3d%% [of %d tasks]',
-                          self.name, percent, len(self.tasks))
+            self.progress('%s %3d%% [%d submitted, %d queued]',
+                          self.name, percent, submitted, queued)
             self.prev_percent = percent
         return done
 
-    def submit(self, *args, func=None, monitor=None):
+    def submit(self, args, func=None, monitor=None):
         """
         Submit the given arguments to the underlying task
         """
@@ -717,27 +779,38 @@ class Starmap(object):
             self.socket = Socket(self.receiver, zmq.PULL, 'bind').__enter__()
             monitor.backurl = 'tcp://%s:%s' % (
                 config.dbserver.host, self.socket.port)
-        assert not isinstance(args[-1], Monitor)  # sanity check
-        dist = 'no' if self.num_tasks == 1 else self.distribute
+            monitor.version = __version__
+        OQ_TASK_NO = os.environ.get('OQ_TASK_NO')
+        if OQ_TASK_NO is not None and self.task_no != int(OQ_TASK_NO):
+            self.task_no += 1
+            return
+        dist = 'no' if self.num_tasks == 1 or OQ_TASK_NO else self.distribute
         if dist != 'no':
-            args = pickle_sequence(args)
-            self.sent += numpy.array([len(p) for p in args])
+            pickled = isinstance(args[0], Pickled)
+            if not pickled:
+                assert not isinstance(args[-1], Monitor)  # sanity check
+                args = pickle_sequence(args)
+            if func is None:
+                fname = self.task_func.__name__
+                argnames = self.argnames[:-1]
+            else:
+                fname = func.__name__
+                argnames = getargnames(func)[:-1]
+            self.sent[fname] += {a: len(p) for a, p in zip(argnames, args)}
         res = submit[dist](self, func, args, monitor)
+        self.task_no += 1
         self.tasks.append(res)
 
-    @property
-    def task_no(self):
-        """
-        :returns: number of the last submitted task, starting from 0
-        """
-        return len(self.tasks)
-
     def submit_all(self):
         """
         :returns: an IterResult object
         """
-        for args in self.task_args:
-            self.submit(*args)
+        if self.num_tasks is None:  # loop on the iterator
+            for args in self.task_args:
+                self.submit(args)
+        else:  # build a task queue in advance
+            self.task_queue = [(self.task_func, args)
+                               for args in self.task_args]
         return self.get_results()
 
     def get_results(self):
@@ -745,7 +818,7 @@ class Starmap(object):
         :returns: an :class:`IterResult` instance
         """
         return IterResult(self._loop(), self.name, self.argnames,
-                          self.sent, self.monitor.hdf5)
+                          self.sent, self.h5)
 
     def reduce(self, agg=operator.add, acc=None):
         """
@@ -756,29 +829,45 @@ class Starmap(object):
     def __iter__(self):
         return iter(self.submit_all())
 
+    def _submit_many(self, howmany):
+        for _ in range(howmany):
+            if self.task_queue:
+                # remove in LIFO order
+                func, args = self.task_queue[0]
+                del self.task_queue[0]
+                self.submit(args, func=func)
+                self.todo += 1
+
     def _loop(self):
+        num_cores = self.num_cores or CT // 2
+        if self.task_queue:
+            first_args = self.task_queue[:num_cores]
+            self.task_queue[:] = self.task_queue[num_cores:]
+            for func, args in first_args:
+                self.submit(args, func=func)
         if not hasattr(self, 'socket'):  # no submit was ever made
             return ()
-        if hasattr(self, 'sender'):
-            self.sender.__exit__(None, None, None)
+
         isocket = iter(self.socket)
-        self.total = self.todo = len(self.tasks)
+        self.todo = len(self.tasks)
         while self.todo:
+            self.log_percent()
             res = next(isocket)
-            if self.calc_id and self.calc_id != res.mon.calc_id:
+            if self.calc_id != res.mon.calc_id:
                 logging.warning('Discarding a result from job %s, since this '
                                 'is job %d', res.mon.calc_id, self.calc_id)
-                continue
             elif res.msg == 'TASK_ENDED':
-                self.log_percent()
                 self.todo -= 1
-            elif res.msg:
-                logging.warning(res.msg)
-            elif res.func_args:  # resubmit subtask
-                func, *args = res.func_args
-                self.submit(*args, func=func, monitor=res.mon)
+                self._submit_many(1)
+                logging.debug('%d tasks todo, %d in queue',
+                              self.todo, len(self.task_queue))
                 yield res
-                self.todo += 1
+            elif res.func:  # add subtask
+                self.task_queue.append((res.func, res.pik))
+                if self.num_cores is None:
+                    self._submit_many(1)  # oversubmit
+                elif self.todo < self.num_cores:
+                    self._submit_many(self.num_cores - self.todo)
             else:
                 yield res
         self.log_percent()
@@ -786,18 +875,48 @@ class Starmap(object):
         self.tasks.clear()
 
 
-def sequential_apply(task, args, concurrent_tasks=cpu_count * 3,
-                     weight=lambda item: 1, key=lambda item: 'Unspecified'):
+def sequential_apply(task, args, concurrent_tasks=CT,
+                     maxweight=None, weight=lambda item: 1,
+                     key=lambda item: 'Unspecified',
+                     progress=logging.info):
     """
     Apply sequentially task to args by splitting args[0] in blocks
     """
-    chunks = split_in_blocks(args[0], concurrent_tasks or 1, weight, key)
-    task_args = [(ch,) + args[1:] for ch in chunks]
-    return itertools.starmap(task, task_args)
+    with mock.patch.dict('os.environ', {'OQ_DISTRIBUTE': 'no'}):
+        return Starmap.apply(task, args, concurrent_tasks, maxweight, weight,
+                             key, progress=progress)
 
 
-def count(word, mon):
+def count(word):
     """
     Used as example in the documentation
     """
     return collections.Counter(word)
+
+
+def split_task(func, *args, duration=1000,
+               weight=operator.attrgetter('weight')):
+    """
+    :param func: a task function with a monitor as last argument
+    :param args: arguments of the task function
+    :param duration: split the task if it exceeds the duration
+    :param weight: weight function for the elements in args[0]
+    :yields: a partial result, 0 or more task objects, 0 or 1 partial result
+    """
+    elements = numpy.array(sorted(args[0], key=weight, reverse=True))
+    n = len(elements)
+    # print('task_no=%d, num_elements=%d' % (args[-1].task_no, n))
+    assert n > 0, 'Passed an empty sequence!'
+    if n == 1:
+        yield func(*args)
+        return
+    first, *other = elements
+    first_weight = weight(first)
+    t0 = time.time()
+    res = func(*([first],) + args[1:])
+    dt = (time.time() - t0) / first_weight  # time per unit of weight
+    yield res
+    blocks = list(block_splitter(other, duration, lambda el: weight(el) * dt))
+    for block in blocks[:-1]:
+        yield (func, block) + args[1:-1]
+    yield func(*(blocks[-1],) + args[1:])
