@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2019 GEM Foundation
+# Copyright (C) 2014-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -23,13 +23,13 @@ from openquake.hazardlib import stats
 from openquake.calculators import base, classical_risk
 
 
-def classical_damage(riskinputs, riskmodel, param, monitor):
+def classical_damage(riskinputs, crmodel, param, monitor):
     """
     Core function for a classical damage computation.
 
     :param riskinputs:
         :class:`openquake.risklib.riskinput.RiskInput` objects
-    :param riskmodel:
+    :param crmodel:
         a :class:`openquake.risklib.riskinput.CompositeRiskModel` instance
     :param param:
         dictionary of extra parameters
@@ -40,8 +40,8 @@ def classical_damage(riskinputs, riskmodel, param, monitor):
     """
     result = AccumDict(accum=AccumDict())
     for ri in riskinputs:
-        for out in riskmodel.gen_outputs(ri, monitor):
-            for l, loss_type in enumerate(riskmodel.loss_types):
+        for out in ri.gen_outputs(crmodel, monitor):
+            for l, loss_type in enumerate(crmodel.loss_types):
                 ordinals = ri.assets['ordinal']
                 result[l, out.rlzi] += dict(zip(ordinals, out[loss_type]))
     return result
@@ -63,7 +63,7 @@ class ClassicalDamageCalculator(classical_risk.ClassicalRiskCalculator):
             a dictionary (l, r) -> asset_ordinal -> fractions per damage state
         """
         damages_dt = numpy.dtype([(ds, numpy.float32)
-                                  for ds in self.riskmodel.damage_states])
+                                  for ds in self.crmodel.damage_states])
         damages = numpy.zeros((self.A, self.R, self.L), damages_dt)
         for l, r in result:
             for aid, fractions in result[l, r].items():
