@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2019 GEM Foundation
+# Copyright (C) 2015-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -18,57 +18,38 @@
 
 import os.path
 import logging
-import collections
 import operator
 import numpy
 
-from openquake.baselib import hdf5, datastore
-from openquake.baselib.python3compat import zip
-from openquake.baselib.general import AccumDict, cached_property, get_indices
+from openquake.baselib import hdf5
+from openquake.baselib.general import AccumDict, get_indices
 from openquake.hazardlib.probability_map import ProbabilityMap
 from openquake.hazardlib.stats import compute_pmap_stats
 from openquake.hazardlib.calc.stochastic import sample_ruptures
+from openquake.hazardlib.calc.filters import nofilter
+from openquake.hazardlib import InvalidFile
 from openquake.hazardlib.source import rupture
 from openquake.risklib.riskinput import str2rsi
 from openquake.baselib import parallel
 from openquake.commonlib import calc, util, logs
 from openquake.calculators import base, extract
 from openquake.calculators.getters import (
-    GmfGetter, RuptureGetter, gen_rupture_getters)
+    GmfGetter, RuptureGetter, gen_rgetters, gen_rupture_getters,
+    sig_eps_dt, time_dt)
 from openquake.calculators.classical import ClassicalCalculator
 from openquake.engine import engine
 
 U8 = numpy.uint8
 U16 = numpy.uint16
 U32 = numpy.uint32
-U64 = numpy.uint64
 F32 = numpy.float32
 F64 = numpy.float64
-TWO32 = U64(2 ** 32)
-rlzs_by_grp_dt = numpy.dtype(
-    [('grp_id', U16), ('gsim_id', U16), ('rlzs', hdf5.vuint16)])
-by_grp = operator.attrgetter('src_group_id')
-
-
-def store_rlzs_by_grp(dstore):
-    """
-    Save in the datastore a composite array with fields (grp_id, gsim_id, rlzs)
-    """
-    lst = []
-    assoc = dstore['csm_info'].get_rlzs_assoc()
-    for grp, arr in assoc.by_grp().items():
-        for gsim_id, rlzs in enumerate(arr):
-            lst.append((int(grp[4:]), gsim_id, rlzs))
-    dstore['csm_info/rlzs_by_grp'] = numpy.array(lst, rlzs_by_grp_dt)
+TWO32 = numpy.float64(2 ** 32)
+by_grp = operator.attrgetter('grp_id')
 
 
 # ######################## GMF calculator ############################ #
 
-def update_nbytes(dstore, key, array):
-    nbytes = dstore.get_attr(key, 'nbytes', 0)
-    dstore.set_attrs(key, nbytes=nbytes + array.nbytes)
-
-
 def get_mean_curves(dstore):
     """
     Extract the mean hazard curves from the datastore, as a composite
@@ -83,13 +64,12 @@ def compute_gmfs(rupgetter, srcfilter, param, monitor):
     """
     Compute GMFs and optionally hazard curves
     """
-    getter = GmfGetter(rupgetter, srcfilter, param['oqparam'])
-    with monitor('getting ruptures'):
-        getter.init()
-    return getter.compute_gmfs_curves(monitor)
+    oq = param['oqparam']
+    getter = GmfGetter(rupgetter, srcfilter, oq, param['amplifier'])
+    return getter.compute_gmfs_curves(param.get('rlz_by_event'), monitor)
 
 
-@base.calculators.add('event_based')
+@base.calculators.add('event_based', 'ucerf_hazard')
 class EventBasedCalculator(base.HazardCalculator):
     """
     Event based PSHA calculator generating the ground motion fields and
@@ -98,29 +78,18 @@ class EventBasedCalculator(base.HazardCalculator):
     """
     core_task = compute_gmfs
     is_stochastic = True
-    accept_precalc = ['event_based', 'event_based_risk', 'ucerf_hazard']
-    build_ruptures = sample_ruptures
-
-    @cached_property
-    def csm_info(self):
-        """
-        :returns: a cached CompositionInfo object
-        """
-        try:
-            return self.csm.info
-        except AttributeError:
-            return self.datastore.parent['csm_info']
+    accept_precalc = ['event_based', 'ebrisk', 'event_based_risk']
 
     def init(self):
         if hasattr(self, 'csm'):
             self.check_floating_spinning()
-        self.rupser = calc.RuptureSerializer(self.datastore)
+        if not self.datastore.parent:
+            self.rupser = calc.RuptureSerializer(self.datastore)
 
-    def init_logic_tree(self, csm_info):
-        self.trt_by_grp = csm_info.grp_by("trt")
-        self.rlzs_assoc = csm_info.get_rlzs_assoc()
-        self.rlzs_by_gsim_grp = csm_info.get_rlzs_by_gsim_grp()
-        self.samples_by_grp = csm_info.get_samples_by_grp()
+    def init_logic_tree(self, full_lt):
+        self.trt_by_grp = full_lt.trt_by_grp
+        self.rlzs_by_gsim_grp = full_lt.get_rlzs_by_gsim_grp()
+        self.samples_by_grp = full_lt.get_samples_by_grp()
         self.num_rlzs_by_grp = {
             grp_id:
             sum(len(rlzs) for rlzs in self.rlzs_by_gsim_grp[grp_id].values())
@@ -134,42 +103,33 @@ class EventBasedCalculator(base.HazardCalculator):
         zd = {r: ProbabilityMap(self.L) for r in range(self.R)}
         return zd
 
-    def build_events_from_sources(self):
+    def build_events_from_sources(self, srcfilter):
         """
         Prefilter the composite source model and store the source_info
         """
-        oq = self.oqparam
-        gsims_by_trt = self.csm.gsim_lt.values
-
-        def weight_src(src):
-            return src.num_ruptures
-
+        gsims_by_trt = self.csm.full_lt.get_gsims_by_trt()
         logging.info('Building ruptures')
+        maxweight = sum(sg.weight for sg in self.csm.src_groups) / (
+            self.oqparam.concurrent_tasks or 1)
+        eff_ruptures = AccumDict(accum=0)  # trt => potential ruptures
+        calc_times = AccumDict(accum=numpy.zeros(3, F32))  # nr, ns, dt
+        allargs = []
+        if self.oqparam.is_ucerf():
+            # manage the filtering in a special way
+            for sg in self.csm.src_groups:
+                for src in sg:
+                    src.src_filter = srcfilter
+            srcfilter = nofilter  # otherwise it would be ultra-slow
+        for sg in self.csm.src_groups:
+            if not sg.sources:
+                continue
+            logging.info('Sending %s', sg)
+            par = self.param.copy()
+            par['gsims'] = gsims_by_trt[sg.trt]
+            for src_group in sg.split(maxweight):
+                allargs.append((src_group, srcfilter, par))
         smap = parallel.Starmap(
-            self.build_ruptures.__func__, monitor=self.monitor())
-        eff_ruptures = AccumDict(accum=0)  # grp_id => potential ruptures
-        calc_times = AccumDict(accum=numpy.zeros(2, F32))
-        ses_idx = 0
-        for sm_id, sm in enumerate(self.csm.source_models):
-            logging.info('Sending %s', sm)
-            for sg in sm.src_groups:
-                if not sg.sources:
-                    continue
-                par = self.param.copy()
-                par['gsims'] = gsims_by_trt[sg.trt]
-                if sg.atomic:  # do not split the group
-                    smap.submit(sg, self.src_filter, par)
-                else:  # traditional groups
-                    for block in self.block_splitter(
-                            sg.sources, weight_src, by_grp):
-                        if 'ucerf' in oq.calculation_mode:
-                            for i in range(oq.ses_per_logic_tree_path):
-                                par['ses_seeds'] = [
-                                    (ses_idx, oq.ses_seed + i + 1)]
-                                smap.submit(block, self.src_filter, par)
-                                ses_idx += 1
-                        else:
-                            smap.submit(block, self.src_filter, par)
+            sample_ruptures, allargs, h5=self.datastore.hdf5)
         mon = self.monitor('saving ruptures')
         for dic in smap:
             if dic['calc_times']:
@@ -186,49 +146,19 @@ class EventBasedCalculator(base.HazardCalculator):
 
         # logic tree reduction, must be called before storing the events
         self.store_rlz_info(eff_ruptures)
-        store_rlzs_by_grp(self.datastore)
-        self.init_logic_tree(self.csm.info)
-        with self.monitor('store source_info', autoflush=True):
+        self.init_logic_tree(self.csm.full_lt)
+        with self.monitor('store source_info'):
             self.store_source_info(calc_times)
         logging.info('Reordering the ruptures and storing the events')
-        attrs = self.datastore.getitem('ruptures').attrs
-        sorted_ruptures = self.datastore.getitem('ruptures').value
-        # order the ruptures by serial
+        sorted_ruptures = self.datastore.getitem('ruptures')[()]
+        # order the ruptures by rup_id
         sorted_ruptures.sort(order='serial')
-        ngroups = len(self.csm.info.trt_by_grp)
-        grp_indices = numpy.zeros((ngroups, 2), U32)
-        grp_ids = sorted_ruptures['grp_id']
-        for grp_id, [startstop] in get_indices(grp_ids).items():
-            grp_indices[grp_id] = startstop
+        nr = len(sorted_ruptures)
+        assert len(numpy.unique(sorted_ruptures['serial'])) == nr  # sanity
         self.datastore['ruptures'] = sorted_ruptures
-        self.datastore.set_attrs('ruptures', grp_indices=grp_indices, **attrs)
-        self.save_events(sorted_ruptures)
-
-    def gen_rupture_getters(self):
-        """
-        :returns: a list of RuptureGetters
-        """
-        dstore = (self.datastore.parent if self.datastore.parent
-                  else self.datastore)
-        hdf5cache = dstore.hdf5cache()
-        mode = 'r+' if os.path.exists(hdf5cache) else 'w'
-        with hdf5.File(hdf5cache, mode) as cache:
-            if 'ruptures' not in cache:
-                dstore.hdf5.copy('ruptures', cache)
-            if 'rupgeoms' not in cache:
-                dstore.hdf5.copy('rupgeoms', cache)
-        yield from gen_rupture_getters(
-            dstore, concurrent_tasks=self.oqparam.concurrent_tasks or 1,
-            hdf5cache=hdf5cache)
-        if self.datastore.parent:
-            self.datastore.parent.close()
-
-    @cached_property
-    def eid2idx(self):
-        """
-        :returns: a dict eid -> index in the events table
-        """
-        return dict(zip(self.datastore['events']['id'], range(self.E)))
+        self.datastore['ruptures']['id'] = numpy.arange(nr)
+        with self.monitor('saving events'):
+            self.save_events(sorted_ruptures)
 
     def agg_dicts(self, acc, result):
         """
@@ -239,32 +169,26 @@ class EventBasedCalculator(base.HazardCalculator):
         agg_mon = self.monitor('aggregating hcurves')
         with sav_mon:
             data = result.pop('gmfdata')
-            if len(data) == 0:
-                return acc
-            idxs = base.get_idxs(data, self.eid2idx)  # this has to be fast
-            data['eid'] = idxs  # replace eid with idx
-            self.datastore.extend('gmf_data/data', data)
-            sig_eps = result.pop('sig_eps')
-            sig_eps['eid'] = base.get_idxs(sig_eps, self.eid2idx)
-            self.datastore.extend('gmf_data/sigma_epsilon', sig_eps)
-            # it is important to save the number of bytes while the
-            # computation is going, to see the progress
-            update_nbytes(self.datastore, 'gmf_data/data', data)
-            for sid, start, stop in result['indices']:
-                self.indices[sid, 0].append(start + self.offset)
-                self.indices[sid, 1].append(stop + self.offset)
-            self.offset += len(data)
-            if self.offset >= TWO32:
-                raise RuntimeError(
-                    'The gmf_data table has more than %d rows' % TWO32)
+            if len(data):
+                times = result.pop('times')
+                rupids = list(times['rup_id'])
+                self.datastore['gmf_data/time_by_rup'][rupids] = times
+                hdf5.extend(self.datastore['gmf_data/data'], data)
+                sig_eps = result.pop('sig_eps')
+                hdf5.extend(self.datastore['gmf_data/sigma_epsilon'], sig_eps)
+                for sid, start, stop in result['indices']:
+                    self.indices[sid, 0].append(start + self.offset)
+                    self.indices[sid, 1].append(stop + self.offset)
+                self.offset += len(data)
+        if self.offset >= TWO32:
+            raise RuntimeError(
+                'The gmf_data table has more than %d rows' % TWO32)
         imtls = self.oqparam.imtls
         with agg_mon:
             for key, poes in result.get('hcurves', {}).items():
                 r, sid, imt = str2rsi(key)
                 array = acc[r].setdefault(sid, 0).array[imtls(imt), 0]
                 array[:] = 1. - (1. - array) * (1. - poes)
-        sav_mon.flush()
-        agg_mon.flush()
         self.datastore.flush()
         return acc
 
@@ -280,48 +204,80 @@ class EventBasedCalculator(base.HazardCalculator):
         events = numpy.zeros(len(eids), rupture.events_dt)
         # when computing the events all ruptures must be considered,
         # including the ones far away that will be discarded later on
-        rgetters = self.gen_rupture_getters()
-
-        # build the associations eid -> rlz in parallel
-        smap = parallel.Starmap(RuptureGetter.get_eid_rlz,
-                                ((rgetter,) for rgetter in rgetters),
-                                self.monitor('get_eid_rlz'),
-                                progress=logging.debug)
+        rgetters = gen_rgetters(self.datastore)
+        # build the associations eid -> rlz sequentially or in parallel
+        # this is very fast: I saw 30 million events associated in 1 minute!
+        logging.info('Building assocs event_id -> rlz_id for {:_d} events'
+                     ' and {:_d} ruptures'.format(len(events), len(rup_array)))
+        if len(events) < 1E5:
+            it = map(RuptureGetter.get_eid_rlz, rgetters)
+        else:
+            it = parallel.Starmap(RuptureGetter.get_eid_rlz,
+                                  ((rgetter,) for rgetter in rgetters),
+                                  progress=logging.debug,
+                                  h5=self.datastore.hdf5)
         i = 0
-        for eid_rlz in smap:  # 30 million of events associated in 1 minute!
+        for eid_rlz in it:
             for er in eid_rlz:
                 events[i] = er
                 i += 1
-        events.sort(order='id')  # fast too
-        n_unique_events = len(numpy.unique(events['id']))
+                if i >= TWO32:
+                    raise ValueError('There are more than %d events!' % i)
+        events.sort(order='rup_id')  # fast too
+        # sanity check
+        n_unique_events = len(numpy.unique(events[['id', 'rup_id']]))
         assert n_unique_events == len(events), (n_unique_events, len(events))
-        self.datastore['events'] = events
+        events['id'] = numpy.arange(len(events))
+        # set event year and event ses starting from 1
+        itime = int(self.oqparam.investigation_time)
+        nses = self.oqparam.ses_per_logic_tree_path
+        extra = numpy.zeros(len(events), [('year', U16), ('ses_id', U16)])
+        numpy.random.seed(self.oqparam.ses_seed)
+        extra['year'] = numpy.random.choice(itime, len(events)) + 1
+        extra['ses_id'] = numpy.random.choice(nses, len(events)) + 1
+        self.datastore['events'] = util.compose_arrays(events, extra)
+        eindices = get_indices(events['rup_id'])
+        arr = numpy.array(list(eindices.values()))[:, 0, :]
+        self.datastore['ruptures']['e0'] = arr[:, 0]
+        self.datastore['ruptures']['e1'] = arr[:, 1]
 
     def check_overflow(self):
         """
         Raise a ValueError if the number of sites is larger than 65,536 or the
         number of IMTs is larger than 256 or the number of ruptures is larger
         than 4,294,967,296. The limits are due to the numpy dtype used to
-        store the GMFs (gmv_dt). They could be relaxed in the future.
+        store the GMFs (gmv_dt). There also a limit of max_potential_gmfs on
+        the number of sites times the number of events, to avoid producing too
+        many GMFs. In that case split the calculation or be smarter.
         """
+        oq = self.oqparam
         max_ = dict(sites=TWO32, events=TWO32, imts=2**8)
         num_ = dict(events=self.E, imts=len(self.oqparam.imtls))
-        if self.sitecol:
-            num_['sites'] = len(self.sitecol)
+        n = len(getattr(self, 'sitecol', ()) or ())
+        num_['sites'] = n
+        if oq.calculation_mode == 'event_based' and oq.ground_motion_fields:
+            if n > oq.max_sites_per_gmf:
+                raise ValueError(
+                    'You cannot compute the GMFs for %d > %d sites' %
+                    (n, oq.max_sites_per_gmf))
+            elif n * self.E > oq.max_potential_gmfs:
+                raise ValueError(
+                    'A GMF calculation with %d sites and %d events is '
+                    'impossibly large' % (n, self.E))
         for var in num_:
             if num_[var] > max_[var]:
                 raise ValueError(
-                    'The event based calculator is restricted to '
-                    '%d %s, got %d' % (max_[var], var, num_[var]))
+                    'The %s calculator is restricted to %d %s, got %d' %
+                    (oq.calculation_mode, max_[var], var, num_[var]))
 
     def set_param(self, **kw):
         oq = self.oqparam
         # set the minimum_intensity
-        if hasattr(self, 'riskmodel') and not oq.minimum_intensity:
+        if hasattr(self, 'crmodel') and not oq.minimum_intensity:
             # infer it from the risk models if not directly set in job.ini
-            oq.minimum_intensity = self.riskmodel.min_iml
+            oq.minimum_intensity = self.crmodel.min_iml
         min_iml = oq.min_iml
-        if min_iml.sum() == 0:
+        if oq.ground_motion_fields and min_iml.sum() == 0:
             logging.warning('The GMFs are not filtered: '
                             'you may want to set a minimum_intensity')
         else:
@@ -330,7 +286,6 @@ class EventBasedCalculator(base.HazardCalculator):
             oqparam=oq,
             gmf=oq.ground_motion_fields,
             truncation_level=oq.truncation_level,
-            ruptures_per_block=oq.ruptures_per_block,
             imtls=oq.imtls, filter_distance=oq.filter_distance,
             ses_per_logic_tree_path=oq.ses_per_logic_tree_path, **kw)
 
@@ -338,45 +293,58 @@ class EventBasedCalculator(base.HazardCalculator):
         oq = self.oqparam
         self.set_param()
         self.offset = 0
-        self.indices = collections.defaultdict(list)  # sid, idx -> indices
-        if oq.hazard_calculation_id and 'ruptures' in self.datastore:
-            # from ruptures
-            self.datastore.parent = datastore.read(oq.hazard_calculation_id)
-            self.init_logic_tree(self.csm_info)
-        else:
-            # from sources
-            self.build_events_from_sources()
-            if oq.ground_motion_fields is False:
+        srcfilter = self.src_filter(self.datastore.tempname)
+        self.indices = AccumDict(accum=[])  # sid, idx -> indices
+        if oq.hazard_calculation_id:  # from ruptures
+            self.datastore.parent = util.read(oq.hazard_calculation_id)
+            self.init_logic_tree(self.datastore.parent['full_lt'])
+        else:  # from sources
+            self.build_events_from_sources(srcfilter)
+            if (oq.ground_motion_fields is False and
+                    oq.hazard_curves_from_gmfs is False):
                 return {}
-        iterargs = ((rgetter, self.src_filter, self.param)
-                    for rgetter in self.gen_rupture_getters())
-        # call compute_gmfs in parallel
+        if not oq.imtls:
+            raise InvalidFile('There are no intensity measure types in %s' %
+                              oq.inputs['job_ini'])
+        N = len(self.sitecol.complete)
+        if oq.ground_motion_fields:
+            nrups = len(self.datastore['ruptures'])
+            self.datastore.create_dset('gmf_data/data', oq.gmf_data_dt())
+            self.datastore.create_dset('gmf_data/sigma_epsilon',
+                                       sig_eps_dt(oq.imtls))
+            self.datastore.create_dset(
+                'gmf_data/indices', hdf5.vuint32, shape=(N, 2), fillvalue=None)
+            self.datastore.create_dset('gmf_data/events_by_sid', U32, (N,))
+            self.datastore.create_dset('gmf_data/time_by_rup',
+                                       time_dt, (nrups,), fillvalue=None)
+        if oq.hazard_curves_from_gmfs:
+            self.param['rlz_by_event'] = self.datastore['events']['rlz_id']
+
+        # compute_gmfs in parallel
+        self.datastore.swmr_on()
+        logging.info('Reading %d ruptures', len(self.datastore['ruptures']))
+        iterargs = ((rgetter, srcfilter, self.param)
+                    for rgetter in gen_rupture_getters(
+                            self.datastore, srcfilter, oq.concurrent_tasks))
         acc = parallel.Starmap(
-            self.core_task.__func__, iterargs, self.monitor()
+            self.core_task.__func__, iterargs, h5=self.datastore.hdf5,
+            num_cores=oq.num_cores
         ).reduce(self.agg_dicts, self.acc0())
 
         if self.indices:
-            N = len(self.sitecol.complete)
+            dset = self.datastore['gmf_data/indices']
+            num_evs = self.datastore['gmf_data/events_by_sid']
             logging.info('Saving gmf_data/indices')
-            with self.monitor('saving gmf_data/indices', measuremem=True,
-                              autoflush=True):
+            with self.monitor('saving gmf_data/indices', measuremem=True):
                 self.datastore['gmf_data/imts'] = ' '.join(oq.imtls)
-                dset = self.datastore.create_dset(
-                    'gmf_data/indices', hdf5.vuint32,
-                    shape=(N, 2), fillvalue=None)
-                num_evs = self.datastore.create_dset(
-                    'gmf_data/events_by_sid', U32, (N,))
                 for sid in self.sitecol.complete.sids:
                     start = numpy.array(self.indices[sid, 0])
                     stop = numpy.array(self.indices[sid, 1])
                     dset[sid, 0] = start
                     dset[sid, 1] = stop
                     num_evs[sid] = (stop - start).sum()
-                avg_events_by_sid = num_evs.value.sum() / N
-                logging.info('Found ~%d GMVs per site', avg_events_by_sid)
-                self.datastore.set_attrs(
-                    'gmf_data', avg_events_by_sid=avg_events_by_sid,
-                    max_events_by_sid=num_evs.value.max())
+            avg_events_by_sid = num_evs[()].sum() / N
+            logging.info('Found ~%d GMVs per site', avg_events_by_sid)
         elif oq.ground_motion_fields:
             raise RuntimeError('No GMFs were generated, perhaps they were '
                                'all below the minimum_intensity threshold')
@@ -384,12 +352,12 @@ class EventBasedCalculator(base.HazardCalculator):
 
     def post_execute(self, result):
         oq = self.oqparam
-        if not oq.ground_motion_fields:
+        if not oq.ground_motion_fields and not oq.hazard_curves_from_gmfs:
             return
         N = len(self.sitecol.complete)
         L = len(oq.imtls.array)
         if result and oq.hazard_curves_from_gmfs:
-            rlzs = self.rlzs_assoc.realizations
+            rlzs = self.datastore['full_lt'].get_realizations()
             # compute and save statistics; this is done in process and can
             # be very slow if there are thousands of realizations
             weights = [rlz.weight for rlz in rlzs]
@@ -457,11 +425,12 @@ class EventBasedCalculator(base.HazardCalculator):
                 os.makedirs(export_dir)
             oq.export_dir = export_dir
             job_id = logs.init('job')
+            oq.calculation_mode = 'classical'
             self.cl = ClassicalCalculator(oq, job_id)
             # TODO: perhaps it is possible to avoid reprocessing the source
             # model, however usually this is quite fast and do not dominate
             # the computation
-            self.cl.run(close=False)
+            self.cl.run()
             engine.expose_outputs(self.cl.datastore)
             cl_mean_curves = get_mean_curves(self.cl.datastore)
             eb_mean_curves = get_mean_curves(self.datastore)
