@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2019 GEM Foundation
+# Copyright (C) 2014-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -18,33 +18,26 @@
 
 import re
 import os
-import logging
+import sys
 import operator
 import collections
-
 import numpy
 
-from openquake.baselib.general import humansize, group_array, deprecated
+from openquake.baselib.general import (
+    group_array, deprecated, AccumDict, DictArray)
 from openquake.hazardlib.imt import from_string
-from openquake.hazardlib.calc import disagg, filters
+from openquake.hazardlib.calc import disagg
 from openquake.calculators.views import view
 from openquake.calculators.extract import extract, get_mesh, get_info
 from openquake.calculators.export import export
-from openquake.calculators.getters import GmfGetter, gen_rupture_getters
-from openquake.commonlib import writers, hazard_writers, calc, util, source
+from openquake.calculators.getters import gen_rgetters
+from openquake.commonlib import writers, hazard_writers, calc, util
 
 F32 = numpy.float32
 F64 = numpy.float64
 U8 = numpy.uint8
 U16 = numpy.uint16
 U32 = numpy.uint32
-U64 = numpy.uint64
-
-
-GMF_MAX_SIZE = 10 * 1024 * 1024  # 10 MB
-GMF_WARNING = '''\
-There are a lot of ground motion fields; the export will be slow.
-Consider canceling the operation and accessing directly %s.'''
 
 # with compression you can save 60% of space by losing only 10% of saving time
 savez = numpy.savez_compressed
@@ -56,6 +49,7 @@ def add_quotes(values):
 
 
 @export.add(('ruptures', 'xml'))
+@deprecated(msg='This exporter will disappear in the future')
 def export_ruptures_xml(ekey, dstore):
     """
     :param ekey: export key, i.e. a pair (datastore key, fmt)
@@ -63,14 +57,15 @@ def export_ruptures_xml(ekey, dstore):
     """
     fmt = ekey[-1]
     oq = dstore['oqparam']
-    num_ses = oq.ses_per_logic_tree_path
-    mesh = get_mesh(dstore['sitecol'])
-    ruptures_by_grp = {}
-    for rgetter in gen_rupture_getters(dstore):
-        ebrs = [ebr.export(mesh, rgetter.rlzs_by_gsim, num_ses)
-                for ebr in rgetter.get_ruptures()]
-        if ebrs:
-            ruptures_by_grp[rgetter.grp_id] = ebrs
+    events = group_array(dstore['events'][()], 'rup_id')
+    ruptures_by_grp = AccumDict(accum=[])
+    for rgetter in gen_rgetters(dstore):
+        ebrs = []
+        for proxy in rgetter.get_proxies():
+            events_by_ses = group_array(events[proxy['id']], 'ses_id')
+            ebr = proxy.to_ebr(rgetter.trt, rgetter.samples)
+            ebrs.append(ebr.export(events_by_ses))
+        ruptures_by_grp[rgetter.grp_id].extend(ebrs)
     dest = dstore.export_path('ses.' + fmt)
     writer = hazard_writers.SESXMLWriter(dest)
     writer.serialize(ruptures_by_grp, oq.investigation_time)
@@ -87,134 +82,19 @@ def export_ruptures_csv(ekey, dstore):
     if 'scenario' in oq.calculation_mode:
         return []
     dest = dstore.export_path('ruptures.csv')
-    header = ('rupid multiplicity mag centroid_lon centroid_lat '
-              'centroid_depth trt strike dip rake boundary').split()
-    rows = []
-    for rgetter in gen_rupture_getters(dstore):
-        rups = rgetter.get_ruptures()
-        rup_data = calc.RuptureData(rgetter.trt, rgetter.rlzs_by_gsim)
-        for r in rup_data.to_array(rups):
-            rows.append(
-                (r['rup_id'], r['multiplicity'], r['mag'],
-                 r['lon'], r['lat'], r['depth'],
-                 rgetter.trt, r['strike'], r['dip'], r['rake'],
-                 r['boundary']))
-    rows.sort()  # by rupture serial
-    comment = 'investigation_time=%s, ses_per_logic_tree_path=%s' % (
-        oq.investigation_time, oq.ses_per_logic_tree_path)
-    writers.write_csv(dest, rows, header=header, sep='\t', comment=comment)
+    arr = extract(dstore, 'rupture_info')
+    if export.sanity_check:
+        bad = view('bad_ruptures', dstore)
+        if bad.count('\n') > 3:  # nonempty rst_table
+            print(bad, file=sys.stderr)
+    comment = dstore.metadata
+    comment.update(investigation_time=oq.investigation_time,
+                   ses_per_logic_tree_path=oq.ses_per_logic_tree_path)
+    arr.array.sort(order='rup_id')
+    writers.write_csv(dest, arr, comment=comment)
     return [dest]
 
 
-# #################### export Ground Motion fields ########################## #
-
-class GmfSet(object):
-    """
-    Small wrapper around the list of Gmf objects associated to the given SES.
-    """
-    def __init__(self, gmfset, investigation_time, ses_idx):
-        self.gmfset = gmfset
-        self.investigation_time = investigation_time
-        self.stochastic_event_set_id = ses_idx
-
-    def __iter__(self):
-        return iter(self.gmfset)
-
-    def __bool__(self):
-        return bool(self.gmfset)
-
-    def __str__(self):
-        return (
-            'GMFsPerSES(investigation_time=%f, '
-            'stochastic_event_set_id=%s,\n%s)' % (
-                self.investigation_time,
-                self.stochastic_event_set_id, '\n'.join(
-                    sorted(str(g) for g in self.gmfset))))
-
-
-class GroundMotionField(object):
-    """
-    The Ground Motion Field generated by the given rupture
-    """
-    def __init__(self, imt, event_id, gmf_nodes):
-        self.imt = imt.name
-        self.sa_period = getattr(imt, 'period', None) or None
-        self.sa_damping = getattr(imt, 'damping', None)
-        self.event_id = event_id
-        self.gmf_nodes = gmf_nodes
-
-    def __iter__(self):
-        return iter(self.gmf_nodes)
-
-    def __getitem__(self, key):
-        return self.gmf_nodes[key]
-
-    def __str__(self):
-        # string representation of a _GroundMotionField object showing the
-        # content of the nodes (lon, lat an gmv). This is useful for debugging
-        # and testing.
-        mdata = ('imt=%(imt)s sa_period=%(sa_period)s '
-                 'sa_damping=%(sa_damping)s event_id=%(event_id)s' %
-                 vars(self))
-        nodes = sorted(map(str, self.gmf_nodes))
-        return 'GMF(%s\n%s)' % (mdata, '\n'.join(nodes))
-
-
-class GroundMotionFieldNode(object):
-    # the signature is not (gmv, x, y) because the XML writer expects
-    # a location object
-    def __init__(self, gmv, loc):
-        self.gmv = gmv
-        self.location = loc
-
-    def __lt__(self, other):
-        """
-        A reproducible ordering by lon and lat; used in
-        :function:`openquake.commonlib.hazard_writers.gen_gmfs`
-        """
-        return (self.location.x, self.location.y) < (
-            other.location.x, other.location.y)
-
-    def __str__(self):
-        """Return lon, lat and gmv of the node in a compact string form"""
-        return '<X=%9.5f, Y=%9.5f, GMV=%9.7f>' % (
-            self.location.x, self.location.y, self.gmv)
-
-
-class GmfCollection(object):
-    """
-    Object converting the parameters
-
-    :param sitecol: SiteCollection
-    :param ruptures: ruptures
-    :param investigation_time: investigation time
-
-    into an object with the right form for the EventBasedGMFXMLWriter.
-    Iterating over a GmfCollection yields GmfSet objects.
-    """
-    def __init__(self, sitecol, imts, ruptures, investigation_time):
-        self.sitecol = sitecol
-        self.ruptures = ruptures
-        self.imts = imts
-        self.investigation_time = investigation_time
-
-    def __iter__(self):
-        completemesh = self.sitecol.complete.mesh
-        gmfset = collections.defaultdict(list)
-        for imti, imt_str in enumerate(self.imts):
-            imt = from_string(imt_str)
-            for rupture in self.ruptures:
-                gmf = rupture.gmfa['gmv'][:, imti]
-                mesh = completemesh[rupture.indices]
-                assert len(mesh) == len(gmf), (len(mesh), len(gmf))
-                nodes = (GroundMotionFieldNode(gmv, loc)
-                         for gmv, loc in zip(gmf, mesh))
-                gmfset[rupture.ses_idx].append(
-                    GroundMotionField(imt, rupture.eid, nodes))
-        for ses_idx in sorted(gmfset):
-            yield GmfSet(gmfset[ses_idx], self.investigation_time, ses_idx)
-
-
 # ####################### export hazard curves ############################ #
 
 HazardCurve = collections.namedtuple('HazardCurve', 'location poes')
@@ -246,22 +126,22 @@ def add_imt(fname, imt):
 
 
 def export_hcurves_by_imt_csv(
-        key, kind, rlzs_assoc, fname, sitecol, array, oq, checksum):
+        key, kind, fname, sitecol, array, imtls, comment):
     """
     Export the curves of the given realization into CSV.
 
     :param key: output_type and export_type
     :param kind: a string with the kind of output (realization or statistics)
-    :param rlzs_assoc: a :class:`openquake.commonlib.source.RlzsAssoc` instance
     :param fname: name of the exported file
     :param sitecol: site collection
     :param array: an array of shape (N, L) and dtype numpy.float32
-    :param oq: job.ini parameters
+    :param imtls: intensity measure types and levels
+    :param comment: comment dictionary
     """
     nsites = len(sitecol)
     fnames = []
-    for imt, imls in oq.imtls.items():
-        slc = oq.imtls(imt)
+    for imt, imls in imtls.items():
+        slc = imtls(imt)
         dest = add_imt(fname, imt)
         lst = [('lon', F32), ('lat', F32), ('depth', F32)]
         for iml in imls:
@@ -270,19 +150,18 @@ def export_hcurves_by_imt_csv(
         for sid, lon, lat, dep in zip(
                 range(nsites), sitecol.lons, sitecol.lats, sitecol.depths):
             hcurves[sid] = (lon, lat, dep) + tuple(array[sid, slc])
-        fnames.append(writers.write_csv(dest, hcurves, comment=_comment(
-            rlzs_assoc, kind, oq.investigation_time) + (
-                ', imt="%s", checksum=%d' % (imt, checksum)
-            ), header=[name for (name, dt) in lst]))
+        comment.update(imt=imt)
+        fnames.append(
+            writers.write_csv(dest, hcurves, comment=comment,
+                              header=[name for (name, dt) in lst]))
     return fnames
 
 
-def hazard_curve_name(dstore, ekey, kind, rlzs_assoc):
+def hazard_curve_name(dstore, ekey, kind):
     """
     :param calc_id: the calculation ID
     :param ekey: the export key
     :param kind: the kind of key
-    :param rlzs_assoc: a RlzsAssoc instance
     """
     key, fmt = ekey
     prefix = {'hcurves': 'hazard_curve', 'hmaps': 'hazard_map',
@@ -294,28 +173,6 @@ def hazard_curve_name(dstore, ekey, kind, rlzs_assoc):
     return fname
 
 
-def _comment(rlzs_assoc, kind, investigation_time):
-    rlz = rlzs_assoc.get_rlz(kind)
-    if not rlz:
-        return '%s, investigation_time=%s' % (kind, investigation_time)
-    else:
-        return (
-            'source_model_tree_path=%s, gsim_tree_path=%s, '
-            'investigation_time=%s' % (
-                rlz.sm_lt_path, rlz.gsim_lt_path, investigation_time))
-
-
-def build_hcurves(getter, imtls, monitor):
-    with getter.dstore:
-        pmaps = getter.get_pmaps()
-        idx = dict(zip(getter.sids, range(len(getter.sids))))
-        curves = numpy.zeros((len(getter.sids), len(pmaps)), imtls.dt)
-        for r, pmap in enumerate(pmaps):
-            for sid in pmap:
-                curves[idx[sid], r] = pmap[sid].convert(imtls)
-    return getter.sids, curves
-
-
 def get_kkf(ekey):
     """
     :param ekey: export key, for instance ('uhs/rlz-1', 'xml')
@@ -339,17 +196,16 @@ def export_hcurves_csv(ekey, dstore):
     """
     oq = dstore['oqparam']
     info = get_info(dstore)
-    rlzs_assoc = dstore['csm_info'].get_rlzs_assoc()
-    R = len(rlzs_assoc.realizations)
+    R = dstore['full_lt'].get_num_rlzs()
     sitecol = dstore['sitecol']
     sitemesh = get_mesh(sitecol)
     key, kind, fmt = get_kkf(ekey)
     fnames = []
-    checksum = dstore.get_attr('/', 'checksum32')
+    comment = dstore.metadata
     hmap_dt = oq.hmap_dt()
     for kind in oq.get_kinds(kind, R):
-        fname = hazard_curve_name(dstore, (key, fmt), kind, rlzs_assoc)
-        comment = _comment(rlzs_assoc, kind, oq.investigation_time)
+        fname = hazard_curve_name(dstore, (key, fmt), kind)
+        comment.update(kind=kind, investigation_time=oq.investigation_time)
         if (key in ('hmaps', 'uhs') and oq.uniform_hazard_spectra or
                 oq.hazard_maps):
             hmap = extract(dstore, 'hmaps?kind=' + kind)[kind]
@@ -357,19 +213,22 @@ def export_hcurves_csv(ekey, dstore):
             uhs_curves = calc.make_uhs(hmap, info)
             writers.write_csv(
                 fname, util.compose_arrays(sitemesh, uhs_curves),
-                comment=comment + ', checksum=%d' % checksum)
+                comment=comment)
             fnames.append(fname)
         elif key == 'hmaps' and oq.poes and oq.hazard_maps:
             fnames.extend(
                 export_hmaps_csv(ekey, fname, sitemesh,
-                                 hmap.flatten().view(hmap_dt),
-                                 comment + ', checksum=%d' % checksum))
+                                 hmap.flatten().view(hmap_dt), comment))
         elif key == 'hcurves':
             hcurves = extract(dstore, 'hcurves?kind=' + kind)[kind]
+            if 'amplification' in oq.inputs:
+                imtls = DictArray(
+                    {imt: oq.soil_intensities for imt in oq.imtls})
+            else:
+                imtls = oq.imtls
             fnames.extend(
                 export_hcurves_by_imt_csv(
-                    ekey, kind, rlzs_assoc, fname, sitecol, hcurves, oq,
-                    checksum))
+                    ekey, kind, fname, sitecol, hcurves, imtls, comment))
     return sorted(fnames)
 
 
@@ -389,7 +248,7 @@ def get_metadata(realizations, kind):
     if kind.startswith('rlz-'):
         rlz = realizations[int(kind[4:])]
         metadata['smlt_path'] = '_'.join(rlz.sm_lt_path)
-        metadata['gsimlt_path'] = rlz.gsim_rlz.uid
+        metadata['gsimlt_path'] = rlz.gsim_rlz.pid
     elif kind.startswith('quantile-'):
         metadata['statistics'] = 'quantile'
         metadata['quantile_value'] = float(kind[9:])
@@ -406,18 +265,17 @@ def get_metadata(realizations, kind):
 @deprecated(msg='Use the CSV exporter instead')
 def export_uhs_xml(ekey, dstore):
     oq = dstore['oqparam']
-    rlzs_assoc = dstore['csm_info'].get_rlzs_assoc()
-    R = len(rlzs_assoc.realizations)
+    rlzs = dstore['full_lt'].get_realizations()
+    R = len(rlzs)
     sitemesh = get_mesh(dstore['sitecol'].complete)
     key, kind, fmt = get_kkf(ekey)
     fnames = []
     periods = [imt.period for imt in oq.imt_periods()]
     for kind in oq.get_kinds(kind, R):
-        metadata = get_metadata(rlzs_assoc.realizations, kind)
+        metadata = get_metadata(rlzs, kind)
         uhs = extract(dstore, 'uhs?kind=' + kind)[kind]
         for p, poe in enumerate(oq.poes):
-            fname = hazard_curve_name(
-                dstore, (key, fmt), kind + '-%s' % poe, rlzs_assoc)
+            fname = hazard_curve_name(dstore, (key, fmt), kind + '-%s' % poe)
             writer = hazard_writers.UHSXMLWriter(
                 fname, periods=periods, poe=poe,
                 investigation_time=oq.investigation_time, **metadata)
@@ -446,19 +304,19 @@ def export_hcurves_xml(ekey, dstore):
     len_ext = len(fmt) + 1
     oq = dstore['oqparam']
     sitemesh = get_mesh(dstore['sitecol'])
-    rlzs_assoc = dstore['csm_info'].get_rlzs_assoc()
-    R = len(rlzs_assoc.realizations)
+    rlzs = dstore['full_lt'].get_realizations()
+    R = len(rlzs)
     fnames = []
     writercls = hazard_writers.HazardCurveXMLWriter
     for kind in oq.get_kinds(kind, R):
         if kind.startswith('rlz-'):
-            rlz = rlzs_assoc.realizations[int(kind[4:])]
+            rlz = rlzs[int(kind[4:])]
             smlt_path = '_'.join(rlz.sm_lt_path)
-            gsimlt_path = rlz.gsim_rlz.uid
+            gsimlt_path = rlz.gsim_rlz.pid
         else:
             smlt_path = ''
             gsimlt_path = ''
-        name = hazard_curve_name(dstore, ekey, kind, rlzs_assoc)
+        name = hazard_curve_name(dstore, ekey, kind)
         hcurves = extract(dstore, 'hcurves?kind=' + kind)[kind]
         for im in oq.imtls:
             slc = oq.imtls(im)
@@ -484,25 +342,24 @@ def export_hmaps_xml(ekey, dstore):
     oq = dstore['oqparam']
     sitecol = dstore['sitecol']
     sitemesh = get_mesh(sitecol)
-    rlzs_assoc = dstore['csm_info'].get_rlzs_assoc()
-    R = len(rlzs_assoc.realizations)
+    rlzs = dstore['full_lt'].get_realizations()
+    R = len(rlzs)
     fnames = []
     writercls = hazard_writers.HazardMapXMLWriter
     for kind in oq.get_kinds(kind, R):
         # shape (N, M, P)
         hmaps = extract(dstore, 'hmaps?kind=' + kind)[kind]
         if kind.startswith('rlz-'):
-            rlz = rlzs_assoc.realizations[int(kind[4:])]
+            rlz = rlzs[int(kind[4:])]
             smlt_path = '_'.join(rlz.sm_lt_path)
-            gsimlt_path = rlz.gsim_rlz.uid
+            gsimlt_path = rlz.gsim_rlz.pid
         else:
             smlt_path = ''
             gsimlt_path = ''
         for m, imt in enumerate(oq.imtls):
             for p, poe in enumerate(oq.poes):
                 suffix = '-%s-%s' % (poe, imt)
-                fname = hazard_curve_name(
-                    dstore, ekey, kind + suffix, rlzs_assoc)
+                fname = hazard_curve_name(dstore, ekey, kind + suffix)
                 data = [HazardMap(site[0], site[1], hmap[m, p])
                         for site, hmap in zip(sitemesh, hmaps)]
                 writer = writercls(
@@ -523,85 +380,54 @@ def _extract(hmap, imt, j):
     return tup
 
 
-@export.add(('hcurves', 'npz'), ('hmaps', 'npz'), ('uhs', 'npz'))
+@export.add(('hcurves', 'npz'), ('hmaps', 'npz'), ('uhs', 'npz'),
+            ('gmf_data', 'npz'),
+            ('losses_by_asset', 'npz'), ('dmg_by_asset', 'npz'))
 def export_hazard_npz(ekey, dstore):
     fname = dstore.export_path('%s.%s' % ekey)
-    savez(fname, **vars(extract(dstore, ekey[0])))
+    out = extract(dstore, ekey[0])
+    kw = {k: v for k, v in vars(out).items() if not k.startswith('_')}
+    savez(fname, **kw)
     return [fname]
 
 
-@export.add(('gmf_data', 'xml'))
-@deprecated(msg='Use the CSV exporter instead')
-def export_gmf(ekey, dstore):
-    """
-    :param ekey: export key, i.e. a pair (datastore key, fmt)
-    :param dstore: datastore object
-    """
-    oq = dstore['oqparam']
-    if not oq.calculation_mode.startswith('scenario'):
-        return []
-    sitecol = dstore['sitecol']
-    investigation_time = (None if oq.calculation_mode == 'scenario'
-                          else oq.investigation_time)
-    fmt = ekey[-1]
-    gmf_data = dstore['gmf_data']
-    nbytes = gmf_data.attrs['nbytes']
-    logging.info('Internal size of the GMFs: %s', humansize(nbytes))
-    if nbytes > GMF_MAX_SIZE:
-        logging.warning(GMF_WARNING, dstore.filename)
-    data = gmf_data['data'].value
-    ses_idx = 1  # for scenario only
-    events = []
-    for eid, gmfa in group_array(data, 'eid').items():
-        rup = Event(eid, ses_idx, sorted(set(gmfa['sid'])), gmfa)
-        events.append(rup)
-    fname = dstore.build_fname('gmf', 'scenario', fmt)
-    writer = hazard_writers.EventBasedGMFXMLWriter(
-        fname, sm_lt_path='', gsim_lt_path='')
-    writer.serialize(
-        GmfCollection(sitecol, oq.imtls, events, investigation_time))
-    return [fname]
-
-
-Event = collections.namedtuple('Event', ['eid', 'ses_idx', 'indices', 'gmfa'])
-
-
 @export.add(('gmf_data', 'csv'))
 def export_gmf_data_csv(ekey, dstore):
     oq = dstore['oqparam']
-    rlzs_assoc = dstore['csm_info'].get_rlzs_assoc()
+    rlzs = dstore['full_lt'].get_realizations()
     imts = list(oq.imtls)
     sc = dstore['sitecol'].array
-    if 'vs30' in sc.dtype.names:
-        arr = sc[['lon', 'lat', 'vs30']]
-    else:
-        arr = sc[['lon', 'lat']]
+    arr = sc[['lon', 'lat']]
     eid = int(ekey[0].split('/')[1]) if '/' in ekey[0] else None
-    gmfa = dstore['gmf_data/data'].value[['eid', 'sid', 'gmv']].copy()
+    gmfa = dstore['gmf_data/data'][('eid', 'sid', 'gmv')]
     event_id = dstore['events']['id']
     gmfa['eid'] = event_id[gmfa['eid']]
     if eid is None:  # we cannot use extract here
-        f = dstore.build_fname('sitemodel', '', 'csv')
+        f = dstore.build_fname('sitemesh', '', 'csv')
         sids = numpy.arange(len(arr), dtype=U32)
         sites = util.compose_arrays(sids, arr, 'site_id')
         writers.write_csv(f, sites)
         fname = dstore.build_fname('gmf', 'data', 'csv')
         gmfa.sort(order=['eid', 'sid'])
-        writers.write_csv(fname, _expand_gmv(gmfa, imts))
+        writers.write_csv(fname, _expand_gmv(gmfa, imts),
+                          renamedict={'sid': 'site_id', 'eid': 'event_id'})
         if 'sigma_epsilon' in dstore['gmf_data']:
             sig_eps_csv = dstore.build_fname('sigma_epsilon', '', 'csv')
-            sig_eps = dstore['gmf_data/sigma_epsilon'].value
+            sig_eps = dstore['gmf_data/sigma_epsilon'][()]
             sig_eps['eid'] = event_id[sig_eps['eid']]
             sig_eps.sort(order='eid')
-            writers.write_csv(sig_eps_csv, sig_eps)
+            header = list(sig_eps.dtype.names)
+            header[0] = 'event_id'
+            writers.write_csv(sig_eps_csv, sig_eps, header=header)
             return [fname, sig_eps_csv, f]
         else:
             return [fname, f]
     # old format for single eid
+    # TODO: is this still used?
     gmfa = gmfa[gmfa['eid'] == eid]
     eid2rlz = dict(dstore['events'])
     rlzi = eid2rlz[eid]
-    rlz = rlzs_assoc.realizations[rlzi]
+    rlz = rlzs[rlzi]
     data, comment = _build_csv_data(
         gmfa, rlz, dstore['sitecol'], imts, oq.investigation_time)
     fname = dstore.build_fname(
@@ -621,13 +447,20 @@ def _expand_gmv(array, imts):
                 dtlist.append(('gmv_' + imt, F32))
         else:
             dtlist.append((name, dt))
-    return array.view(dtlist)
+    new = numpy.zeros(len(array), dtlist)
+    for name in dtype.names:
+        if name == 'gmv':
+            for i, imt in enumerate(imts):
+                new['gmv_' + imt] = array['gmv'][:, i]
+        else:
+            new[name] = array[name]
+    return new
 
 
 def _build_csv_data(array, rlz, sitecol, imts, investigation_time):
     # lon, lat, gmv_imt1, ..., gmv_imtN
     smlt_path = '_'.join(rlz.sm_lt_path)
-    gsimlt_path = rlz.gsim_rlz.uid
+    gsimlt_path = rlz.gsim_rlz.pid
     comment = ('smlt_path=%s, gsimlt_path=%s, investigation_time=%s' %
                (smlt_path, gsimlt_path, investigation_time))
     rows = [['lon', 'lat'] + imts]
@@ -638,77 +471,22 @@ def _build_csv_data(array, rlz, sitecol, imts, investigation_time):
     return rows, comment
 
 
-@export.add(('gmf_scenario', 'csv'))
-def export_gmf_scenario_csv(ekey, dstore):
-    what = ekey[0].split('/')
-    if len(what) == 1:
-        raise ValueError(r'Missing "/rup-\d+"')
-    oq = dstore['oqparam']
-    csm_info = dstore['csm_info']
-    rlzs_assoc = csm_info.get_rlzs_assoc()
-    num_ruptures = len(dstore['ruptures'])
-    imts = list(oq.imtls)
-    mo = re.match(r'rup-(\d+)$', what[1])
-    if mo is None:
-        raise ValueError(
-            r"Invalid format: %r does not match 'rup-(\d+)$'" % what[1])
-    ridx = int(mo.group(1))
-    assert 0 <= ridx < num_ruptures, ridx
-    # for scenario there is an unique grp_id=0
-    [rgetter] = gen_rupture_getters(dstore, slice(ridx, ridx + 1))
-    [ebr] = rgetter.get_ruptures()
-    sitecol = dstore['sitecol'].complete
-    srcfilter = filters.SourceFilter(sitecol, oq.maximum_distance)
-    getter = GmfGetter(rgetter, srcfilter, oq)
-    getter.init()
-    eids = rgetter.get_eid_rlz()['eid']
-    sids = getter.computers[0].sids
-    rlzs = rlzs_assoc.realizations
-    hazardr = [collections.defaultdict(list) for rlz in rlzs]
-    for sid, haz in getter.get_hazard().items():
-        for rec in haz:
-            hazardr[rec['rlzi']][sid].append(rec)
-    fields = ['eid-%03d' % eid for eid in eids]
-    dt = numpy.dtype([(f, F32) for f in fields])
-    mesh = numpy.zeros(len(sids), [('lon', F64), ('lat', F64)])
-    mesh['lon'] = sitecol.lons[sids]
-    mesh['lat'] = sitecol.lats[sids]
-    writer = writers.CsvWriter(fmt='%.5f')
-    for rlzi in range(len(rlzs)):
-        hazard = hazardr[rlzi]
-        for imti, imt in enumerate(imts):
-            gmfs = numpy.zeros(len(sids), dt)
-            for s, sid in enumerate(sids):
-                for rec in hazard[sid]:
-                    event = 'eid-%03d' % rec['eid']
-                    gmfs[s][event] = rec['gmv'][imti]
-            dest = dstore.build_fname(
-                'gmf', 'rup-%s-rlz-%s-%s' % (ebr.serial, rlzi, imt), 'csv')
-            data = util.compose_arrays(mesh, gmfs)
-            writer.save(data, dest)
-    return writer.getsaved()
-
-
-@export.add(('gmf_data', 'npz'))
-def export_gmf_scenario_npz(ekey, dstore):
-    fname = dstore.export_path('%s.%s' % ekey)
-    savez(fname, **vars(extract(dstore, 'gmf_data')))
-    return [fname]
-
-
 DisaggMatrix = collections.namedtuple(
     'DisaggMatrix', 'poe iml dim_labels matrix')
 
-
+# NB: this is not able to export the mean files
 @export.add(('disagg', 'xml'))
+@deprecated(msg='Use the CSV exporter instead')
 def export_disagg_xml(ekey, dstore):
     oq = dstore['oqparam']
-    rlzs = dstore['csm_info'].get_rlzs_assoc().realizations
+    rlzs = dstore['full_lt'].get_realizations()
     group = dstore['disagg']
     fnames = []
     writercls = hazard_writers.DisaggXMLWriter
-    trts = dstore.get_attr('csm_info', 'trts')
+    trts = dstore.get_attr('full_lt', 'trts')
     for key in group:
+        if not key.startswith('rlz-'):
+            continue
         matrix = dstore['disagg/' + key]
         attrs = group[key].attrs
         rlz = rlzs[attrs['rlzi']]
@@ -720,7 +498,7 @@ def export_disagg_xml(ekey, dstore):
         writer = writercls(
             fname, investigation_time=oq.investigation_time,
             imt=imt.name, smlt_path='_'.join(rlz.sm_lt_path),
-            gsimlt_path=rlz.gsim_rlz.uid, lon=lon, lat=lat,
+            gsimlt_path=rlz.gsim_rlz.pid, lon=lon, lat=lat,
             sa_period=getattr(imt, 'period', None) or None,
             sa_damping=getattr(imt, 'damping', None),
             mag_bin_edges=attrs['mag_bin_edges'],
@@ -737,70 +515,38 @@ def export_disagg_xml(ekey, dstore):
     return sorted(fnames)
 
 
-# adapted from the nrml_converters
-def save_disagg_to_csv(metadata, matrices):
-    """
-    Save disaggregation matrices to multiple .csv files.
-    """
-    skip_keys = ('Mag', 'Dist', 'Lon', 'Lat', 'Eps', 'TRT')
-    base_header = ','.join(
-        '%s=%s' % (key, value) for key, value in metadata.items()
-        if value is not None and key not in skip_keys)
-    for disag_tup, (poe, iml, matrix, fname) in matrices.items():
-        header = '%s,poe=%.7f,iml=%.7e\n' % (base_header, poe, iml)
-
-        if disag_tup == ('Mag', 'Lon', 'Lat'):
-            matrix = numpy.swapaxes(matrix, 0, 1)
-            matrix = numpy.swapaxes(matrix, 1, 2)
-            disag_tup = ('Lon', 'Lat', 'Mag')
-
-        axis = [metadata[v] for v in disag_tup]
-        header += ','.join(v for v in disag_tup)
-        header += ',poe'
-
-        # compute axis mid points
-        axis = [(ax[: -1] + ax[1:]) / 2. if ax.dtype == float
-                else ax for ax in axis]
-
-        values = None
-        if len(axis) == 1:
-            values = numpy.array([axis[0], matrix.flatten()]).T
-        else:
-            grids = numpy.meshgrid(*axis, indexing='ij')
-            values = [g.flatten() for g in grids]
-            values.append(matrix.flatten())
-            values = numpy.array(values).T
-
-        writers.write_csv(fname, values, comment=header, fmt='%.5E')
-
-
-@export.add(('disagg', 'csv'), ('disagg-stats', 'csv'))
+@export.add(('disagg', 'csv'))
 def export_disagg_csv(ekey, dstore):
     oq = dstore['oqparam']
-    disagg_outputs = oq.disagg_outputs or disagg.pmf_map
-    rlzs = dstore['csm_info'].get_rlzs_assoc().realizations
+    rlzs = dstore['full_lt'].get_realizations()
     group = dstore[ekey[0]]
     fnames = []
-    trts = dstore.get_attr('csm_info', 'trts')
+    skip_keys = ('Mag', 'Dist', 'Lon', 'Lat', 'Eps', 'TRT')
     for key in group:
-        matrix = dstore[ekey[0] + '/' + key]
         attrs = group[key].attrs
-        iml = attrs['iml']
+        try:
+            poe = attrs['poe']
+        except Exception:  # no poes_disagg were given
+            poe = attrs['poe_agg'][0]
         try:
             rlz = rlzs[attrs['rlzi']]
-        except TypeError:  # for stats
-            rlz = attrs['rlzi']
-        try:
-            poes = [attrs['poe']] * len(disagg_outputs)
-        except Exception:  # no poes_disagg were given
-            poes = attrs['poe_agg']
+        except TypeError:
+            rlz = ''
+            dic = dict(poe='%.7f' % poe)
+            ex = 'disagg?kind=%s&imt=%s&site_id=%s&poe_id=%d'
+        else:
+            dic = dict(poe='%.7f' % poe, iml='%.7e' % attrs['iml'],
+                       rlz=rlz.ordinal)
+            ex = ('disagg?kind=%s&imt=%s&site_id=%s&poe_id=%d&rlz=' +
+                  str(rlz.ordinal))
         imt = from_string(attrs['imt'])
+        site_id = attrs['site_id']
         lon, lat = attrs['location']
-        metadata = {}
+        metadata = dstore.metadata
         # Loads "disaggMatrices" nodes
         if hasattr(rlz, 'sm_lt_path'):
             metadata['smlt_path'] = '_'.join(rlz.sm_lt_path)
-            metadata['gsimlt_path'] = rlz.gsim_rlz.uid
+            metadata['gsimlt_path'] = rlz.gsim_rlz.pid
         metadata['imt'] = imt.name
         metadata['investigation_time'] = oq.investigation_time
         metadata['lon'] = lon
@@ -810,25 +556,30 @@ def export_disagg_csv(ekey, dstore):
         metadata['Lon'] = attrs['lon_bin_edges']
         metadata['Lat'] = attrs['lat_bin_edges']
         metadata['Eps'] = attrs['eps_bin_edges']
-        metadata['TRT'] = trts
-        data = {}
-        for poe, label in zip(poes, disagg_outputs):
-            tup = tuple(label.split('_'))
+        metadata['TRT'] = attrs['trt_bin_edges']
+        # example: key = 'rlz-0-PGA-sid-0-poe-0'
+        poe_id = int(key.rsplit('-', 1)[1])
+        for label, dset in sorted(group[key].items()):
+            header = label.lower().split('_') + ['poe']
+            com = {key: value for key, value in metadata.items()
+                   if value is not None and key not in skip_keys}
+            com.update(dic)
             fname = dstore.export_path(key + '_%s.csv' % label)
-            data[tup] = poe, iml, matrix[label].value, fname
+            values = extract(dstore, ex % (label, imt, site_id, poe_id))
+            writers.write_csv(fname, values, header=header, comment=com,
+                              fmt='%.5E')
             fnames.append(fname)
-        save_disagg_to_csv(metadata, data)
     return fnames
 
 
 @export.add(('disagg_by_src', 'csv'))
 def export_disagg_by_src_csv(ekey, dstore):
     paths = []
-    srcdata = dstore['disagg_by_grp'].value
+    srcdata = dstore['disagg_by_grp'][()]
     header = ['source_id', 'poe']
     by_poe = operator.itemgetter(1)
     for name in dstore['disagg_by_src']:
-        probs = dstore['disagg_by_src/' + name].value
+        probs = dstore['disagg_by_src/' + name][()]
         ok = probs > 0
         src = srcdata[ok]
         data = [header] + sorted(zip(add_quotes(src['grp_name']), probs[ok]),
@@ -841,23 +592,17 @@ def export_disagg_by_src_csv(ekey, dstore):
 
 @export.add(('realizations', 'csv'))
 def export_realizations(ekey, dstore):
-    data = dstore['csm_info'].rlzs
+    data = extract(dstore, 'realizations').array
     path = dstore.export_path('realizations.csv')
     writers.write_csv(path, data, fmt='%.7e')
     return [path]
 
 
-@export.add(('sourcegroups', 'csv'))
-def export_sourcegroups(ekey, dstore):
-    csm_info = dstore['csm_info']
-    data = [['grp_id', 'trt', 'eff_ruptures']]
-    for i, sm in enumerate(csm_info.source_models):
-        for src_group in sm.src_groups:
-            trt = source.capitalize(src_group.trt)
-            er = src_group.eff_ruptures
-            data.append((src_group.id, trt, er))
-    path = dstore.export_path('sourcegroups.csv')
-    writers.write_csv(path, data, fmt='%s')
+@export.add(('events', 'csv'))
+def export_events(ekey, dstore):
+    events = dstore['events'][()]
+    path = dstore.export_path('events.csv')
+    writers.write_csv(path, events, fmt='%s', renamedict=dict(id='event_id'))
     return [path]
 
 
