@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2019 GEM Foundation
+# Copyright (C) 2015-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -20,10 +20,11 @@
 """
 Utilities to build a report writer generating a .rst report for a calculation
 """
-from openquake.baselib.python3compat import decode
 import os
 import sys
-from openquake.baselib.python3compat import encode
+import logging
+from openquake.baselib import parallel
+from openquake.baselib.python3compat import encode, decode
 from openquake.commonlib import readinput, logs
 from openquake.calculators import views
 
@@ -39,20 +40,17 @@ class ReportWriter(object):
     title = {
         'params': 'Parameters',
         'inputs': 'Input files',
-        'csm_info': 'Composite source model',
-        'dupl_sources': 'Duplicated sources',
+        'full_lt': 'Composite source model',
         'required_params_per_trt':
         'Required parameters per tectonic region type',
-        'ruptures_per_trt': 'Number of ruptures per tectonic region type',
         'ruptures_events': 'Specific information for event based',
-        'rlzs_assoc': 'Realizations per (GRP, GSIM)',
         'job_info': 'Data transfer',
         'biggest_ebr_gmf': 'Maximum memory allocated for the GMFs',
         'avglosses_data_transfer': 'Estimated data transfer for the avglosses',
         'exposure_info': 'Exposure model',
         'slow_sources': 'Slowest sources',
-        'task_hazard:0': 'Fastest task',
-        'task_hazard:-1': 'Slowest task',
+        'task:classical_split_filter:0': 'Fastest task',
+        'task:classical_split_filter:-1': 'Slowest task',
         'task_info': 'Information about the tasks',
         'times_by_source_class': 'Computation times by source typology',
         'performance': 'Slowest operations',
@@ -63,7 +61,7 @@ class ReportWriter(object):
         self.oq = oq = dstore['oqparam']
         self.text = (decode(oq.description) + '\n' + '=' * len(oq.description))
         try:
-            num_rlzs = dstore['csm_info'].get_num_rlzs()
+            num_rlzs = dstore['full_lt'].get_num_rlzs()
         except KeyError:
             num_rlzs = '?'
         versions = sorted(dstore['/'].attrs.items())
@@ -87,14 +85,11 @@ class ReportWriter(object):
         oq, ds = self.oq, self.dstore
         for name in ('params', 'inputs'):
             self.add(name)
-        if 'csm_info' in ds:
-            self.add('csm_info')
-            if ds['csm_info'].source_models[0].name != 'scenario':
+        if 'full_lt' in ds:
+            self.add('full_lt')
+            if ds['full_lt'].sm_rlzs[0].name != 'scenario':
                 # required_params_per_trt makes no sense for GMFs from file
                 self.add('required_params_per_trt')
-            self.add('rlzs_assoc', ds['csm_info'].get_rlzs_assoc())
-        if 'csm_info' in ds:
-            self.add('ruptures_per_trt')
         if 'rup_data' in ds:
             self.add('ruptures_events')
         if oq.calculation_mode in ('event_based_risk',):
@@ -104,13 +99,12 @@ class ReportWriter(object):
         if 'source_info' in ds:
             self.add('slow_sources')
             self.add('times_by_source_class')
-            self.add('dupl_sources')
         if 'task_info' in ds:
             self.add('task_info')
-            tasks = set(ds['task_info'])
-            if 'classical' in tasks:
-                self.add('task_hazard:0')
-                self.add('task_hazard:-1')
+            tasks = set(ds['task_info']['taskname'])
+            if 'classical_split_filter' in tasks:
+                self.add('task:classical_split_filter:0')
+                self.add('task:classical_split_filter:-1')
             self.add('job_info')
         if 'performance_data' in ds:
             self.add('performance')
@@ -134,7 +128,7 @@ def build_report(job_ini, output_dir=None):
     """
     calc_id = logs.init()
     oq = readinput.get_oqparam(job_ini)
-    if oq.calculation_mode == 'classical':
+    if 'source_model_logic_tree' in oq.inputs:
         oq.calculation_mode = 'preclassical'
     oq.ground_motion_fields = False
     output_dir = output_dir or os.path.dirname(job_ini)
@@ -147,8 +141,12 @@ def build_report(job_ini, output_dir=None):
     calc.pre_execute()
     if oq.calculation_mode == 'preclassical':
         calc.execute()
+    logging.info('Making the .rst report')
     rw = ReportWriter(calc.datastore)
-    rw.make_report()
+    try:
+        rw.make_report()
+    finally:
+        parallel.Starmap.shutdown()
     report = (os.path.join(output_dir, 'report.rst') if output_dir
               else calc.datastore.export_path('report.rst'))
     try:
