@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2019 GEM Foundation
+# Copyright (C) 2018-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -18,19 +18,22 @@
 import os
 import copy
 import math
+import logging
+import pickle
 from datetime import datetime
 import numpy
 import h5py
-from openquake.baselib.general import random_filter
+import zlib
+
+from openquake.baselib.general import random_filter, AccumDict, cached_property
 from openquake.hazardlib.calc.filters import SourceFilter
 from openquake.hazardlib.source.base import BaseSeismicSource
 from openquake.hazardlib.geo.geodetic import min_geodetic_distance
 from openquake.hazardlib.geo.surface.planar import PlanarSurface
 from openquake.hazardlib.geo.surface.multi import MultiSurface
-from openquake.hazardlib.geo.nodalplane import NodalPlane
+from openquake.hazardlib.geo.utils import KM_TO_DEGREES, angular_distance
 from openquake.hazardlib.source.point import PointSource
 from openquake.hazardlib.mfd import EvenlyDiscretizedMFD
-from openquake.hazardlib.pmf import PMF
 from openquake.hazardlib.tom import PoissonTOM
 from openquake.hazardlib.scalerel.wc1994 import WC1994
 from openquake.hazardlib.source.rupture import ParametricProbabilisticRupture
@@ -39,25 +42,11 @@ from openquake.hazardlib import valid
 from openquake.hazardlib.sourceconverter import SourceConverter
 
 DEFAULT_TRT = "Active Shallow Crust"
-RUPTURES_PER_BLOCK = 200  # decided by MS
-HDD = PMF([(0.2, 3.0), (0.6, 6.0), (0.2, 9.0)])
-NPD = PMF([(0.15, NodalPlane(0.0, 90.0, 0.0)),
-           (0.15, NodalPlane(45.0, 90.0, 0.0)),
-           (0.15, NodalPlane(90.0, 90.0, 0.0)),
-           (0.15, NodalPlane(135.0, 90.0, 0.0)),
-           (0.05, NodalPlane(0.0, 45.0, 90.)),
-           (0.05, NodalPlane(45.0, 45.0, 90.)),
-           (0.05, NodalPlane(90.0, 45.0, 90.)),
-           (0.05, NodalPlane(135.0, 45.0, 90.)),
-           (0.05, NodalPlane(180.0, 45.0, 90.)),
-           (0.05, NodalPlane(225.0, 45.0, 90.)),
-           (0.05, NodalPlane(270.0, 45.0, 90.)),
-           (0.05, NodalPlane(325.0, 45.0, 90.))])
 
 
 def convert_UCERFSource(self, node):
     """
-    Converts the Ucerf Source node into an SES Control object
+    Converts the node into an UCERFSource object
     """
     dirname = os.path.dirname(self.fname)  # where the source_model_file is
     source_file = os.path.join(dirname, node["filename"])
@@ -113,11 +102,11 @@ class UcerfFilter(SourceFilter):
         for src in srcs:
             if hasattr(src, 'start'):  # fault sources
                 src.src_filter = self  # hack: needed for .iter_ruptures
+                src.all_ridx = src.get_ridx()
                 ridx = set()
-                for idx in range(src.start, src.stop):
-                    ridx.update(src.get_ridx(idx))
-                mag = src.mags[src.start:src.stop].max()
-                src.indices = self.get_indices(src, ridx, mag)
+                for arr in src.all_ridx:
+                    ridx.update(arr)
+                src.indices = self.get_indices(src, ridx, src.mags.max())
                 if len(src.indices):
                     yield src
             else:  # background sources
@@ -172,11 +161,13 @@ class UCERFSource(BaseSeismicSource):
     code = b'U'
     MODIFICATIONS = set()
     tectonic_region_type = DEFAULT_TRT
-    RUPTURE_WEIGHT = 1  # not very heavy
+    ruptures_per_block = None  # overridden by the source_reader
+    checksum = 0
+    _wkt = ''
 
     def __init__(
             self, source_file, investigation_time, start_date, min_mag,
-            npd=NPD, hdd=HDD, aspect=1.5, upper_seismogenic_depth=0.0,
+            npd, hdd, aspect=1.5, upper_seismogenic_depth=0.0,
             lower_seismogenic_depth=15.0, msr=WC1994(), mesh_spacing=1.0,
             trt="Active Shallow Crust", integration_distance=1000):
         assert os.path.exists(source_file), source_file
@@ -194,9 +185,8 @@ class UCERFSource(BaseSeismicSource):
         self.msr = msr
         self.mesh_spacing = mesh_spacing
         self.tectonic_region_type = trt
-        self.stop = 0
-        self.start = -1
-        self.orig = None  # set by .new()
+        self.stop = None
+        self.start = None
 
     @property
     def num_ruptures(self):
@@ -206,32 +196,26 @@ class UCERFSource(BaseSeismicSource):
     def num_ruptures(self, value):  # hack to make the sourceconverter happy
         pass
 
-    @property
+    @cached_property
     def mags(self):
         # read from FM0_0/MEANFS/MEANMSR/Magnitude
-        if hasattr(self.orig, '_mags'):
-            return self.orig._mags
         with h5py.File(self.source_file, "r") as hdf5:
-            self.orig._mags = hdf5[self.idx_set["mag"]].value
-            return self.orig._mags
+            return hdf5[self.idx_set["mag"]][self.start: self.stop]
 
-    @property
+    @cached_property
     def rate(self):
         # read from FM0_0/MEANFS/MEANMSR/Rates/MeanRates
-        if hasattr(self.orig, '_rate'):
-            return self.orig._rate
         with h5py.File(self.source_file, "r") as hdf5:
-            self.orig._rate = hdf5[self.idx_set["rate"]].value
-            return self.orig._rate
+            return hdf5[self.idx_set["rate"]][self.start: self.stop]
 
-    @property
+    @cached_property
     def rake(self):
         # read from FM0_0/MEANFS/Rake
-        if hasattr(self.orig, '_rake'):
-            return self.orig._rake
         with h5py.File(self.source_file, "r") as hdf5:
-            self.orig._rake = hdf5[self.idx_set["rake"]].value
-            return self.orig._rake
+            return hdf5[self.idx_set["rake"]][self.start:self.stop]
+
+    def wkt(self):
+        return ''
 
     def count_ruptures(self):
         """
@@ -247,8 +231,7 @@ class UCERFSource(BaseSeismicSource):
         :returns: a new UCERFSource associated to the branch_id
         """
         new = copy.copy(self)
-        new.orig = new
-        new.src_group_id = grp_id
+        new.grp_id = grp_id
         new.source_id = branch_id
         new.idx_set = build_idx_set(branch_id, self.start_date)
         with h5py.File(self.source_file, "r") as hdf5:
@@ -268,9 +251,11 @@ class UCERFSource(BaseSeismicSource):
         """
         return PoissonTOM(self.inv_time)
 
-    def get_ridx(self, iloc):
+    def get_ridx(self, iloc=None):
         """List of rupture indices for the given iloc"""
         with h5py.File(self.source_file, "r") as hdf5:
+            if iloc is None:
+                iloc = slice(self.start, self.stop)
             return hdf5[self.idx_set["geol"] + "/RuptureIndex"][iloc]
 
     def get_centroids(self, ridx):
@@ -281,7 +266,7 @@ class UCERFSource(BaseSeismicSource):
         with h5py.File(self.source_file, "r") as hdf5:
             for idx in ridx:
                 trace = "{:s}/{:s}".format(self.idx_set["sec"], str(idx))
-                centroids.append(hdf5[trace + "/Centroids"].value)
+                centroids.append(hdf5[trace + "/Centroids"][()])
         return numpy.concatenate(centroids)
 
     def gen_trace_planes(self, ridx):
@@ -294,40 +279,58 @@ class UCERFSource(BaseSeismicSource):
                 plane = hdf5[trace + "/RupturePlanes"][:].astype("float64")
                 yield trace, plane
 
-    def get_background_sids(self, src_filter):
+    def get_bounding_box(self, maxdist):
+        """
+        :returns: min_lon, min_lat, max_lon, max_lat
+        """
+        with h5py.File(self.source_file, 'r') as hdf5:
+            locations = hdf5["Grid/Locations"][()]
+        lons, lats = locations[:, 0], locations[:, 1]
+        bbox = lons.min(), lats.min(), lons.max(), lats.max()
+        a1 = min(maxdist * KM_TO_DEGREES, 90)
+        a2 = angular_distance(maxdist, bbox[1], bbox[3])
+        return bbox[0] - a2, bbox[1] - a1, bbox[2] + a2, bbox[3] + a1
+
+    def get_background_sids(self):
         """
         We can apply the filtering of the background sites as a pre-processing
         step - this is done here rather than in the sampling of the ruptures
         themselves
         """
         branch_key = self.idx_set["grid_key"]
-        idist = src_filter.integration_distance(DEFAULT_TRT)
         with h5py.File(self.source_file, 'r') as hdf5:
-            bg_locations = hdf5["Grid/Locations"].value
+            bg_locations = hdf5["Grid/Locations"][()]
+            if hasattr(self, 'src_filter'):
+                # in event based
+                idist = self.src_filter.integration_distance(DEFAULT_TRT)
+            else:
+                # in classical
+                return range(len(bg_locations))
             distances = min_geodetic_distance(
-                src_filter.sitecol.xyz,
+                self.src_filter.sitecol.xyz,
                 (bg_locations[:, 0], bg_locations[:, 1]))
             # Add buffer equal to half of length of median area from Mmax
             mmax_areas = self.msr.get_median_area(
-                hdf5["/".join(["Grid", branch_key, "MMax"])].value, 0.0)
+                hdf5["/".join(["Grid", branch_key, "MMax"])][()], 0.0)
             # for instance hdf5['Grid/FM0_0_MEANFS_MEANMSR/MMax']
             mmax_lengths = numpy.sqrt(mmax_areas / self.aspect)
             ok = distances <= (0.5 * mmax_lengths + idist)
             # get list of indices from array of booleans
             return numpy.where(ok)[0].tolist()
 
-    def get_ucerf_rupture(self, iloc, src_filter):
+    def get_ucerf_rupture(self, iloc):
         """
         :param iloc:
             Location of the rupture plane in the hdf5 file
-        :param src_filter:
-            Sites for consideration and maximum distance
         """
         trt = self.tectonic_region_type
-        ridx = self.get_ridx(iloc)
-        mag = self.orig.mags[iloc]
+        if hasattr(self, 'all_ridx'):  # already computed by the UcerfFilter
+            ridx = self.all_ridx[iloc - self.start]
+        else:
+            ridx = self.get_ridx(iloc)
+        mag = self.mags[iloc - self.start]
         surface_set = []
-        indices = src_filter.get_indices(self, ridx, mag)
+        indices = self.src_filter.get_indices(self, ridx, mag)
         if len(indices) == 0:
             return None
         for trace, plane in self.gen_trace_planes(ridx):
@@ -350,57 +353,54 @@ class UCERFSource(BaseSeismicSource):
                                      bottom_right, bottom_left)
 
         rupture = ParametricProbabilisticRupture(
-            mag, self.orig.rake[iloc], trt,
+            mag, self.rake[iloc - self.start], trt,
             surface_set[len(surface_set) // 2].get_middle_point(),
-            MultiSurface(surface_set), self.orig.rate[iloc], self.tom)
+            MultiSurface(surface_set), self.rate[iloc - self.start], self.tom)
 
         return rupture
 
-    def iter_ruptures(self):
+    def iter_ruptures(self, **kwargs):
         """
         Yield ruptures for the current set of indices
         """
-        assert self.orig, '%s is not fully initialized' % self
         for ridx in range(self.start, self.stop):
-            if self.orig.rate[ridx]:  # ruptures may have have zero rate
-                rup = self.get_ucerf_rupture(ridx, self.src_filter)
+            if self.rate[ridx - self.start]:  # may have have zero rate
+                rup = self.get_ucerf_rupture(ridx)
                 if rup:
                     yield rup
 
+    # called upfront, before classical_split_filter
     def __iter__(self):
-        assert self.orig, '%s is not fully initialized' % self
-        start = self.start
-        stop = self.stop
-        while stop > start:
+        if self.stop - self.start <= self.ruptures_per_block:  # already split
+            yield self
+            return
+        for start in range(self.start, self.stop, self.ruptures_per_block):
+            stop = min(start + self.ruptures_per_block, self.stop)
             new = copy.copy(self)
             new.id = self.id
-            new.orig = self.orig
+            new.source_id = '%s:%d-%d' % (self.source_id, start, stop)
             new.start = start
-            new.stop = min(start + RUPTURES_PER_BLOCK, stop)
-            start += RUPTURES_PER_BLOCK
+            new.stop = stop
             yield new
 
     def __repr__(self):
-        return '<%s %s[%d:%d]>' % (self.__class__.__name__, self.source_id,
-                                   self.start, self.stop)
+        return '<%s %s>' % (self.__class__.__name__, self.source_id)
 
-    def get_background_sources(self, src_filter, sample_factor=None):
+    def get_background_sources(self, sample_factor=None):
         """
         Turn the background model of a given branch into a set of point sources
 
-        :param src_filter:
-            SourceFilter instance
         :param sample_factor:
             Used to reduce the sources if OQ_SAMPLE_SOURCES is set
         """
-        background_sids = self.get_background_sids(src_filter)
+        background_sids = self.get_background_sids()
         if sample_factor is not None:  # hack for use in the mosaic
             background_sids = random_filter(
                 background_sids, sample_factor, seed=42)
         with h5py.File(self.source_file, "r") as hdf5:
             grid_loc = "/".join(["Grid", self.idx_set["grid_key"]])
             # for instance Grid/FM0_0_MEANFS_MEANMSR_MeanRates
-            mags = hdf5[grid_loc + "/Magnitude"].value
+            mags = hdf5[grid_loc + "/Magnitude"][()]
             mmax = hdf5[grid_loc + "/MMax"][background_sids]
             rates = hdf5[grid_loc + "/RateArray"][background_sids, :]
             locations = hdf5["Grid/Locations"][background_sids, :]
@@ -420,8 +420,10 @@ class UCERFSource(BaseSeismicSource):
                     self.usd, self.lsd,
                     Point(locations[i, 0], locations[i, 1]),
                     self.npd, self.hdd)
+                ps.checksum = zlib.adler32(pickle.dumps(vars(ps), protocol=4))
+                ps._wkt = ps.wkt()
                 ps.id = self.id
-                ps.src_group_id = self.src_group_id
+                ps.grp_id = self.grp_id
                 ps.num_ruptures = ps.count_ruptures()
                 sources.append(ps)
         return sources
@@ -429,6 +431,107 @@ class UCERFSource(BaseSeismicSource):
     def get_one_rupture(self):
         raise ValueError('Unsupported option')
 
+    def generate_event_set(self, background_sids, eff_num_ses):
+        """
+        Generates the event set corresponding to a particular branch
+        """
+        # get rates from file
+        with h5py.File(self.source_file, 'r') as hdf5:
+            occurrences = self.tom.sample_number_of_occurrences(
+                self.rate * eff_num_ses, self.serial)
+            indices, = numpy.where(occurrences)
+            logging.debug(
+                'Considering "%s", %d ruptures', self.source_id, len(indices))
+
+            # get ruptures from the indices
+            ruptures = []
+            rupture_occ = []
+            for iloc, n_occ in zip(indices, occurrences[indices]):
+                ucerf_rup = self.get_ucerf_rupture(iloc)
+                if ucerf_rup:
+                    ruptures.append(ucerf_rup)
+                    rupture_occ.append(n_occ)
+
+            # sample background sources
+            background_ruptures, background_n_occ = sample_background_model(
+                hdf5, self.idx_set["grid_key"], self.tom, eff_num_ses,
+                self.serial, background_sids, self.min_mag, self.npd,
+                self.hdd, self.usd, self.lsd, self.msr, self.aspect,
+                self.tectonic_region_type)
+            ruptures.extend(background_ruptures)
+            rupture_occ.extend(background_n_occ)
+        return ruptures, rupture_occ
+
+    def _sample_ruptures(self, eff_num_ses):
+        background_sids = self.get_background_sids()
+        n_occ = AccumDict(accum=0)
+        rups, occs = self.generate_event_set(background_sids, eff_num_ses)
+        for rup, occ in zip(rups, occs):
+            n_occ[rup] += occ
+        yield from n_occ.items()
+
+
+def sample_background_model(
+        hdf5, branch_key, tom, eff_num_ses, seed, filter_idx, min_mag, npd,
+        hdd, upper_seismogenic_depth, lower_seismogenic_depth, msr=WC1994(),
+        aspect=1.5, trt=DEFAULT_TRT):
+    """
+    Generates a rupture set from a sample of the background model
+
+    :param branch_key:
+        Key to indicate the branch for selecting the background model
+    :param tom:
+        Temporal occurrence model as instance of :class:
+        openquake.hazardlib.tom.TOM
+    :param seed:
+        Random seed to use in the call to tom.sample_number_of_occurrences
+    :param filter_idx:
+        Sites for consideration (can be None!)
+    :param float min_mag:
+        Minimim magnitude for consideration of background sources
+    :param npd:
+        Nodal plane distribution as instance of :class:
+        openquake.hazardlib.pmf.PMF
+    :param hdd:
+        Hypocentral depth distribution as instance of :class:
+        openquake.hazardlib.pmf.PMF
+    :param float aspect:
+        Aspect ratio
+    :param float upper_seismogenic_depth:
+        Upper seismogenic depth (km)
+    :param float lower_seismogenic_depth:
+        Lower seismogenic depth (km)
+    :param msr:
+        Magnitude scaling relation
+    :param float integration_distance:
+        Maximum distance from rupture to site for consideration
+    """
+    bg_magnitudes = hdf5["/".join(["Grid", branch_key, "Magnitude"])][()]
+    # Select magnitudes above the minimum magnitudes
+    mag_idx = bg_magnitudes >= min_mag
+    mags = bg_magnitudes[mag_idx]
+    rates = hdf5["/".join(["Grid", branch_key, "RateArray"])][filter_idx, :]
+    rates = rates[:, mag_idx]
+    valid_locs = hdf5["Grid/Locations"][filter_idx, :]
+    # Sample remaining rates
+    sampler = tom.sample_number_of_occurrences(rates * eff_num_ses, seed)
+    background_ruptures = []
+    background_n_occ = []
+    for i, mag in enumerate(mags):
+        rate_idx = numpy.where(sampler[:, i])[0]
+        rate_cnt = sampler[rate_idx, i]
+        occurrence = rates[rate_idx, i]
+        locations = valid_locs[rate_idx, :]
+        ruptures = generate_background_ruptures(
+            tom, locations, occurrence,
+            mag, npd, hdd, upper_seismogenic_depth,
+            lower_seismogenic_depth, msr, aspect, trt)
+        background_ruptures.extend(ruptures)
+        background_n_occ.extend(rate_cnt.tolist())
+    return background_ruptures, background_n_occ
+
+# #################################################################### #
+
 
 def build_idx_set(branch_id, start_date):
     """
