@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2019 GEM Foundation
+# Copyright (C) 2015-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -20,18 +20,17 @@ import os.path
 import numbers
 import operator
 import functools
-import itertools
 import collections
 import numpy
 
 from openquake.baselib.general import (
-    humansize, groupby, countby, AccumDict, CallableDict,
-    get_array, group_array)
-from openquake.baselib.performance import perf_dt
-from openquake.baselib.python3compat import decode
+    humansize, countby, AccumDict, CallableDict,
+    get_array, group_array, fast_agg, fast_agg3)
+from openquake.baselib.performance import performance_view
+from openquake.baselib.python3compat import encode, decode
 from openquake.hazardlib import valid
 from openquake.hazardlib.gsim.base import ContextMaker
-from openquake.commonlib import util, source, calc
+from openquake.commonlib import util, calc
 from openquake.commonlib.writers import build_header, scientificformat
 from openquake.calculators import getters
 from openquake.calculators.extract import extract
@@ -58,7 +57,7 @@ def form(value):
     >>> form(0.0001)
     '1.000E-04'
     >>> form(1003.4)
-    '1,003'
+    '1_003'
     >>> form(103.4)
     '103'
     >>> form(9.3)
@@ -74,7 +73,7 @@ def form(value):
         elif value < 10 and isinstance(value, FLOAT):
             return '%.5f' % value
         elif value > 1000:
-            return '{:,d}'.format(int(round(value)))
+            return '{:_d}'.format(int(round(value)))
         elif numpy.isnan(value):
             return 'NaN'
         else:  # in the range 10-1000
@@ -141,40 +140,13 @@ def rst_table(data, header=None, fmt=None):
     return '\n'.join(lines)
 
 
-def sum_tbl(tbl, kfield, vfields):
-    """
-    Aggregate a composite array and compute the totals on a given key.
-
-    >>> dt = numpy.dtype([('name', (bytes, 10)), ('value', int)])
-    >>> tbl = numpy.array([('a', 1), ('a', 2), ('b', 3)], dt)
-    >>> sum_tbl(tbl, 'name', ['value'])['value']
-    array([3, 3])
-    """
-    pairs = [(n, tbl.dtype[n]) for n in [kfield] + vfields]
-    dt = numpy.dtype(pairs + [('counts', int)])
-
-    def sum_all(group):
-        vals = numpy.zeros(1, dt)[0]
-        for rec in group:
-            for vfield in vfields:
-                vals[vfield] += rec[vfield]
-            vals['counts'] += 1
-        vals[kfield] = rec[kfield]
-        return vals
-    rows = groupby(tbl, operator.itemgetter(kfield), sum_all).values()
-    array = numpy.zeros(len(rows), dt)
-    for i, row in enumerate(rows):
-        for j, name in enumerate(dt.names):
-            array[i][name] = row[j]
-    return array
-
-
 @view.add('times_by_source_class')
 def view_times_by_source_class(token, dstore):
     """
     Returns the calculation times depending on the source typology
     """
-    totals = sum_tbl(dstore['source_info'], 'code', ['calc_time'])
+    totals = fast_agg3(dstore['source_info']['code', 'calc_time'],
+                       'code', ['calc_time'])
     return rst_table(totals)
 
 
@@ -183,9 +155,29 @@ def view_slow_sources(token, dstore, maxrows=20):
     """
     Returns the slowest sources
     """
-    info = dstore['source_info'].value
+    info = dstore['source_info']['source_id', 'code', 'multiplicity',
+                                 'calc_time', 'num_sites', 'eff_ruptures']
+    info = info[info['eff_ruptures'] > 0]
     info.sort(order='calc_time')
-    return rst_table(info[::-1][:maxrows])
+    data = numpy.zeros(len(info), [(nam, object) for nam in info.dtype.names])
+    for name in info.dtype.names:
+        data[name] = info[name]
+    data['num_sites'] /= data['eff_ruptures']
+    return rst_table(data[::-1][:maxrows])
+
+
+@view.add('slow_ruptures')
+def view_slow_ruptures(token, dstore, maxrows=25):
+    """
+    Show the slowest ruptures
+    """
+    fields = ['code', 'n_occ', 'mag', 'grp_id']
+    rups = dstore['ruptures'][()][fields]
+    time = dstore['gmf_data/time_by_rup'][()]
+    arr = util.compose_arrays(rups, time)
+    arr = arr[arr['nsites'] > 0]
+    arr.sort(order='time')
+    return rst_table(arr[-maxrows:])
 
 
 @view.add('contents')
@@ -204,45 +196,27 @@ def view_contents(token, dstore):
     return rst_table(rows, header=(desc, '')) + total
 
 
-@view.add('csm_info')
-def view_csm_info(token, dstore):
-    csm_info = dstore['csm_info']
-    header = ['smlt_path', 'weight', 'gsim_logic_tree', 'num_realizations']
+@view.add('full_lt')
+def view_full_lt(token, dstore):
+    full_lt = dstore['full_lt']
+    if full_lt.num_samples == 0:
+        num_rlzs = full_lt.gsim_lt.get_num_paths()
+    header = ['smlt_path', 'weight', 'num_realizations']
     rows = []
-    for sm in csm_info.source_models:
-        kind, num_rlzs = csm_info.classify_gsim_lt(sm)
-        row = ('_'.join(sm.path), sm.weight, kind, num_rlzs)
+    for sm in full_lt.sm_rlzs:
+        if full_lt.num_samples:
+            num_rlzs = sm.samples
+        row = ('_'.join(sm.lt_path), sm.weight, num_rlzs)
         rows.append(row)
     return rst_table(rows, header)
 
 
-@view.add('ruptures_per_trt')
-def view_ruptures_per_trt(token, dstore):
-    tbl = []
-    header = ('source_model grp_id trt eff_ruptures tot_ruptures'.split())
-    num_trts = 0
-    eff_ruptures = 0
-    tot_ruptures = 0
-    csm_info = dstore['csm_info']
-    for i, sm in enumerate(csm_info.source_models):
-        for src_group in sm.src_groups:
-            trt = source.capitalize(src_group.trt)
-            er = src_group.eff_ruptures
-            if er:
-                num_trts += 1
-                eff_ruptures += er
-                tbl.append(
-                    (sm.name, src_group.id, trt, er, src_group.tot_ruptures))
-            tot_ruptures += src_group.tot_ruptures
-    rows = [('#TRT models', num_trts),
-            ('#eff_ruptures', eff_ruptures),
-            ('#tot_ruptures', tot_ruptures),
-            ('#tot_weight', csm_info.tot_weight)]
-    if len(tbl) > 1:
-        summary = '\n\n' + rst_table(rows)
-    else:
-        summary = ''
-    return rst_table(tbl, header=header) + summary
+@view.add('eff_ruptures')
+def view_eff_ruptures(token, dstore):
+    header = ['num_ruptures', 'eff_ruptures']
+    info = dstore['source_info']['num_ruptures', 'eff_ruptures']
+    return rst_table([[info['num_ruptures'].sum(),
+                       info['eff_ruptures'].sum()]], header)
 
 
 @view.add('short_source_info')
@@ -258,6 +232,7 @@ def view_params(token, dstore):
               'ses_per_logic_tree_path', 'truncation_level',
               'rupture_mesh_spacing', 'complex_fault_mesh_spacing',
               'width_of_mfd_bin', 'area_source_discretization',
+              'pointsource_distance',
               'ground_motion_correlation_model', 'minimum_intensity',
               'random_seed', 'master_seed', 'ses_seed']
     if 'risk' in oq.calculation_mode:
@@ -311,15 +286,13 @@ def view_job_info(token, dstore):
     to the workers and back in a classical calculation.
     """
     data = [['task', 'sent', 'received']]
-    for task in dstore['task_info']:
-        dset = dstore['task_info/' + task]
-        if 'argnames' in dset.attrs:
-            argnames = dset.attrs['argnames'].split()
-            totsent = dset.attrs['sent']
-            sent = ['%s=%s' % (a, humansize(s))
-                    for s, a in sorted(zip(totsent, argnames), reverse=True)]
-            recv = dset['received'].sum()
-            data.append((task, ' '.join(sent), humansize(recv)))
+    task_info = dstore['task_info'][()]
+    task_sent = ast.literal_eval(dstore['task_sent'][()])
+    for task, dic in task_sent.items():
+        sent = sorted(dic.items(), key=operator.itemgetter(1), reverse=True)
+        sent = ['%s=%s' % (k, humansize(v)) for k, v in sent[:3]]
+        recv = get_array(task_info, taskname=encode(task))['received'].sum()
+        data.append((task, ' '.join(sent), humansize(recv)))
     return rst_table(data)
 
 
@@ -331,7 +304,7 @@ def avglosses_data_transfer(token, dstore):
     """
     oq = dstore['oqparam']
     N = len(dstore['assetcol'])
-    R = dstore['csm_info'].get_num_rlzs()
+    R = dstore['full_lt'].get_num_rlzs()
     L = len(dstore.get_attr('risk_model', 'loss_types'))
     ct = oq.concurrent_tasks
     size_bytes = N * R * L * 8 * ct  # 8 byte floats
@@ -340,17 +313,6 @@ def avglosses_data_transfer(token, dstore):
         '8 bytes x %d tasks = %s' % (N, R, L, ct, humansize(size_bytes)))
 
 
-@view.add('ebr_data_transfer')
-def ebr_data_transfer(token, dstore):
-    """
-    Display the data transferred in an event based risk calculation
-    """
-    attrs = dstore['losses_by_event'].attrs
-    sent = humansize(attrs['sent'])
-    received = humansize(attrs['tot_received'])
-    return 'Event Based Risk: sent %s, received %s' % (sent, received)
-
-
 # for scenario_risk
 @view.add('totlosses')
 def view_totlosses(token, dstore):
@@ -367,13 +329,12 @@ def view_totlosses(token, dstore):
 
 # for event based risk and ebrisk
 def portfolio_loss(dstore):
-    R = dstore['csm_info'].get_num_rlzs()
-    array = dstore['losses_by_event'].value
-    L = array.dtype['loss'].shape[0]  # loss has shape L, T...
+    R = dstore['full_lt'].get_num_rlzs()
+    array = dstore['losses_by_event'][()]
+    L, = array.dtype['loss'].shape  # loss has shape L
     data = numpy.zeros((R, L), F32)
     for row in array:
-        for lti in range(L):
-            data[row['rlzi'], lti] += row['loss'][lti].sum()
+        data[row['rlzi']] += row['loss']
     return data
 
 
@@ -387,7 +348,7 @@ def view_portfolio_losses(token, dstore):
     loss_dt = oq.loss_dt()
     data = portfolio_loss(dstore).view(loss_dt)[:, 0]
     rlzids = [str(r) for r in range(len(data))]
-    array = util.compose_arrays(numpy.array(rlzids), data, 'rlz')
+    array = util.compose_arrays(numpy.array(rlzids), data, 'rlz_id')
     # this is very sensitive to rounding errors, so I am using a low precision
     return rst_table(array, fmt='%.5E')
 
@@ -433,13 +394,8 @@ def view_exposure_info(token, dstore):
     """
     assetcol = dstore['assetcol/array'][:]
     taxonomies = sorted(set(dstore['assetcol'].taxonomies))
-    cc = dstore['assetcol/cost_calculator']
-    ra_flag = ['relative', 'absolute']
     data = [('#assets', len(assetcol)),
-            ('#taxonomies', len(taxonomies)),
-            ('deductibile', ra_flag[int(cc.deduct_abs)]),
-            ('insurance_limit', ra_flag[int(cc.limit_abs)]),
-            ]
+            ('#taxonomies', len(taxonomies))]
     return rst_table(data) + '\n\n' + view_assets_by_site(token, dstore)
 
 
@@ -447,7 +403,7 @@ def view_exposure_info(token, dstore):
 def view_ruptures_events(token, dstore):
     num_ruptures = len(dstore['ruptures'])
     num_events = len(dstore['events'])
-    events_by_rlz = countby(dstore['events'].value, 'rlz')
+    events_by_rlz = countby(dstore['events'][()], 'rlz_id')
     mult = round(num_events / num_ruptures, 3)
     lst = [('Total number of ruptures', num_ruptures),
            ('Total number of events', num_events),
@@ -466,25 +422,6 @@ def view_fullreport(token, dstore):
     return ReportWriter(dstore).make_report()
 
 
-def performance_view(dstore):
-    """
-    Returns the performance view as a numpy array.
-    """
-    data = sorted(dstore['performance_data'], key=operator.itemgetter(0))
-    out = []
-    for operation, group in itertools.groupby(data, operator.itemgetter(0)):
-        counts = 0
-        time = 0
-        mem = 0
-        for _operation, time_sec, memory_mb, counts_ in group:
-            counts += counts_
-            time += time_sec
-            mem = max(mem, memory_mb)
-        out.append((operation, time, mem, counts))
-    out.sort(key=operator.itemgetter(1), reverse=True)  # sort by time
-    return numpy.array(out, perf_dt)
-
-
 @view.add('performance')
 def view_performance(token, dstore):
     """
@@ -510,7 +447,7 @@ def view_num_units(token, dstore):
     """
     Display the number of units by taxonomy
     """
-    taxo = dstore['assetcol/tagcol/taxonomy'].value
+    taxo = dstore['assetcol/tagcol/taxonomy'][()]
     counts = collections.Counter()
     for asset in dstore['assetcol']:
         counts[taxo[asset['taxonomy']]] += asset['number']
@@ -524,7 +461,7 @@ def view_assets_by_site(token, dstore):
     """
     Display statistical information about the distribution of the assets
     """
-    taxonomies = dstore['assetcol/tagcol/taxonomy'].value
+    taxonomies = dstore['assetcol/tagcol/taxonomy'][()]
     assets_by_site = dstore['assetcol'].assets_by_site()
     data = ['taxonomy mean stddev min max num_sites num_assets'.split()]
     num_assets = AccumDict()
@@ -545,10 +482,10 @@ def view_required_params_per_trt(token, dstore):
     """
     Display the parameters needed by each tectonic region type
     """
-    csm_info = dstore['csm_info']
+    full_lt = dstore['full_lt']
     tbl = []
-    for grp_id, trt in sorted(csm_info.grp_by("trt").items()):
-        gsims = csm_info.gsim_lt.get_gsims(trt)
+    for grp_id, trt in sorted(full_lt.trt_by_grp.items()):
+        gsims = full_lt.gsim_lt.get_gsims(trt)
         maker = ContextMaker(trt, gsims)
         distances = sorted(maker.REQUIRES_DISTANCES)
         siteparams = sorted(maker.REQUIRES_SITES_PARAMETERS)
@@ -569,18 +506,20 @@ def view_task_info(token, dstore):
 
       $ oq show task_info:classical
     """
+    task_info = dstore['task_info']
+    task_info.refresh()
     args = token.split(':')[1:]  # called as task_info:task_name
     if args:
         [task] = args
-        array = dstore['task_info/' + task].value
+        array = get_array(task_info[()], taskname=task.encode('utf8'))
         rduration = array['duration'] / array['weight']
         data = util.compose_arrays(rduration, array, 'rduration')
         data.sort(order='duration')
         return rst_table(data)
 
     data = ['operation-duration mean stddev min max outputs'.split()]
-    for task in dstore['task_info']:
-        val = dstore['task_info/' + task]['duration']
+    for task, arr in group_array(task_info[()], 'taskname').items():
+        val = arr['duration']
         if len(val):
             data.append(stats(task, val))
     if len(data) == 1:
@@ -596,55 +535,64 @@ def view_task_durations(token, dstore):
       $ oq show task_durations:classical
     """
     task = token.split(':')[1]  # called as task_duration:task_name
-    array = dstore['task_info/' + task]['duration']
+    array = get_array(dstore['task_info'][()], taskname=task)['duration']
     return '\n'.join(map(str, array))
 
 
-@view.add('task_hazard')
+@view.add('task')
 def view_task_hazard(token, dstore):
     """
     Display info about a given task. Here are a few examples of usage::
 
-     $ oq show task_hazard:0  # the fastest task
-     $ oq show task_hazard:-1  # the slowest task
+     $ oq show task:classical:0  # the fastest task
+     $ oq show task:classical:-1  # the slowest task
     """
-    tasks = set(dstore['task_info'])
-    if 'source_data' not in dstore:
-        return 'Missing source_data'
-    if 'classical_split_filter' in tasks:
-        data = dstore['task_info/classical_split_filter'].value
-    else:
-        data = dstore['task_info/compute_gmfs'].value
+    _, name, index = token.split(':')
+    if 'by_task' not in dstore:
+        return 'Missing by_task'
+    data = get_array(dstore['task_info'][()], taskname=encode(name))
+    if len(data) == 0:
+        raise RuntimeError('No task_info for %s' % name)
     data.sort(order='duration')
-    rec = data[int(token.split(':')[1])]
-    taskno = rec['taskno']
-    arr = get_array(dstore['source_data'].value, taskno=taskno)
-    st = [stats('nsites', arr['nsites']), stats('weight', arr['weight'])]
-    sources = dstore['task_sources'][taskno - 1].split()
-    srcs = set(decode(s).split(':', 1)[0] for s in sources)
-    res = 'taskno=%d, weight=%d, duration=%d s, sources="%s"\n\n' % (
-        taskno, rec['weight'], rec['duration'], ' '.join(sorted(srcs)))
-    return res + rst_table(st, header='variable mean stddev min max n'.split())
-
-
-@view.add('task_risk')
-def view_task_risk(token, dstore):
-    """
-    Display info about a given risk task. Here are a few examples of usage::
-
-     $ oq show task_risk:0  # the fastest task
-     $ oq show task_risk:-1  # the slowest task
-    """
-    [key] = dstore['task_info']
-    data = dstore['task_info/' + key].value
-    data.sort(order='duration')
-    rec = data[int(token.split(':')[1])]
-    taskno = rec['taskno']
-    res = 'taskno=%d, weight=%d, duration=%d s' % (
-        taskno, rec['weight'], rec['duration'])
+    rec = data[int(index)]
+    taskno = rec['task_no']
+    eff_ruptures = dstore['by_task/eff_ruptures'][taskno]
+    eff_sites = dstore['by_task/eff_sites'][taskno]
+    srcids = dstore['by_task/srcids'][taskno]
+    res = ('taskno=%d, eff_ruptures=%d, eff_sites=%d, duration=%d s\n'
+           'sources="%s"' % (taskno, eff_ruptures, eff_sites, rec['duration'],
+                             srcids))
     return res
 
 
+@view.add('task_ebrisk')
+def view_task_ebrisk(token, dstore):
+    """
+    Display info about ebrisk tasks:
+
+    $ oq show task_ebrisk:-1  # the slowest task
+    """
+    idx = int(token.split(':')[1])
+    task_info = get_array(dstore['task_info'][()], taskname=b'ebrisk')
+    task_info.sort(order='duration')
+    info = task_info[idx]
+    times = get_array(dstore['gmf_info'][()], task_no=info['task_no'])
+    extra = times[['nsites', 'gmfbytes', 'dt']]
+    ds = dstore.parent if dstore.parent else dstore
+    rups = ds['ruptures']['id', 'code', 'n_occ', 'mag'][times['rup_id']]
+    codeset = set('code_%d' % code for code in numpy.unique(rups['code']))
+    tbl = rst_table(util.compose_arrays(rups, extra))
+    codes = ['%s: %s' % it for it in ds.getitem('ruptures').attrs.items()
+             if it[0] in codeset]
+    msg = '%s\n%s\nHazard time for task %d: %d of %d s, ' % (
+        tbl, '\n'.join(codes), info['task_no'], extra['dt'].sum(),
+        info['duration'])
+    msg += 'gmfbytes=%s, w=%d' % (
+        humansize(extra['gmfbytes'].sum()),
+        (rups['n_occ'] * extra['nsites']).sum())
+    return msg
+
+
 @view.add('hmap')
 def view_hmap(token, dstore):
     """
@@ -676,8 +624,9 @@ def view_global_hcurves(token, dstore):
     """
     oq = dstore['oqparam']
     nsites = len(dstore['sitecol'])
-    rlzs_assoc = dstore['csm_info'].get_rlzs_assoc()
-    mean = getters.PmapGetter(dstore, rlzs_assoc).get_mean()
+    rlzs = dstore['full_lt'].get_realizations()
+    weights = [rlz.weight for rlz in rlzs]
+    mean = getters.PmapGetter(dstore, weights).get_mean()
     array = calc.convert_to_array(mean, nsites, oq.imtls)
     res = numpy.zeros(1, array.dtype)
     for name in array.dtype.names:
@@ -685,30 +634,6 @@ def view_global_hcurves(token, dstore):
     return rst_table(res)
 
 
-@view.add('dupl_sources_time')
-def view_dupl_sources_time(token, dstore):
-    """
-    Display the time spent computing duplicated sources
-    """
-    info = dstore['source_info']
-    items = sorted(group_array(info.value, 'source_id').items())
-    tbl = []
-    tot_time = 0
-    for source_id, records in items:
-        if len(records) > 1:  # dupl
-            calc_time = records['calc_time'].sum()
-            tot_time += calc_time + records['split_time'].sum()
-            tbl.append((source_id, calc_time, len(records)))
-    if tbl and info.attrs.get('has_dupl_sources'):
-        tot = info['calc_time'].sum() + info['split_time'].sum()
-        percent = tot_time / tot * 100
-        m = '\nTotal time in duplicated sources: %d/%d (%d%%)' % (
-            tot_time, tot, percent)
-        return rst_table(tbl, ['source_id', 'calc_time', 'num_dupl']) + m
-    else:
-        return 'There are no duplicated sources'
-
-
 @view.add('global_poes')
 def view_global_poes(token, dstore):
     """
@@ -737,7 +662,7 @@ def view_global_hmaps(token, dstore):
     oq = dstore['oqparam']
     dt = numpy.dtype([('%s-%s' % (imt, poe), F32)
                       for imt in oq.imtls for poe in oq.poes])
-    array = dstore['hmaps/mean'].value.view(dt)[:, 0]
+    array = dstore['hmaps/mean'][()].view(dt)[:, 0]
     res = numpy.zeros(1, array.dtype)
     for name in array.dtype.names:
         res[name] = array[name].mean()
@@ -754,6 +679,20 @@ def view_global_gmfs(token, dstore):
     return rst_table([row], header=imtls)
 
 
+@view.add('gmv_by_rup')
+def view_gmv_by_rup(token, dstore):
+    """
+    Display a synthetic gmv per rupture serial for debugging purposes
+    """
+    rup_id = dstore['events']['rup_id']
+    serial = dstore['ruptures']['serial']
+    data = dstore['gmf_data/data'][()]
+    gmv = fast_agg3(data, 'eid', ['gmv'])
+    gmv['eid'] = serial[rup_id[gmv['eid']]]
+    gm = fast_agg3(gmv, 'eid', ['gmv'])
+    return rst_table(gm, header=['serial', 'gmv'])
+
+
 @view.add('mean_disagg')
 def view_mean_disagg(token, dstore):
     """
@@ -762,7 +701,7 @@ def view_mean_disagg(token, dstore):
     """
     tbl = []
     for key, dset in sorted(dstore['disagg'].items()):
-        vals = [ds.value.mean() for k, ds in sorted(dset.items())]
+        vals = [ds[()].mean() for k, ds in sorted(dset.items())]
         tbl.append([key] + vals)
     header = ['key'] + sorted(dset)
     return rst_table(sorted(tbl), header=header)
@@ -774,8 +713,8 @@ def view_elt(token, dstore):
     Display the event loss table averaged by event
     """
     oq = dstore['oqparam']
-    R = len(dstore['csm_info'].rlzs)
-    dic = group_array(dstore['losses_by_event'].value, 'rlzi')
+    R = len(dstore['full_lt'].rlzs)
+    dic = group_array(dstore['losses_by_event'][()], 'rlzi')
     header = oq.loss_dt().names
     tbl = []
     for rlzi in range(R):
@@ -793,8 +732,9 @@ def view_pmap(token, dstore):
     """
     grp = token.split(':')[1]  # called as pmap:grp
     pmap = {}
-    rlzs_assoc = dstore['csm_info'].get_rlzs_assoc()
-    pgetter = getters.PmapGetter(dstore, rlzs_assoc)
+    rlzs = dstore['full_lt'].get_realizations()
+    weights = [rlz.weight for rlz in rlzs]
+    pgetter = getters.PmapGetter(dstore, weights)
     pmap = pgetter.get_mean(grp)
     return str(pmap)
 
@@ -804,10 +744,10 @@ def view_act_ruptures_by_src(token, dstore):
     """
     Display the actual number of ruptures by source in event based calculations
     """
-    data = dstore['ruptures'].value[['srcidx', 'serial']]
+    data = dstore['ruptures'][('srcidx', 'rup_id')]
     counts = sorted(countby(data, 'srcidx').items(),
                     key=operator.itemgetter(1), reverse=True)
-    src_info = dstore['source_info'].value[['grp_id', 'source_id']]
+    src_info = dstore['source_info'][('grp_id', 'source_id')]
     table = [['src_id', 'grp_id', 'act_ruptures']]
     for srcidx, act_ruptures in counts:
         src = src_info[srcidx]
@@ -815,52 +755,20 @@ def view_act_ruptures_by_src(token, dstore):
     return rst_table(table)
 
 
-Source = collections.namedtuple('Source', 'source_id code geom num_ruptures')
-
-
-def all_equal(records):
-    rec0 = records[0]
-    for rec in records[1:]:
-        for v1, v2 in zip(rec0, rec):
-            if isinstance(v1, numpy.ndarray):  # field geom
-                if len(v1) != len(v2):
-                    return False
-                for name in v1.dtype.names:
-                    if not numpy.allclose(v1[name], v2[name]):
-                        return False
-            elif v1 != v2:
-                return False
-    return True
-
-
-@view.add('dupl_sources')
-def view_dupl_sources(token, dstore):
+@view.add('bad_ruptures')
+def view_bad_ruptures(token, dstore):
     """
-    Show the sources with the same ID and the truly duplicated sources
+    Display the ruptures degenerating to a point
     """
-    fields = ['source_id', 'code', 'gidx1', 'gidx2', 'num_ruptures']
-    dic = group_array(dstore['source_info'].value[fields], 'source_id')
-    sameid = []
-    dupl = []
-    for source_id, group in dic.items():
-        if len(group) > 1:  # same ID sources
-            sources = []
-            for rec in group:
-                geom = dstore['source_geom'][rec['gidx1']:rec['gidx2']]
-                src = Source(source_id, rec['code'], geom, rec['num_ruptures'])
-                sources.append(src)
-            if all_equal(sources):
-                dupl.append(source_id)
-            sameid.append(source_id)
-    if not dupl:
-        return ''
-    msg = str(dupl) + '\n'
-    msg += ('Found %d source(s) with the same ID and %d true duplicate(s)'
-            % (len(sameid), len(dupl)))
-    fakedupl = set(sameid) - set(dupl)
-    if fakedupl:
-        msg += '\nHere is a fake duplicate: %s' % fakedupl.pop()
-    return msg
+    data = dstore['ruptures']['id', 'code', 'mag',
+                              'minlon', 'maxlon', 'minlat', 'maxlat']
+    bad = data[numpy.logical_and(data['minlon'] == data['maxlon'],
+                                 data['minlat'] == data['maxlat'])]
+    return rst_table(bad)
+
+
+Source = collections.namedtuple(
+    'Source', 'source_id code num_ruptures checksum')
 
 
 @view.add('extreme_groups')
@@ -868,7 +776,7 @@ def view_extreme_groups(token, dstore):
     """
     Show the source groups contributing the most to the highest IML
     """
-    data = dstore['disagg_by_grp'].value
+    data = dstore['disagg_by_grp'][()]
     data.sort(order='extreme_poe')
     return rst_table(data[::-1])
 
@@ -886,10 +794,10 @@ def view_gmvs_to_hazard(token, dstore):
     else:
         sid, rlz = int(args[0]), int(args[1])
     assert sid in dstore['sitecol'].sids
-    assert rlz < dstore['csm_info'].get_num_rlzs()
+    assert rlz < dstore['full_lt'].get_num_rlzs()
     oq = dstore['oqparam']
     num_ses = oq.ses_per_logic_tree_path
-    data = dstore['gmf_data/data'].value
+    data = dstore['gmf_data/data'][()]
     data = data[(data['sid'] == sid) & (data['rlzi'] == rlz)]
     tbl = []
     gmv = data['gmv']
@@ -900,3 +808,38 @@ def view_gmvs_to_hazard(token, dstore):
             poe = 1 - numpy.exp(- exceeding / num_ses)
             tbl.append((sid, rlz, imt, iml, exceeding, poe))
     return rst_table(tbl, ['sid', 'rlz', 'imt', 'iml', 'num_exceeding', 'poe'])
+
+
+@view.add('gmvs')
+def view_gmvs(token, dstore):
+    """
+    Show the GMVs on a given site ID
+    """
+    sid = int(token.split(':')[1])  # called as view_gmvs:sid
+    assert sid in dstore['sitecol'].sids
+    data = dstore['gmf_data/data'][()]
+    gmvs = data[data['sid'] == sid]['gmv']
+    return rst_table(gmvs)
+
+
+@view.add('events_by_mag')
+def view_events_by_mag(token, dstore):
+    """
+    Show how many events there are for each magnitude
+    """
+    rups = dstore['ruptures'][()]
+    num_evs = fast_agg(dstore['events']['rup_id'])
+    counts = {}
+    for mag, grp in group_array(rups, 'mag').items():
+        counts[mag] = sum(num_evs[rup_id] for rup_id in grp['id'])
+    return rst_table(counts.items(), ['mag', 'num_events'])
+
+
+@view.add('maximum_intensity')
+def view_maximum_intensity(token, dstore):
+    """
+    Show intensities at minimum and maximum distance for the highest magnitude
+    """
+    effect = extract(dstore, 'effect')
+    data = zip(dstore['full_lt'].trts, effect[-1, -1], effect[-1, 0])
+    return rst_table(data, ['trt', 'intensity1', 'intensity2'])
