@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2019 GEM Foundation
+# Copyright (C) 2014-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -19,10 +19,10 @@ import os
 import sys
 import getpass
 import logging
-from openquake.baselib import sap, config, datastore
+from openquake.baselib import sap, config, datastore, parallel
 from openquake.baselib.general import safeprint
 from openquake.hazardlib import valid
-from openquake.commonlib import logs, readinput
+from openquake.commonlib import logs, readinput, oqvalidation
 from openquake.engine import engine as eng
 from openquake.engine.export import core
 from openquake.engine.utils import confirm
@@ -31,8 +31,11 @@ from openquake.server import dbserver
 from openquake.commands.abort import abort
 
 
+DEFAULT_EXPORTS = 'csv,xml,rst'
 HAZARD_CALCULATION_ARG = "--hazard-calculation-id"
 MISSING_HAZARD_MSG = "Please specify '%s=<id>'" % HAZARD_CALCULATION_ARG
+ZMQ = os.environ.get(
+    'OQ_DISTRIBUTE', config.distribution.oq_distribute) == 'zmq'
 
 
 def get_job_id(job_id, username=None):
@@ -61,23 +64,19 @@ def run_job(job_ini, log_level='info', log_file=None, exports='',
         Extra parameters like hazard_calculation_id and calculation_mode
     """
     job_id = logs.init('job', getattr(logging, log_level.upper()))
-    with logs.handle(job_id, log_level, log_file):
-        job_ini = os.path.abspath(job_ini)
-        oqparam = eng.job_from_file(job_ini, job_id, username, **kw)
-        kw['username'] = username
-        eng.run_calc(job_id, oqparam, exports, **kw)
-        for line in logs.dbcmd('list_outputs', job_id, False):
-            safeprint(line)
+    try:
+        with logs.handle(job_id, log_level, log_file):
+            job_ini = os.path.abspath(job_ini)
+            oqparam = eng.job_from_file(job_ini, job_id, username, **kw)
+            kw['username'] = username
+            eng.run_calc(job_id, oqparam, exports)
+            for line in logs.dbcmd('list_outputs', job_id, False):
+                safeprint(line)
+    finally:
+        parallel.Starmap.shutdown()
     return job_id
 
 
-def run_tile(job_ini, sites_slice):
-    """
-    Used in tiling calculations
-    """
-    return run_job(job_ini, sites_slice=(sites_slice.start, sites_slice.stop))
-
-
 def del_calculation(job_id, confirmed=False):
     """
     Delete a calculation and all associated outputs.
@@ -101,7 +100,8 @@ def del_calculation(job_id, confirmed=False):
                 print(resp['error'])
 
 
-def smart_run(job_ini, oqparam, log_level, log_file, exports, reuse_hazard):
+def smart_run(job_ini, oqparam, log_level, log_file, exports,
+              reuse_hazard, **params):
     """
     Run calculations by storing their hazard checksum and reusing previous
     calculations if requested.
@@ -111,30 +111,17 @@ def smart_run(job_ini, oqparam, log_level, log_file, exports, reuse_hazard):
     job = logs.dbcmd('get_job_from_checksum', haz_checksum)
     reuse = reuse_hazard and job and os.path.exists(job.ds_calc_dir + '.hdf5')
     # recompute the hazard and store the checksum
-    ebr = (oqparam.calculation_mode == 'event_based_risk' and
-           'gmfs' not in oqparam.inputs)
-    if ebr:
-        kw = dict(calculation_mode='event_based')
-        if (oqparam.sites or 'sites' in oqparam.inputs or
-                'site_model' in oqparam.inputs):
-            # remove exposure from the hazard
-            kw['exposure_file'] = ''
-    else:
-        kw = {}
     if not reuse:
-        hc_id = run_job(job_ini, log_level, log_file, exports, **kw)
+        hc_id = run_job(job_ini, log_level, log_file, exports, **params)
         if job is None:
             logs.dbcmd('add_checksum', hc_id, haz_checksum)
         elif not reuse_hazard or not os.path.exists(job.ds_calc_dir + '.hdf5'):
             logs.dbcmd('update_job_checksum', hc_id, haz_checksum)
-        if ebr:
-            run_job(job_ini, log_level, log_file,
-                    exports, hazard_calculation_id=hc_id)
     else:
         hc_id = job.id
         logging.info('Reusing job #%d', job.id)
         run_job(job_ini, log_level, log_file,
-                exports, hazard_calculation_id=hc_id)
+                exports, hazard_calculation_id=hc_id, **params)
 
 
 @sap.Script  # do not use sap.script, other oq engine will break
@@ -144,7 +131,7 @@ def engine(log_file, no_distribute, yes, config_file, make_html_report,
            delete_calculation, delete_uncompleted_calculations,
            hazard_calculation_id, list_outputs, show_log,
            export_output, export_outputs, exports='',
-           log_level='info', reuse_hazard=False):
+           log_level='info', reuse_hazard=False, param=''):
     """
     Run a calculation using the traditional command line API
     """
@@ -155,7 +142,9 @@ def engine(log_file, no_distribute, yes, config_file, make_html_report,
     if config_file:
         config.read(os.path.abspath(os.path.expanduser(config_file)),
                     soft_mem_limit=int, hard_mem_limit=int, port=int,
-                    multi_user=valid.boolean, multi_node=valid.boolean)
+                    multi_user=valid.boolean,
+                    serialize_jobs=valid.boolean, strict=valid.boolean,
+                    code=exec)
 
     if no_distribute:
         os.environ['OQ_DISTRIBUTE'] = 'no'
@@ -202,6 +191,8 @@ def engine(log_file, no_distribute, yes, config_file, make_html_report,
     else:
         hc_id = None
     if run:
+        pars = dict(p.split('=', 1) for p in param.split(',')) if param else {}
+        oqvalidation.OqParam.check(pars)
         log_file = os.path.expanduser(log_file) \
             if log_file is not None else None
         job_inis = [os.path.expanduser(f) for f in run]
@@ -211,12 +202,12 @@ def engine(log_file, no_distribute, yes, config_file, make_html_report,
             # not using logs.handle that logs on the db
             oq = readinput.get_oqparam(job_inis[0])
             smart_run(job_inis[0], oq, log_level, log_file,
-                      exports, reuse_hazard)
+                      exports, reuse_hazard, **pars)
             return
         for i, job_ini in enumerate(job_inis):
             open(job_ini, 'rb').read()  # IOError if the file does not exist
             job_id = run_job(job_ini, log_level, log_file,
-                             exports, hazard_calculation_id=hc_id)
+                             exports, hazard_calculation_id=hc_id, **pars)
             if not hc_id:  # use the first calculation as base for the others
                 hc_id = job_id
     # hazard
@@ -250,14 +241,15 @@ def engine(log_file, no_distribute, yes, config_file, make_html_report,
         dskey, calc_id, datadir = logs.dbcmd('get_output', int(output_id))
         for line in core.export_output(
                 dskey, calc_id, datadir, os.path.expanduser(target_dir),
-                exports or 'csv,xml'):
+                exports or DEFAULT_EXPORTS):
             safeprint(line)
 
     elif export_outputs is not None:
         job_id, target_dir = export_outputs
         hc_id = get_job_id(job_id)
         for line in core.export_outputs(
-                hc_id, os.path.expanduser(target_dir), exports or 'csv,xml'):
+                hc_id, os.path.expanduser(target_dir),
+                exports or DEFAULT_EXPORTS):
             safeprint(line)
 
     elif delete_uncompleted_calculations:
@@ -318,3 +310,6 @@ engine.opt('exports', 'Comma-separated string specifing the export formats, '
 engine.opt('log_level', 'Defaults to "info"',
            choices=['debug', 'info', 'warn', 'error', 'critical'])
 engine.flg('reuse_hazard', 'Reuse the event based hazard if available')
+engine._add('param', '--param', '-p',
+            help='Override parameters specified with the syntax '
+            'NAME1=VALUE1,NAME2=VALUE2,...')
