@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2019 GEM Foundation
+# Copyright (C) 2015-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -18,46 +18,41 @@
 import os
 import re
 import getpass
-import shutil
 from openquake.baselib import sap, datastore
 from openquake.commonlib.logs import dbcmd
 
 datadir = datastore.get_datadir()
 
 
-def purge_one(calc_id, user):
+def purge_one(calc_id, user, force):
     """
     Remove one calculation ID from the database and remove its datastore
     """
-    filename = os.path.join(datadir, 'calc_%s.hdf5' % calc_id)
-    err = dbcmd('del_calc', calc_id, user)
-    if err:
-        print(err)
-    elif os.path.exists(filename):  # not removed yet
-        os.remove(filename)
-        print('Removed %s' % filename)
+    dbcmd('del_calc', calc_id, user, force)
+    f1 = os.path.join(datadir, 'calc_%s.hdf5' % calc_id)
+    f2 = os.path.join(datadir, 'calc_%s_tmp.hdf5' % calc_id)
+    for f in [f1, f2]:
+        if os.path.exists(f):  # not removed yet
+            os.remove(f)
+            print('Removed %s' % f)
 
 
 # used in the reset command
-def purge_all(user=None, fast=False):
+def purge_all(user=None):
     """
     Remove all calculations of the given user
     """
     user = user or getpass.getuser()
     if os.path.exists(datadir):
-        if fast:
-            shutil.rmtree(datadir)
-            print('Removed %s' % datadir)
-        else:
-            for fname in os.listdir(datadir):
-                mo = re.match('calc_(\d+)\.hdf5', fname)
-                if mo is not None:
-                    calc_id = int(mo.group(1))
-                    purge_one(calc_id, user)
+        for fname in os.listdir(datadir):
+            mo = re.match(r'(calc_|cache_)(\d+)\.hdf5', fname)
+            if mo is not None:
+                calc_id = int(mo.group(2))
+                purge_one(calc_id, user, force=True)
 
 
 @sap.script
-def purge(calc_id):
+def purge(calc_id, force=False):
     """
     Remove the given calculation. If you want to remove all calculations,
     use oq reset.
@@ -68,7 +63,8 @@ def purge(calc_id):
         except IndexError:
             print('Calculation %d not found' % calc_id)
             return
-    purge_one(calc_id, getpass.getuser())
+    purge_one(calc_id, getpass.getuser(), force)
 
 
 purge.arg('calc_id', 'calculation ID', type=int)
+purge.flg('force', 'ignore dependent calculations')
