@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright (C) 2017-2019 GEM Foundation
+# Copyright (C) 2017-2020 GEM Foundation
 
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -34,8 +34,8 @@ def restore(archive, oqdata):
     """
     Build a new oqdata directory from the data contained in the zip archive
     """
-    if os.path.exists(oqdata):
-        sys.exit('%s exists already' % oqdata)
+    if os.path.exists(oqdata) and os.listdir(oqdata):
+        sys.exit('%s is not empty' % oqdata)
     if '://' in archive:
         # get the zip archive from an URL
         resp = requests.get(archive)
@@ -47,14 +47,15 @@ def restore(archive, oqdata):
     t0 = time.time()
     oqdata = os.path.abspath(oqdata)
     assert archive.endswith('.zip'), archive
-    os.mkdir(oqdata)
+    if not os.path.exists(oqdata):
+        os.mkdir(oqdata)
     zipfile.ZipFile(archive).extractall(oqdata)
     dbpath = os.path.join(oqdata, 'db.sqlite3')
     db = Db(sqlite3.connect, dbpath, isolation_level=None,
             detect_types=sqlite3.PARSE_DECLTYPES)
     n = 0
     for fname in os.listdir(oqdata):
-        mo = re.match('calc_(\d+)\.hdf5', fname)
+        mo = re.match(r'calc_(\d+)\.hdf5', fname)
         if mo:
             job_id = int(mo.group(1))
             fullname = os.path.join(oqdata, fname)[:-5]  # strip .hdf5
