@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2019 GEM Foundation
+# Copyright (C) 2018-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -21,6 +21,11 @@ from functools import partial
 import numpy
 from openquake.baselib import sap
 from openquake.hazardlib import nrml
+from openquake.baselib.datastore import read
+from openquake.hazardlib.geo.geodetic import geodetic_distance
+from openquake.commonlib import readinput, calc, logs
+from openquake.calculators.base import get_calc
+from openquake.calculators.extract import extract, WebExtractor
 
 
 class OpenQuake(object):
@@ -29,10 +34,6 @@ class OpenQuake(object):
     engine utilities for work in the interactive interpreter.
     """
     def __init__(self):
-        from openquake.baselib.datastore import read
-        from openquake.hazardlib.geo.geodetic import geodetic_distance
-        from openquake.commonlib import readinput, calc
-        from openquake.calculators.extract import extract
         try:
             from matplotlib import pyplot
             self.plt = pyplot
@@ -48,9 +49,26 @@ class OpenQuake(object):
         self.get_site_collection = readinput.get_site_collection
         self.get_composite_source_model = readinput.get_composite_source_model
         self.get_exposure = readinput.get_exposure
+        self.get_calc = lambda job_ini: get_calc(job_ini, logs.init())
         self.make_hmap = calc.make_hmap
         self.geodetic_distance = geodetic_distance
-        # TODO: more utilities when be added when deemed useful
+        # TODO: more utilities will be added when deemed useful
+
+    def webex(self, calc_id, what):
+        """Extract data from a remote calculation"""
+        ex = WebExtractor(calc_id)
+        try:
+            return ex.get(what)
+        finally:
+            ex.close()
+
+    def ex(self, calc_id, what):
+        """Extract data from a local engine server"""
+        ex = WebExtractor(calc_id, 'http://localhost:8800', '', '')
+        try:
+            return ex.get(what)
+        finally:
+            ex.close()
 
 
 @sap.script
