@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2016-2019 GEM Foundation
+# Copyright (C) 2016-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -30,7 +30,7 @@ commands = ['start', 'migrate', 'createsuperuser', 'collectstatic']
 
 def rundjango(subcmd, hostport=None, skip_browser=False):
     args = [sys.executable, '-m', 'openquake.server.manage', subcmd]
-    if subcmd == 'start':
+    if subcmd == 'runserver':
         # the reload functionality of the Django development server interferes
         # with SIGCHLD and causes zombies, thus it is disabled
         args.append('--noreload')
