@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2019 GEM Foundation
+# Copyright (C) 2014-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -18,10 +18,8 @@
 import warnings
 import numpy
 
-from openquake.baselib import hdf5, general
-from openquake.baselib.python3compat import decode
+from openquake.baselib import hdf5
 from openquake.hazardlib.source.rupture import BaseRupture
-from openquake.hazardlib.gsim.base import ContextMaker
 from openquake.hazardlib import calc, probability_map
 
 TWO16 = 2 ** 16
@@ -37,11 +35,12 @@ F32 = numpy.float32
 U64 = numpy.uint64
 F64 = numpy.float64
 
-BaseRupture.init()
+code2cls = BaseRupture.init()
 
 # ############## utilities for the classical calculator ############### #
 
 
+# used only in the view global_hcurves
 def convert_to_array(pmap, nsites, imtls, inner_idx=0):
     """
     Convert the probability map into a composite array with header
@@ -56,14 +55,14 @@ def convert_to_array(pmap, nsites, imtls, inner_idx=0):
     # build the export dtype, of the form PGA-0.1, PGA-0.2 ...
     for imt, imls in imtls.items():
         for iml in imls:
-            lst.append(('%s-%s' % (imt, iml), F32))
+            lst.append(('%s-%.3f' % (imt, iml), F32))
     curves = numpy.zeros(nsites, numpy.dtype(lst))
     for sid, pcurve in pmap.items():
         curve = curves[sid]
         idx = 0
         for imt, imls in imtls.items():
             for iml in imls:
-                curve['%s-%s' % (imt, iml)] = pcurve.array[idx, inner_idx]
+                curve['%s-%.3f' % (imt, iml)] = pcurve.array[idx, inner_idx]
                 idx += 1
     return curves
 
@@ -169,24 +168,29 @@ def _gmvs_to_haz_curve(gmvs, imls, ses_per_logic_tree_path):
 
 # ################## utilities for classical calculators ################ #
 
-def make_hmap(pmap, imtls, poes):
+def make_hmap(pmap, imtls, poes, sid=None):
     """
     Compute the hazard maps associated to the passed probability map.
 
     :param pmap: hazard curves in the form of a ProbabilityMap
     :param imtls: DictArray with M intensity measure types
     :param poes: P PoEs where to compute the maps
+    :param sid: not None when pmap is actually a ProbabilityCurve
     :returns: a ProbabilityMap with size (N, M, P)
     """
+    if sid is None:
+        sids = pmap.sids
+    else:  # passed a probability curve
+        pmap = {sid: pmap}
+        sids = [sid]
     M, P = len(imtls), len(poes)
-    hmap = probability_map.ProbabilityMap.build(M, P, pmap, dtype=F32)
+    hmap = probability_map.ProbabilityMap.build(M, P, sids, dtype=F32)
     if len(pmap) == 0:
         return hmap  # empty hazard map
     for i, imt in enumerate(imtls):
-        curves = numpy.array([pmap[sid].array[imtls(imt), 0]
-                              for sid in pmap.sids])
+        curves = numpy.array([pmap[sid].array[imtls(imt), 0] for sid in sids])
         data = compute_hazard_maps(curves, imtls[imt], poes)  # array (N, P)
-        for sid, value in zip(pmap.sids, data):
+        for sid, value in zip(sids, data):
             array = hmap[sid].array
             for j, val in enumerate(value):
                 array[i, j] = val
@@ -197,16 +201,7 @@ def make_hmap_array(pmap, imtls, poes, nsites):
     """
     :returns: a compound array of hazard maps of shape nsites
     """
-    if isinstance(pmap, probability_map.ProbabilityMap):
-        # this is here for compatibility with the
-        # past, it could be removed in the future
-        hmap = make_hmap(pmap, imtls, poes)
-        pdic = general.DictArray({imt: poes for imt in imtls})
-        return convert_to_array(hmap, nsites, pdic)
-    try:
-        hcurves = pmap.value
-    except AttributeError:
-        hcurves = pmap
+    hcurves = pmap[()]
     dtlist = [('%s-%s' % (imt, poe), F32) for imt in imtls for poe in poes]
     array = numpy.zeros(len(pmap), dtlist)
     for imt, imls in imtls.items():
@@ -236,49 +231,6 @@ def make_uhs(hmap, info):
     return uhs
 
 
-class RuptureData(object):
-    """
-    Container for information about the ruptures of a given
-    tectonic region type.
-    """
-    def __init__(self, trt, gsims):
-        self.trt = trt
-        self.cmaker = ContextMaker(trt, gsims)
-        self.params = sorted(self.cmaker.REQUIRES_RUPTURE_PARAMETERS -
-                             set('mag strike dip rake hypo_depth'.split()))
-        self.dt = numpy.dtype([
-            ('rup_id', U32), ('srcidx', U32), ('multiplicity', U16),
-            ('occurrence_rate', F64),
-            ('mag', F32), ('lon', F32), ('lat', F32), ('depth', F32),
-            ('strike', F32), ('dip', F32), ('rake', F32),
-            ('boundary', hdf5.vstr)] + [(param, F32) for param in self.params])
-
-    def to_array(self, ebruptures):
-        """
-        Convert a list of ebruptures into an array of dtype RuptureRata.dt
-        """
-        data = []
-        for ebr in ebruptures:
-            rup = ebr.rupture
-            self.cmaker.add_rup_params(rup)
-            ruptparams = tuple(getattr(rup, param) for param in self.params)
-            point = rup.surface.get_middle_point()
-            multi_lons, multi_lats = rup.surface.get_surface_boundaries()
-            bounds = ','.join('((%s))' % ','.join(
-                '%.5f %.5f' % (lon, lat) for lon, lat in zip(lons, lats))
-                              for lons, lats in zip(multi_lons, multi_lats))
-            try:
-                rate = ebr.rupture.occurrence_rate
-            except AttributeError:  # for nonparametric sources
-                rate = numpy.nan
-            data.append(
-                (ebr.serial, ebr.srcidx, ebr.n_occ, rate,
-                 rup.mag, point.x, point.y, point.z, rup.surface.get_strike(),
-                 rup.surface.get_dip(), rup.rake,
-                 'MULTIPOLYGON(%s)' % decode(bounds)) + ruptparams)
-        return numpy.array(data, self.dt)
-
-
 class RuptureSerializer(object):
     """
     Serialize event based ruptures on an HDF5 files. Populate the datasets
@@ -300,10 +252,8 @@ class RuptureSerializer(object):
         offset = len(self.datastore['rupgeoms'])
         rup_array.array['gidx1'] += offset
         rup_array.array['gidx2'] += offset
-        previous = self.datastore.get_attr('ruptures', 'nbytes', 0)
-        self.datastore.extend(
-            'ruptures', rup_array, nbytes=previous + rup_array.nbytes)
-        self.datastore.extend('rupgeoms', rup_array.geom)
+        hdf5.extend(self.datastore['ruptures'], rup_array)
+        hdf5.extend(self.datastore['rupgeoms'], rup_array.geom)
         # TODO: PMFs for nonparametric ruptures are not stored
         self.datastore.flush()
 
@@ -316,6 +266,5 @@ class RuptureSerializer(object):
             return
         codes = numpy.unique(self.datastore['ruptures']['code'])
         attr = {'code_%d' % code: ' '.join(
-            cls.__name__ for cls in BaseRupture.types[code])
-                for code in codes}
+            cls.__name__ for cls in code2cls[code]) for code in codes}
         self.datastore.set_attrs('ruptures', **attr)
