@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2010-2019 GEM Foundation
+# Copyright (C) 2010-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -19,6 +19,7 @@
 Set up some system-wide loggers
 """
 import os.path
+import socket
 import logging
 from datetime import datetime
 from contextlib import contextmanager
@@ -30,8 +31,6 @@ LEVELS = {'debug': logging.DEBUG,
           'error': logging.ERROR,
           'critical': logging.CRITICAL}
 
-LOG = logging.getLogger()
-
 DBSERVER_PORT = int(os.environ.get('OQ_DBSERVER_PORT') or config.dbserver.port)
 
 
@@ -39,11 +38,12 @@ def dbcmd(action, *args):
     """
     A dispatcher to the database server.
 
-    :param action: database action to perform
-    :param args: arguments
+    :param string action: database action to perform
+    :param tuple args: arguments
     """
-    sock = zeromq.Socket('tcp://%s:%s' % (config.dbserver.host, DBSERVER_PORT),
-                         zeromq.zmq.REQ, 'connect')
+    host = socket.gethostbyname(config.dbserver.host)
+    sock = zeromq.Socket(
+        'tcp://%s:%s' % (host, DBSERVER_PORT), zeromq.zmq.REQ, 'connect')
     with sock:
         res = sock.send((action,) + args)
         if isinstance(res, parallel.Result):
@@ -147,18 +147,6 @@ def handle(job_id, log_level='info', log_file=None):
             logging.root.removeHandler(handler)
 
 
-def get_last_calc_id(username=None):
-    """
-    :param username: if given, restrict to it
-    :returns: the last calculation in the database or the datastore
-    """
-    if config.dbserver.multi_user:
-        job = dbcmd('get_job', -1, username)  # can be None
-        return getattr(job, 'id', 0)
-    else:  # single user
-        return datastore.get_last_calc_id()
-
-
 def init(calc_id='nojob', level=logging.INFO):
     """
     1. initialize the root logger (if not already initialized)
@@ -173,8 +161,12 @@ def init(calc_id='nojob', level=logging.INFO):
     elif calc_id == 'nojob':  # produce a calc_id without creating a job
         calc_id = datastore.get_last_calc_id() + 1
     else:
-        assert isinstance(calc_id, int), calc_id
+        calc_id = int(calc_id)
+        path = os.path.join(datastore.get_datadir(), 'calc_%d.hdf5' % calc_id)
+        if os.path.exists(path):
+            raise OSError('%s already exists' % path)
     fmt = '[%(asctime)s #{} %(levelname)s] %(message)s'.format(calc_id)
     for handler in logging.root.handlers:
-        handler.setFormatter(logging.Formatter(fmt))
+        f = logging.Formatter(fmt, datefmt='%Y-%m-%d %H:%M:%S')
+        handler.setFormatter(f)
     return calc_id
