@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2018-2019 GEM Foundation
+# Copyright (C) 2018-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 import sys
-import mock
+import unittest.mock as mock
 import os.path
 import logging
 from openquake.baselib import general
@@ -59,8 +59,7 @@ def zip_source_model(ssmLT, archive_zip='', log=logging.info):
     with open(checkfile, 'w') as f:
         f.write(str(checksum))
     files = [os.path.abspath(ssmLT), os.path.abspath(checkfile)]
-    for fs in logictree.collect_info(ssmLT).smpaths.values():
-        files.extend(fs)
+    files.extend(logictree.collect_info(ssmLT).smpaths)
     general.zipfiles(files, archive_zip, log=log, cleanup=True)
     return archive_zip
 
