@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2019 GEM Foundation
+# Copyright (C) 2012-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -253,7 +253,7 @@ def distance_matrix(lons, lats, diameter=2*EARTH_RADIUS):
         b = numpy.sin((lons[i] - lons) / 2.0)
         result[i, :] = numpy.arcsin(
             numpy.sqrt(a * a + cos_lats[i] * cos_lats * b * b)) * diameter
-    return numpy.matrix(result, copy=False)
+    return result
 
 
 def intervals_between(lon1, lat1, depth1, lon2, lat2, depth2, length):
