@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2019 GEM Foundation
+# Copyright (C) 2012-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -29,10 +29,8 @@ from openquake.hazardlib.geo.mesh import Mesh
 from openquake.hazardlib.geo import geodetic
 from openquake.hazardlib.geo.nodalplane import NodalPlane
 from openquake.hazardlib.geo import utils as geo_utils
-from openquake.baselib.slots import with_slots
 
 
-@with_slots
 class PlanarSurface(BaseSurface):
     """
     Planar rectangular surface with two sides parallel to the Earth surface.
@@ -66,10 +64,6 @@ class PlanarSurface(BaseSurface):
     #: as a fraction of the surface's area.
     IMPERFECT_RECTANGLE_TOLERANCE = 0.002
 
-    _slots_ = ('strike dip width length '
-               'corner_lons corner_lats corner_depths '
-               'normal d uv1 uv2 zero_zero').split()
-
     @property
     def surface_nodes(self):
         """
@@ -618,5 +612,13 @@ class PlanarSurface(BaseSurface):
         """
         The corners lons/lats in WKT-friendly order (clockwise)
         """
-        return [self.corner_lons.take([0, 1, 3, 2, 0])], \
-               [self.corner_lats.take([0, 1, 3, 2, 0])]
+        return (self.corner_lons.take([0, 1, 3, 2, 0]),
+                self.corner_lats.take([0, 1, 3, 2, 0]))
+
+    def get_surface_boundaries_3d(self):
+        """
+        The corners lons/lats/depths in WKT-friendly order (clockwise)
+        """
+        return (self.corner_lons.take([0, 1, 3, 2, 0]),
+                self.corner_lats.take([0, 1, 3, 2, 0]),
+                self.corner_depths.take([0, 1, 3, 2, 0]))
