@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2019 GEM Foundation
+# Copyright (C) 2014-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -69,15 +69,15 @@ class AbrahamsonEtAl2014(GMPE):
 
     #: Required site parameters are Vs30 and Z1.0, see table 2, page 1031
     #: Unit of measure for Z1.0 is [m]
-    REQUIRES_SITES_PARAMETERS = set(('vs30', 'z1pt0', 'vs30measured'))
+    REQUIRES_SITES_PARAMETERS = {'vs30', 'z1pt0', 'vs30measured'}
 
     #: Required rupture parameters are magnitude, rake, dip, ztor, and width
     #: (see table 2, page 1031)
-    REQUIRES_RUPTURE_PARAMETERS = set(('mag', 'rake', 'dip', 'ztor', 'width'))
+    REQUIRES_RUPTURE_PARAMETERS = {'mag', 'rake', 'dip', 'ztor', 'width'}
 
     #: Required distance measures are Rrup, Rjb, Ry0 and Rx (see Table 2,
     #: page 1031).
-    REQUIRES_DISTANCES = set(('rrup', 'rjb', 'rx', 'ry0'))
+    REQUIRES_DISTANCES = {'rrup', 'rjb', 'rx', 'ry0'}
 
     def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
         """
