@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2019 GEM Foundation
+# Copyright (C) 2012-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -47,6 +47,7 @@ ADMITTED_SET_PARAMETERS = ['DEFINED_FOR_INTENSITY_MEASURE_TYPES',
                            'REQUIRES_RUPTURE_PARAMETERS']
 
 registry = {}  # GSIM name -> GSIM class
+gsim_aliases = {}  # populated for instance in nbcc2015_AA13.py
 
 
 class NotVerifiedWarning(UserWarning):
@@ -62,6 +63,13 @@ class ExperimentalWarning(UserWarning):
     """
 
 
+class AdaptedWarning(UserWarning):
+    """
+    Raised for GMPEs that are intended for experimental use or maybe subject
+    to changes in future version.
+    """
+
+
 def gsim_imt_dt(sorted_gsims, sorted_imts):
     """
     Build a numpy dtype as a nested record with keys 'idx' and nested
@@ -75,6 +83,134 @@ def gsim_imt_dt(sorted_gsims, sorted_imts):
     return numpy.dtype([(str(gsim), imt_dt) for gsim in sorted_gsims])
 
 
+def get_mean_std(sctx, rctx, dctx, imts, gsims):
+    """
+    :returns: an array of shape (2, N, M, G) with means and stddevs
+    """
+    N = len(sctx.sids)
+    M = len(imts)
+    G = len(gsims)
+    arr = numpy.zeros((2, N, M, G))
+    num_tables = CoeffsTable.num_instances
+    for g, gsim in enumerate(gsims):
+        d = dctx.roundup(gsim.minimum_distance)
+        for m, imt in enumerate(imts):
+            mean, [std] = gsim.get_mean_and_stddevs(sctx, rctx, d, imt,
+                                                    [const.StdDev.TOTAL])
+            arr[0, :, m, g] = mean
+            arr[1, :, m, g] = std
+            if CoeffsTable.num_instances > num_tables:
+                raise RuntimeError('Instantiating CoeffsTable inside '
+                                   '%s.get_mean_and_stddevs' %
+                                   gsim.__class__.__name__)
+    return arr
+
+
+def get_poes(mean_std, loglevels, truncation_level, gsims=()):
+    """
+    Calculate and return probabilities of exceedance (PoEs) of one or more
+    intensity measure levels (IMLs) of one intensity measure type (IMT)
+    for one or more pairs "site -- rupture".
+
+    :param mean_std:
+        An array of shape (2, N, M, G) with mean and standard deviation for
+        the current intensity measure type
+    :param loglevels:
+        A DictArray imt -> logs of intensity measure levels
+    :param truncation_level:
+        Can be ``None``, which means that the distribution of intensity
+        is treated as Gaussian distribution with possible values ranging
+        from minus infinity to plus infinity.
+
+        When set to zero, the mean intensity is treated as an exact
+        value (standard deviation is not even computed for that case)
+        and resulting array contains 0 in places where IMT is strictly
+        lower than the mean value of intensity and 1.0 where IMT is equal
+        or greater.
+
+        When truncation level is positive number, the intensity
+        distribution is processed as symmetric truncated Gaussian with
+        range borders being ``mean - truncation_level * stddev`` and
+        ``mean + truncation_level * stddev``. That is, the truncation
+        level expresses how far the range borders are from the mean
+        value and is defined in units of sigmas. The resulting PoEs
+        for that mode are values of complementary cumulative distribution
+        function of that truncated Gaussian applied to IMLs.
+
+    :returns:
+        array of PoEs of shape (N, L, G)
+
+    :raises ValueError:
+        If truncation level is not ``None`` and neither non-negative
+        float number, and if ``imts`` dictionary contain wrong or
+        unsupported IMTs (see :attr:`DEFINED_FOR_INTENSITY_MEASURE_TYPES`).
+    """
+    if truncation_level is not None and truncation_level < 0:
+        raise ValueError('truncation level must be zero, positive number '
+                         'or None')
+    if len(gsims):
+        assert mean_std.shape[-1] == len(gsims)
+    tl = truncation_level
+    if any(hasattr(gsim, 'weights_signs') for gsim in gsims):
+        # implement average get_poes for the nshmp_2014 model
+        shp = list(mean_std[0].shape)  # (N, M, G)
+        shp[1] = len(loglevels.array)  # L
+        arr = numpy.zeros(shp)
+        for g, gsim in enumerate(gsims):
+            if hasattr(gsim, 'weights_signs'):
+                outs = []
+                weights, signs = zip(*gsim.weights_signs)
+                for s in signs:
+                    ms = numpy.array(mean_std[:, :, :, g])  # make a copy
+                    for m in range(len(loglevels)):
+                        ms[0, :, m] += s * gsim.adjustment
+                    outs.append(_get_poes(ms, loglevels, tl, squeeze=1))
+                arr[:, :, g] = numpy.average(outs, weights=weights, axis=0)
+            else:
+                ms = mean_std[:, :, :, g]
+                arr[:, :, g] = _get_poes(ms, loglevels, tl, squeeze=1)
+        return arr
+    elif any("mixture_model" in gsim.kwargs for gsim in gsims):
+        shp = list(mean_std[0].shape)  # (N, M, G)
+        shp[1] = len(loglevels.array)  # L
+        arr = numpy.zeros(shp)
+        for g, gsim in enumerate(gsims):
+            if "mixture_model" in gsim.kwargs:
+                for fact, wgt in zip(
+                    gsim.kwargs["mixture_model"]["factors"],
+                    gsim.kwargs["mixture_model"]["weights"]):
+                    mean_stdi = numpy.copy(mean_std)
+                    mean_stdi[1, :, :, :] *= fact
+                    arr[:, :, g] += (wgt * _get_poes(mean_stdi, loglevels, tl,
+                                                     squeeze=1))
+            else:
+                ms = mean_std[:, :, :, g]
+                arr[:, :, g] = _get_poes(ms, loglevels, tl, squeeze=1)
+        return arr
+    else:
+        # regular case
+        return _get_poes(mean_std, loglevels, truncation_level)
+
+
+# this is the critical function for the performance of the classical calculator
+# it is dominated by memory allocations (i.e. _truncnorm_sf is ultra-fast)
+# the only way to speedup is to reduce the maximum_distance, then the array
+# will become shorted in the N dimension (number of affected sites)
+def _get_poes(mean_std, loglevels, truncation_level, squeeze=False):
+    mean, stddev = mean_std  # shape (N, M, G) each
+    N, L, G = len(mean), len(loglevels.array), mean.shape[-1]
+    out = numpy.zeros((N, L) if squeeze else (N, L, G))
+    lvl = 0
+    for m, imt in enumerate(loglevels):
+        for iml in loglevels[imt]:
+            if truncation_level == 0:  # just compare imls to mean
+                out[:, lvl] = iml <= mean[:, m]
+            else:
+                out[:, lvl] = (iml - mean[:, m]) / stddev[:, m]
+            lvl += 1
+    return _truncnorm_sf(truncation_level, out)
+
+
 class MetaGSIM(abc.ABCMeta):
     """
     A metaclass converting set class attributes into frozensets, to avoid
@@ -104,7 +240,7 @@ class GroundShakingIntensityModel(metaclass=MetaGSIM):
     at a site given an earthquake rupture.
 
     This class is not intended to be subclassed directly, instead
-    the actual GSIMs should subclass either :class:`GMPE` or :class:`IPE`.
+    the actual GSIMs should subclass :class:`GMPE`
 
     Subclasses of both must implement :meth:`get_mean_and_stddevs`
     and all the class attributes with names starting from ``DEFINED_FOR``
@@ -190,10 +326,18 @@ class GroundShakingIntensityModel(metaclass=MetaGSIM):
     superseded_by = None
     non_verified = False
     experimental = False
+    adapted = False
+    get_poes = staticmethod(get_poes)
 
     @classmethod
     def __init_subclass__(cls):
-        registry[cls.__name__] = cls
+        stddevtypes = cls.DEFINED_FOR_STANDARD_DEVIATION_TYPES
+        if not isinstance(stddevtypes, abc.abstractproperty):  # concrete class
+            if const.StdDev.TOTAL not in stddevtypes:
+                raise ValueError('%s.DEFINED_FOR_STANDARD_DEVIATION_TYPES is '
+                                 'not defined for const.StdDev.TOTAL' %
+                                 cls.__name__)
+            registry[cls.__name__] = cls
 
     def __init__(self, **kwargs):
         self.kwargs = kwargs
@@ -210,11 +354,11 @@ class GroundShakingIntensityModel(metaclass=MetaGSIM):
             msg = ('%s is experimental and may change in future versions - '
                    'the user is liable for their application') % cls.__name__
             warnings.warn(msg, ExperimentalWarning)
-
-    def init(self):
-        """
-        Override this method if you want to further initialize the GSIM
-        """
+        if cls.adapted:
+            msg = ('%s is not intended for general use and the behaviour '
+                   'may not be as expected - '
+                   'the user is liable for their application') % cls.__name__
+            warnings.warn(msg, AdaptedWarning)
 
     @abc.abstractmethod
     def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
@@ -271,162 +415,6 @@ class GroundShakingIntensityModel(metaclass=MetaGSIM):
         compute interim steps).
         """
 
-    def get_poes(self, sctx, rctx, dctx, imt, imls, truncation_level):
-        """
-        Calculate and return probabilities of exceedance (PoEs) of one or more
-        intensity measure levels (IMLs) of one intensity measure type (IMT)
-        for one or more pairs "site -- rupture".
-
-        :param sctx:
-            An instance of :class:`SitesContext` with sites information
-            to calculate PoEs on.
-        :param rctx:
-            An instance of :class:`RuptureContext` with a single rupture
-            information.
-        :param dctx:
-            An instance of :class:`DistancesContext` with information about
-            the distances between sites and a rupture.
-
-            All three contexts (``sctx``, ``rctx`` and ``dctx``) must conform
-            to each other. The easiest way to get them is to call
-            ContextMaker.make_contexts.
-        :param imt:
-            An intensity measure type object (that is, an instance of one
-            of classes from :mod:`openquake.hazardlib.imt`).
-        :param imls:
-            List of interested intensity measure levels (of type ``imt``).
-        :param truncation_level:
-            Can be ``None``, which means that the distribution of intensity
-            is treated as Gaussian distribution with possible values ranging
-            from minus infinity to plus infinity.
-
-            When set to zero, the mean intensity is treated as an exact
-            value (standard deviation is not even computed for that case)
-            and resulting array contains 0 in places where IMT is strictly
-            lower than the mean value of intensity and 1.0 where IMT is equal
-            or greater.
-
-            When truncation level is positive number, the intensity
-            distribution is processed as symmetric truncated Gaussian with
-            range borders being ``mean - truncation_level * stddev`` and
-            ``mean + truncation_level * stddev``. That is, the truncation
-            level expresses how far the range borders are from the mean
-            value and is defined in units of sigmas. The resulting PoEs
-            for that mode are values of complementary cumulative distribution
-            function of that truncated Gaussian applied to IMLs.
-
-        :returns:
-            A dictionary of the same structure as parameter ``imts`` (see
-            above). Instead of lists of IMLs values of the dictionaries
-            have 2d numpy arrays of corresponding PoEs, first dimension
-            represents sites and the second represents IMLs.
-
-        :raises ValueError:
-            If truncation level is not ``None`` and neither non-negative
-            float number, and if ``imts`` dictionary contain wrong or
-            unsupported IMTs (see :attr:`DEFINED_FOR_INTENSITY_MEASURE_TYPES`).
-        """
-        if truncation_level is not None and truncation_level < 0:
-            raise ValueError('truncation level must be zero, positive number '
-                             'or None')
-        self._check_imt(imt)
-
-        if truncation_level == 0:
-            # zero truncation mode, just compare imls to mean
-            imls = self.to_distribution_values(imls)
-            mean, _ = self.get_mean_and_stddevs(sctx, rctx, dctx, imt, [])
-            mean = mean.reshape(mean.shape + (1, ))
-            return (imls <= mean).astype(float)
-        else:
-            # use real normal distribution
-            assert (const.StdDev.TOTAL
-                    in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES)
-            imls = self.to_distribution_values(imls)
-            mean, [stddev] = self.get_mean_and_stddevs(sctx, rctx, dctx, imt,
-                                                       [const.StdDev.TOTAL])
-            mean = mean.reshape(mean.shape + (1, ))
-            stddev = stddev.reshape(stddev.shape + (1, ))
-            values = (imls - mean) / stddev
-            if truncation_level is None:
-                return _norm_sf(values)
-            else:
-                return _truncnorm_sf(truncation_level, values)
-
-    def disaggregate_pne(self, rupture, sctx, dctx, imt, iml,
-                         truncnorm, epsilons):
-        """
-        Disaggregate (separate) PoE of ``iml`` in different contributions
-        each coming from ``epsilons`` distribution bins.
-
-        Other parameters are the same as for :meth:`get_poes`, with
-        differences that ``truncation_level`` is required to be positive.
-
-        :returns:
-            Contribution to probability of exceedance of ``iml`` coming
-            from different sigma bands in the form of a 2d numpy array of
-            probabilities with shape (n_sites, n_epsilons)
-        """
-        # compute mean and standard deviations
-        mean, [stddev] = self.get_mean_and_stddevs(sctx, rupture, dctx, imt,
-                                                   [const.StdDev.TOTAL])
-
-        # compute iml value with respect to standard (mean=0, std=1)
-        # normal distributions
-        standard_imls = (self.to_distribution_values(iml) - mean) / stddev
-
-        # compute epsilon bins contributions
-        contribution_by_bands = (truncnorm.cdf(epsilons[1:]) -
-                                 truncnorm.cdf(epsilons[:-1]))
-
-        # take the minimum epsilon larger than standard_iml
-        bins = numpy.searchsorted(epsilons, standard_imls)
-        poe_by_site = []
-        n_epsilons = len(epsilons) - 1
-        for lvl, bin in zip(standard_imls, bins):  # one per site
-            if bin == 0:
-                poe_by_site.append(contribution_by_bands)
-            elif bin > n_epsilons:
-                poe_by_site.append(numpy.zeros(n_epsilons))
-            else:
-                # for other cases (when ``lvl`` falls somewhere in the
-                # histogram):
-                poe = numpy.concatenate([
-                    # take zeros for bins that are on the left hand side
-                    # from the bin ``lvl`` falls into,
-                    numpy.zeros(bin - 1),
-                    # ... area of the portion of the bin containing ``lvl``
-                    # (the portion is limited on the left hand side by
-                    # ``lvl`` and on the right hand side by the bin edge),
-                    [truncnorm.sf(lvl) - contribution_by_bands[bin:].sum()],
-                    # ... and all bins on the right go unchanged.
-                    contribution_by_bands[bin:]])
-                poe_by_site.append(poe)
-        poes = numpy.array(poe_by_site)  # shape (n_sites, n_epsilons)
-        return rupture.get_probability_no_exceedance(poes)
-
-    @abc.abstractmethod
-    def to_distribution_values(self, values):
-        """
-        Convert a list or array of values in units of IMT to a numpy array
-        of values of intensity measure distribution (like taking the natural
-        logarithm for :class:`GMPE`).
-
-        This method is implemented by both :class:`GMPE` and :class:`IPE`
-        so there is no need to override it in actual GSIM implementations.
-        """
-
-    @abc.abstractmethod
-    def to_imt_unit_values(self, values):
-        """
-        Convert a list or array of values of intensity measure distribution
-        (like ones returned from :meth:`get_mean_and_stddevs`) to values
-        in units of IMT. This is the opposite operation
-        to :meth:`to_distribution_values`.
-
-        This method is implemented by both :class:`GMPE` and :class:`IPE`
-        so there is no need to override it in actual GSIM implementations.
-        """
-
     def _check_imt(self, imt):
         """
         Make sure that ``imt`` is valid and is supported by this GSIM.
@@ -474,7 +462,7 @@ def _truncnorm_sf(truncation_level, values):
 
     :param truncation_level:
         Positive float number representing the truncation on both sides
-        around the mean, in units of sigma.
+        around the mean, in units of sigma, or None, for non-truncation
     :param values:
         Numpy array of values as input to a survival function for the given
         distribution.
@@ -484,7 +472,16 @@ def _truncnorm_sf(truncation_level, values):
     >>> from scipy.stats import truncnorm
     >>> truncnorm(-3, 3).sf(0.12345) == _truncnorm_sf(3, 0.12345)
     True
+    >>> from scipy.stats import norm
+    >>> norm.sf(0.12345) == _truncnorm_sf(None, 0.12345)
+    True
     """
+    if truncation_level == 0:
+        return values
+
+    if truncation_level is None:
+        return ndtr(- values)
+
     # notation from http://en.wikipedia.org/wiki/Truncated_normal_distribution.
     # given that mu = 0 and sigma = 1, we have alpha = a and beta = b.
 
@@ -512,26 +509,15 @@ def _truncnorm_sf(truncation_level, values):
     return ((phi_b - ndtr(values)) / z).clip(0.0, 1.0)
 
 
-def _norm_sf(values):
+def to_distribution_values(vals, imt):
     """
-    Survival function for normal distribution.
-
-    Assumes zero mean and standard deviation equal to one.
-
-    ``values`` parameter and the return value are the same
-    as in :func:`_truncnorm_sf`.
-
-    >>> from scipy.stats import norm
-    >>> norm.sf(0.12345) == _norm_sf(0.12345)
-    True
+    :returns: the logarithm of the values unless the IMT is MMI
     """
-    # survival function by definition is ``SF(x) = 1 - CDF(x)``,
-    # which is equivalent to ``SF(x) = CDF(- x)``, since (given
-    # that the normal distribution is symmetric with respect to 0)
-    # the integral between ``[x, +infinity]`` (that is the survival
-    # function) is equal to the integral between ``[-infinity, -x]``
-    # (that is the CDF at ``- x``).
-    return ndtr(- values)
+    if str(imt) == 'MMI':
+        return vals
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        return numpy.log(vals)
 
 
 class GMPE(GroundShakingIntensityModel):
@@ -544,20 +530,14 @@ class GMPE(GroundShakingIntensityModel):
     of actual GMPE implementations is supposed to return the mean
     value as a natural logarithm of intensity.
     """
-    def to_distribution_values(self, values):
+    def open(self, fname_or_file):
         """
-        Returns numpy array of natural logarithms of ``values``.
+        :param fname_or_file: filename or filelike object
+        :returns: the file object
         """
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            # avoid RuntimeWarning: divide by zero encountered in log
-            return numpy.log(values)
-
-    def to_imt_unit_values(self, values):
-        """
-        Returns numpy array of exponents of ``values``.
-        """
-        return numpy.exp(values)
+        if hasattr(fname_or_file, 'read'):
+            return fname_or_file
+        return open(fname_or_file, 'rb')
 
     def set_parameters(self):
         """
@@ -574,26 +554,6 @@ class GMPE(GroundShakingIntensityModel):
                 setattr(self, key, val)
 
 
-class IPE(GroundShakingIntensityModel):
-    """
-    Intensity Prediction Equation is a subclass of generic
-    :class:`GroundShakingIntensityModel` which is suitable for
-    intensity measures that are normally distributed. In particular,
-    for :class:`~openquake.hazardlib.imt.MMI`.
-    """
-    def to_distribution_values(self, values):
-        """
-        Returns numpy array of ``values`` without any conversion.
-        """
-        return numpy.array(values, dtype=float)
-
-    def to_imt_unit_values(self, values):
-        """
-        Returns numpy array of ``values`` without any conversion.
-        """
-        return numpy.array(values, dtype=float)
-
-
 class CoeffsTable(object):
     r"""
     Instances of :class:`CoeffsTable` encapsulate tables of coefficients
@@ -708,9 +668,12 @@ class CoeffsTable(object):
     ...           imt.PGV(): {"a": 0.5, "b": 10.0}}
     >>> ct = CoeffsTable(sa_damping=5, table=coeffs)
     """
+    num_instances = 0
+
     def __init__(self, **kwargs):
         if 'table' not in kwargs:
             raise TypeError('CoeffsTable requires "table" kwarg')
+        self._coeffs = {}  # cache
         table = kwargs.pop('table')
         self.sa_coeffs = {}
         self.non_sa_coeffs = {}
@@ -728,6 +691,7 @@ class CoeffsTable(object):
         else:
             raise TypeError("CoeffsTable cannot be constructed with inputs "
                             "of the form '%s'" % table.__class__.__name__)
+        self.__class__.num_instances += 1
 
     def _setup_table_from_str(self, table, sa_damping):
         """
@@ -771,11 +735,16 @@ class CoeffsTable(object):
             If ``imt`` is not available in the table and no interpolation
             can be done.
         """
-        if imt.name != 'SA':
-            return self.non_sa_coeffs[imt]
-
         try:
-            return self.sa_coeffs[imt]
+            return self._coeffs[imt]
+        except KeyError:
+            pass
+        if imt.name != 'SA':
+            self._coeffs[imt] = c = self.non_sa_coeffs[imt]
+            return c
+        try:
+            self._coeffs[imt] = c = self.sa_coeffs[imt]
+            return c
         except KeyError:
             pass
 
@@ -799,6 +768,7 @@ class CoeffsTable(object):
                  / (math.log(min_above.period) - math.log(max_below.period)))
         max_below = self.sa_coeffs[max_below]
         min_above = self.sa_coeffs[min_above]
-        return dict(
-            (co, (min_above[co] - max_below[co]) * ratio + max_below[co])
-            for co in max_below)
+        self._coeffs[imt] = c = {
+            co: (min_above[co] - max_below[co]) * ratio + max_below[co]
+            for co in max_below}
+        return c
