@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2013-2019 GEM Foundation
+# Copyright (C) 2013-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -72,13 +72,13 @@ class CampbellBozorgnia2008(GMPE):
 
     #: Required site parameters are Vs30, Vs30 type (measured or inferred),
     #: and depth (km) to the 2.5 km/s shear wave velocity layer (z2pt5)
-    REQUIRES_SITES_PARAMETERS = set(('vs30', 'z2pt5'))
+    REQUIRES_SITES_PARAMETERS = {'vs30', 'z2pt5'}
 
     #: Required rupture parameters are magnitude, rake, dip, ztor
-    REQUIRES_RUPTURE_PARAMETERS = set(('mag', 'rake', 'dip', 'ztor'))
+    REQUIRES_RUPTURE_PARAMETERS = {'mag', 'rake', 'dip', 'ztor'}
 
     #: Required distance measures are Rrup and Rjb.
-    REQUIRES_DISTANCES = set(('rrup', 'rjb'))
+    REQUIRES_DISTANCES = {'rrup', 'rjb'}
 
     def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
         """
