@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2019 GEM Foundation
+# Copyright (C) 2012-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -71,20 +71,18 @@ class CauzziFaccioli2008(GMPE):
 
     #: Supported standard deviation type is only total, see paragraph 'On
     #: functional forms', page 462.
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
-        const.StdDev.TOTAL
-    ])
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {const.StdDev.TOTAL}
 
     #: Required site parameter is only Vs30 (used to distinguish rock
     #: and deep soils), see paragraph 'On functional forms', page 463.
-    REQUIRES_SITES_PARAMETERS = set(('vs30', ))
+    REQUIRES_SITES_PARAMETERS = {'vs30'}
 
     #: Required rupture parameters are magnitude and rake, see paragraph 'On
     #: functional forms', page 463
-    REQUIRES_RUPTURE_PARAMETERS = set(('rake', 'mag'))
+    REQUIRES_RUPTURE_PARAMETERS = {'rake', 'mag'}
 
     #: Required distance measure is Rhypo, see paragraph 'Distance', page 456.
-    REQUIRES_DISTANCES = set(('rhypo', ))
+    REQUIRES_DISTANCES = {'rhypo'}
 
     def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
         """
@@ -114,9 +112,8 @@ class CauzziFaccioli2008(GMPE):
         Distances are clipped at 15 km (as per Ezio Faccioli's personal
         communication.)
         """
-        d = rhypo
+        d = np.array(rhypo)  # make a copy
         d[d <= 15.0] = 15.0
-
         return C['a3'] * np.log10(d)
 
     def _compute_site_term(self, C, vs30):
