@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2019 GEM Foundation
+# Copyright (C) 2015-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -23,7 +23,6 @@ in the form of binary tables, and
 :class:`openquake.hazardlib.gsim.gmpe_table.AmplificationTable` for defining
 the corresponding amplification of the IMLs
 """
-import os
 from copy import deepcopy
 
 import h5py
@@ -291,7 +290,7 @@ class GMPETable(GMPE):
 
     DEFINED_FOR_INTENSITY_MEASURE_COMPONENT = ""
 
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set((const.StdDev.TOTAL,))
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {const.StdDev.TOTAL}
 
     REQUIRES_SITES_PARAMETERS = set()
 
@@ -299,47 +298,35 @@ class GMPETable(GMPE):
 
     REQUIRES_RUPTURE_PARAMETERS = {"mag"}
 
-    GMPE_TABLE = None
+    gmpe_table = None  # see subclasses like NBCC2015_AA13_activecrustFRjb_low
 
     amplification = None
 
-    def init(self, fle=None):
+    def __init__(self, **kwargs):
         """
         Executes the preprocessing steps at the instantiation stage to read in
         the tables from hdf5 and hold them in memory.
         """
-        if fle is None:
-            fname = self.kwargs.get('gmpe_table', self.GMPE_TABLE)
-            if fname is None:
-                raise ValueError('You forgot to set GMPETable.GMPE_TABLE!')
-            elif os.path.isabs(fname):
-                self.GMPE_TABLE = fname
-            else:
-                # NB: (hackish) GMPE_DIR must be set externally
-                self.GMPE_TABLE = os.path.abspath(
-                    os.path.join(self.GMPE_DIR, fname))
-            fle = h5py.File(self.GMPE_TABLE, "r")
-        try:
-            # this is the format inside the datastore
-            self.distance_type = fle["distance_type"].value
-        except KeyError:
-            # this is the original format outside the datastore
+        super().__init__(**kwargs)
+        fname = self.kwargs.get('gmpe_table', self.gmpe_table)
+        with h5py.File(fname, "r") as fle:
             self.distance_type = decode(fle["Distances"].attrs["metric"])
-        self.REQUIRES_DISTANCES = set([self.distance_type])
-        # Load in magnitude
-        self.m_w = fle["Mw"][:]
-        # Load in distances
-        self.distances = fle["Distances"][:]
-        # Load intensity measure types and levels
-        self.imls = hdf_arrays_to_dict(fle["IMLs"])
-        self.DEFINED_FOR_INTENSITY_MEASURE_TYPES = set(self._supported_imts())
-        if "SA" in self.imls and "T" not in self.imls:
-            raise ValueError("Spectral Acceleration must be accompanied by "
-                             "periods")
-        # Get the standard deviations
-        self._setup_standard_deviations(fle)
-        if "Amplification" in fle:
-            self._setup_amplification(fle)
+            self.REQUIRES_DISTANCES = set([self.distance_type])
+            # Load in magnitude
+            self.m_w = fle["Mw"][:]
+            # Load in distances
+            self.distances = fle["Distances"][:]
+            # Load intensity measure types and levels
+            self.imls = hdf_arrays_to_dict(fle["IMLs"])
+            self.DEFINED_FOR_INTENSITY_MEASURE_TYPES = set(
+                self._supported_imts())
+            if "SA" in self.imls and "T" not in self.imls:
+                raise ValueError("Spectral Acceleration must be accompanied by"
+                                 " periods")
+            # Get the standard deviations
+            self._setup_standard_deviations(fle)
+            if "Amplification" in fle:
+                self._setup_amplification(fle)
 
     def _setup_standard_deviations(self, fle):
         """
