@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2019 GEM Foundation
+# Copyright (C) 2012-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -68,15 +68,15 @@ class ZhaoEtAl2006Asc(GMPE):
 
     #: Required site parameters is Vs30.
     #: See table 2, p. 901.
-    REQUIRES_SITES_PARAMETERS = set(('vs30', ))
+    REQUIRES_SITES_PARAMETERS = {'vs30'}
 
     #: Required rupture parameters are magnitude, rake, and focal depth.
     #: See paragraph 'Development of Base Model', p. 901.
-    REQUIRES_RUPTURE_PARAMETERS = set(('mag', 'rake', 'hypo_depth'))
+    REQUIRES_RUPTURE_PARAMETERS = {'mag', 'rake', 'hypo_depth'}
 
     #: Required distance measure is Rrup.
     #: See paragraph 'Development of Base Model', p. 902.
-    REQUIRES_DISTANCES = set(('rrup', ))
+    REQUIRES_DISTANCES = {'rrup'}
 
     def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
         """
@@ -243,7 +243,7 @@ class ZhaoEtAl2006SInter(ZhaoEtAl2006Asc):
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTERFACE
 
     #: Required rupture parameters are magnitude and focal depth.
-    REQUIRES_RUPTURE_PARAMETERS = set(('mag', 'hypo_depth'))
+    REQUIRES_RUPTURE_PARAMETERS = {'mag', 'hypo_depth'}
 
     def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
         """
@@ -326,7 +326,7 @@ class ZhaoEtAl2006SSlab(ZhaoEtAl2006Asc):
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTRASLAB
 
     #: Required rupture parameters are magnitude and focal depth.
-    REQUIRES_RUPTURE_PARAMETERS = set(('mag', 'hypo_depth'))
+    REQUIRES_RUPTURE_PARAMETERS = {'mag', 'hypo_depth'}
 
     def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
         """
@@ -341,7 +341,7 @@ class ZhaoEtAl2006SSlab(ZhaoEtAl2006Asc):
 
         # to avoid singularity at 0.0 (in the calculation of the
         # slab correction term), replace 0 values with 0.1
-        d = dists.rrup
+        d = np.array(dists.rrup)  # make a copy
         d[d == 0.0] = 0.1
 
         # mean value as given by equation 1, p. 901, without considering the
@@ -427,7 +427,7 @@ class ZhaoEtAl2006SInterNSHMP2008(ZhaoEtAl2006SInter):
         Call super class method with hypocentral depth fixed at 20 km
         """
         # create new rupture context to avoid changing the original one
-        new_rup = copy.deepcopy(rup)
+        new_rup = copy.copy(rup)
         new_rup.hypo_depth = 20.
 
         mean, stddevs = super().get_mean_and_stddevs(
@@ -479,7 +479,7 @@ class ZhaoEtAl2006SSlabNSHMP2014(ZhaoEtAl2006SSlab):
 
         # to avoid singularity at 0.0 (in the calculation of the
         # slab correction term), replace 0 values with 0.1
-        d = dists.rrup
+        d = np.array(dists.rrup)  # make a copy
         d[d == 0.0] = 0.1
 
         if rup.mag > 7.8:
@@ -616,7 +616,7 @@ class ZhaoEtAl2006SSlabCascadia(ZhaoEtAl2006SSlab):
 
         # to avoid singularity at 0.0 (in the calculation of the
         # slab correction term), replace 0 values with 0.1
-        d = dists.rrup
+        d = np.array(dists.rrup)  # make a copy
         d[d == 0.0] = 0.1
 
         # mean value as given by equation 1, p. 901, without considering the
@@ -655,7 +655,6 @@ class ZhaoEtAl2006AscSGS(ZhaoEtAl2006Asc):
         """
         Using a minimum distance of 5km for the calculation.
         """
-
         dists_mod = copy.deepcopy(dists)
         dists_mod.rrup[dists.rrup <= 5.] = 5.
 
