@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2019 GEM Foundation
+# Copyright (C) 2012-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -222,6 +222,14 @@ class MMI(IMT):
     and on humans and their structures.
     """
 
+
+class JMA(IMT):
+    """
+    Modified Mercalli intensity, a Roman numeral describing the severity
+    of an earthquake in terms of its effects on the earth's surface
+    and on humans and their structures.
+    """
+
 # geotechnical IMTs
 
 
@@ -255,21 +263,3 @@ class ASH(IMT):
     """
     Level of the ash fall in millimeters
     """
-
-
-class LAVA(IMT):
-    """
-    Boolean value for the lava flow (1 if affected 0 otherwise)
-    """
-
-
-class LAHAR(IMT):
-    """
-    Boolean value for the lahars (1 if affected 0 otherwise)
-    """
-
-
-class PYRO(IMT):
-    """
-    Boolean value for the pyroclastic flow (1 if affected 0 otherwise)
-    """
