@@ -1,5 +1,5 @@
 # The Hazard Library
-# Copyright (C) 2013-2019 GEM Foundation
+# Copyright (C) 2013-2020 GEM Foundation
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -27,12 +27,10 @@ from openquake.hazardlib.geo.utils import angular_distance, KM_TO_DEGREES
 from openquake.hazardlib.geo.mesh import Mesh, point3d
 from openquake.hazardlib.geo.point import Point
 from openquake.hazardlib.pmf import PMF
-from openquake.baselib.slots import with_slots
 
 F32 = numpy.float32
 
 
-@with_slots
 class NonParametricSeismicSource(BaseSeismicSource):
     """
     Non Parametric Seismic Source explicitly defines earthquake ruptures in the
@@ -51,15 +49,13 @@ class NonParametricSeismicSource(BaseSeismicSource):
         of occurrences equal to 0)
     """
     code = b'N'
-    _slots_ = BaseSeismicSource._slots_ + ['data']
-
     MODIFICATIONS = set()
 
     def __init__(self, source_id, name, tectonic_region_type, data):
         super().__init__(source_id, name, tectonic_region_type)
         self.data = data
 
-    def iter_ruptures(self):
+    def iter_ruptures(self, **kwargs):
         """
         Get a generator object that yields probabilistic ruptures the source
         consists of.
@@ -83,7 +79,7 @@ class NonParametricSeismicSource(BaseSeismicSource):
             src = self.__class__(source_id, self.name,
                                  self.tectonic_region_type, [rup_pmf])
             src.num_ruptures = 1
-            src.src_group_id = self.src_group_id
+            src.grp_id = self.grp_id
             yield src
 
     def count_ruptures(self):
@@ -161,16 +157,27 @@ class NonParametricSeismicSource(BaseSeismicSource):
     def __repr__(self):
         return '<%s gridded=%s>' % (self.__class__.__name__, self.is_gridded())
 
-    def geom(self):
+    @property
+    def polygon(self):
         """
-        :returns: the geometry as an array of shape (N, 3)
+        The convex hull of the underlying mesh of points
         """
-        # the rupture can have a faultSurface which is a 3D array
-        # or can be a griddedSurface which is a 2D array or others
-        arr = numpy.concatenate([rup.surface.mesh.array.reshape(3, -1)
-                                 for rup, pmf in self.data],
-                                axis=1)  # shape (3, N)
-        return arr.T
+        lons = numpy.concatenate(
+            [rup.surface.mesh.lons.flatten() for rup, pmf in self.data])
+        lats = numpy.concatenate(
+            [rup.surface.mesh.lats.flatten() for rup, pmf in self.data])
+        points = numpy.zeros(len(lons), [('lon', F32), ('lat', F32)])
+        points['lon'] = lons
+        points['lat'] = lats
+        points = numpy.unique(points)
+        mesh = Mesh(points['lon'], points['lat'])
+        return mesh.get_convex_hull()
+
+    def wkt(self):
+        """
+        :returns: the geometry as a WKT string
+        """
+        return self.polygon.wkt
 
     def get_one_rupture(self, rupture_mutex=False):
         """
@@ -184,6 +191,6 @@ class NonParametricSeismicSource(BaseSeismicSource):
         idx = numpy.random.choice(range(num_ruptures), p=weights)
         for i, rup in enumerate(self.iter_ruptures()):
             if i == idx:
-                rup.serial = self.serial + i
+                rup.rup_id = self.serial + i
                 rup.idx = idx
                 return rup
