@@ -1,6 +1,6 @@
 # coding: utf-8
 # The Hazard Library
-# Copyright (C) 2012-2019 GEM Foundation
+# Copyright (C) 2012-2020 GEM Foundation
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -24,12 +24,12 @@ import abc
 import numpy
 import math
 import itertools
-import collections
+import toml
 from openquake.baselib import general
-from openquake.baselib.slots import with_slots
 from openquake.hazardlib import geo, contexts
 from openquake.hazardlib.geo.nodalplane import NodalPlane
-from openquake.hazardlib.geo.mesh import RectangularMesh
+from openquake.hazardlib.geo.mesh import (
+    Mesh, RectangularMesh, surface_to_array)
 from openquake.hazardlib.geo.point import Point
 from openquake.hazardlib.geo.geodetic import geodetic_distance
 from openquake.hazardlib.near_fault import (
@@ -38,15 +38,98 @@ from openquake.hazardlib.geo.surface.base import BaseSurface
 
 U16 = numpy.uint16
 U32 = numpy.uint32
-U64 = numpy.uint64
-TWO16 = U64(2 ** 16)
-TWO32 = U64(2 ** 32)
+F32 = numpy.float32
+TWO16 = 2 ** 16
+TWO32 = 2 ** 32
 pmf_dt = numpy.dtype([('prob', float), ('occ', U32)])
-events_dt = numpy.dtype([('id', U64), ('rlz', U16)])
-classes = {}  # initialized in .init()
+events_dt = numpy.dtype([('id', U32), ('rup_id', U32), ('rlz_id', U16)])
+code2cls = {}
+
+
+def _get_rupture(dic, geom=None, trt=None):
+    # dic: a dictionary or a record
+    # geom: if any, an array with fields lon, lat, depth
+    if not code2cls:
+        code2cls.update(BaseRupture.init())
+    if geom is None:
+        lons = dic['lons']
+        mesh = numpy.zeros((3, len(lons), len(lons[0])), F32)
+        mesh[0] = dic['lons']
+        mesh[1] = dic['lats']
+        mesh[2] = dic['depths']
+    else:
+        mesh = numpy.zeros((3,) + geom.shape, F32)
+        mesh[0] = geom['lon']
+        mesh[1] = geom['lat']
+        mesh[2] = geom['depth']
+    rupture_cls, surface_cls = code2cls[dic['code']]
+    rupture = object.__new__(rupture_cls)
+    rupture.rup_id = dic['serial']
+    rupture.surface = object.__new__(surface_cls)
+    rupture.mag = dic['mag']
+    rupture.rake = dic['rake']
+    rupture.hypocenter = geo.Point(*dic['hypo'])
+    rupture.occurrence_rate = dic['occurrence_rate']
+    rupture.tectonic_region_type = trt or dic['trt']
+    if surface_cls is geo.PlanarSurface:
+        rupture.surface = geo.PlanarSurface.from_array(
+            mesh[:, 0, :])
+    elif surface_cls is geo.MultiSurface:
+        # mesh has shape (3, n, 4)
+        rupture.surface.__init__([
+            geo.PlanarSurface.from_array(mesh[:, i, :])
+            for i in range(mesh.shape[1])])
+    elif surface_cls is geo.GriddedSurface:
+        # fault surface, strike and dip will be computed
+        rupture.surface.strike = rupture.surface.dip = None
+        rupture.surface.mesh = Mesh(*mesh)
+    else:
+        # fault surface, strike and dip will be computed
+        rupture.surface.strike = rupture.surface.dip = None
+        rupture.surface.__init__(RectangularMesh(*mesh))
+    return rupture
+
+
+def from_toml(toml_str):
+    """
+    :param toml_str: a string in TOML format
+    :returns: a rupture instance
+    """
+    return _get_rupture(toml.loads(toml_str))
+
+
+def float5(x):
+    # a float with 5 digits
+    return round(float(x), 5)
+
+
+def _fixfloat32(dic):
+    # work around a TOML/numpy issue
+    for k, v in dic.items():
+        if isinstance(v, F32):
+            dic[k] = float5(v)
+        elif isinstance(v, tuple):
+            dic[k] = [float5(x) for x in v]
+        elif isinstance(v, numpy.ndarray):
+            dic[k] = [[float5(y) for y in x] for x in v]
+
+
+def to_toml(rup):
+    """
+    :param rup: a rupture instance
+    :returns: a TOML string
+    """
+    return toml.dumps(rup.todict())
+
+
+def to_checksum(cls1, cls2):
+    """
+    Convert a pair of classes into a numeric code (uint8)
+    """
+    names = '%s,%s' % (cls1.__name__, cls2.__name__)
+    return sum(map(ord, names)) % 256
 
 
-@with_slots
 class BaseRupture(metaclass=abc.ABCMeta):
     """
     Rupture object represents a single earthquake rupture.
@@ -74,31 +157,30 @@ class BaseRupture(metaclass=abc.ABCMeta):
     NB: if you want to convert the rupture into XML, you should set the
     attribute surface_nodes to an appropriate value.
     """
-    _slots_ = '''mag rake tectonic_region_type hypocenter surface
-    rupture_slip_direction weight'''.split()
-    serial = 0  # set to a value > 0 by the engine
+    rup_id = 0  # set to a value > 0 by the engine
     _code = {}
-    types = {}
 
     @classmethod
     def init(cls):
         """
-        Initialize the class dictionaries `._code` and .`types` encoding the
+        Initialize the class dictionary `._code` by encoding the
         bidirectional correspondence between an integer in the range 0..255
         (the code) and a pair of classes (rupture_class, surface_class).
         This is useful when serializing the rupture to and from HDF5.
+        :returns: {code: pair of classes}
         """
-        rupture_classes = [BaseRupture] + list(get_subclasses(BaseRupture))
-        surface_classes = list(get_subclasses(BaseSurface))
-        for cl in rupture_classes + surface_classes:
-            classes[cl.__name__] = cl
-        n = 0
+        rupture_classes = [BaseRupture] + list(
+            general.gen_subclasses(BaseRupture))
+        surface_classes = list(general.gen_subclasses(BaseSurface))
+        code2cls = {}
         for rup, sur in itertools.product(rupture_classes, surface_classes):
-            cls._code[rup, sur] = n
-            cls.types[n] = rup, sur
-            n += 1
-        if n >= 256:
-            raise ValueError('Too many rupture codes: %d' % n)
+            chk = to_checksum(rup, sur)
+            if chk in code2cls and code2cls[chk] != (rup, sur):
+                raise ValueError('Non-unique checksum %d for %s, %s' %
+                                 (chk, rup, sur))
+            cls._code[rup, sur] = chk
+            code2cls[chk] = rup, sur
+        return code2cls
 
     def __init__(self, mag, rake, tectonic_region_type, hypocenter,
                  surface, rupture_slip_direction=None, weight=None):
@@ -121,6 +203,25 @@ class BaseRupture(metaclass=abc.ABCMeta):
     get_probability_no_exceedance = (
         contexts.RuptureContext.get_probability_no_exceedance)
 
+    def todict(self):
+        """
+        :returns: a representation of the rupture as a dict
+        """
+        if not code2cls:
+            code2cls.update(BaseRupture.init())
+        hypo = self.hypocenter.x, self.hypocenter.y, self.hypocenter.z
+        mesh = surface_to_array(self.surface)  # shape (3, sy, sz)
+        sy, sz = mesh.shape[1:]
+        dic = {'serial': int(self.rup_id),
+               'mag': self.mag, 'rake': self.rake, 'hypo': hypo,
+               'trt': self.tectonic_region_type,
+               'code': self.code, 'occurrence_rate': self.occurrence_rate,
+               'rupture_cls': self.__class__.__name__,
+               'surface_cls': self.surface.__class__.__name__,
+               'lons': mesh[0], 'lats': mesh[1], 'depths': mesh[2]}
+        _fixfloat32(dic)
+        return dic
+
     def sample_number_of_occurrences(self, n=1):
         """
         Randomly sample number of occurrences from temporal occurrence model
@@ -189,7 +290,6 @@ class NonParametricProbabilisticRupture(BaseRupture):
         return n_occ
 
 
-@with_slots
 class ParametricProbabilisticRupture(BaseRupture):
     """
     :class:`Rupture` associated with an occurrence rate and a temporal
@@ -204,9 +304,6 @@ class ParametricProbabilisticRupture(BaseRupture):
     :raises ValueError:
         If occurrence rate is not positive.
     """
-    _slots_ = BaseRupture._slots_ + [
-        'occurrence_rate', 'temporal_occurrence_model']
-
     def __init__(self, mag, rake, tectonic_region_type, hypocenter, surface,
                  occurrence_rate, temporal_occurrence_model,
                  rupture_slip_direction=None):
@@ -399,11 +496,47 @@ class ParametricProbabilisticRupture(BaseRupture):
         return cdpp
 
 
-def get_subclasses(cls):
-    for subclass in cls.__subclasses__():
-        yield subclass
-        for ssc in get_subclasses(subclass):
-            yield ssc
+class PointSurface:
+    """
+    A fake surface used in PointRuptures
+    """
+    def __init__(self, hypocenter):
+        self.hypocenter = hypocenter
+
+    def get_strike(self):
+        return 0
+
+    def get_dip(self):
+        return 0
+
+    def get_top_edge_depth(self):
+        return self.hypocenter.depth
+
+    def get_width(self):
+        return 0
+
+    def get_closest_points(self, mesh):
+        return mesh
+
+    def __bool__(self):
+        return False
+
+
+class PointRupture(ParametricProbabilisticRupture):
+    """
+    A rupture coming from a far away PointSource, so that the finite
+    size effects can be neglected.
+    """
+    def __init__(self, mag, tectonic_region_type, hypocenter,
+                 occurrence_rate, temporal_occurrence_model):
+        self.mag = mag
+        self.rake = 0
+        self.tectonic_region_type = tectonic_region_type
+        self.hypocenter = hypocenter
+        self.occurrence_rate = occurrence_rate
+        self.temporal_occurrence_model = temporal_occurrence_model
+        self.surface = PointSurface(hypocenter)
+        self.weight = None  # no mutex
 
 
 def get_geom(surface, is_from_fault_source, is_multi_surface,
@@ -483,19 +616,20 @@ class ExportedRupture(object):
     Simplified Rupture class with attributes rupid, events_by_ses, indices
     and others, used in export.
 
-    :param rupid: rupture serial ID
+    :param rupid: rupture rup_id ID
     :param events_by_ses: dictionary ses_idx -> event records
     :param indices: site indices
     """
-    def __init__(self, rupid, events_by_ses, indices=None):
+    def __init__(self, rupid, n_occ, events_by_ses, indices=None):
         self.rupid = rupid
+        self.n_occ = n_occ
         self.events_by_ses = events_by_ses
         self.indices = indices
 
 
 def get_eids(rup_array, samples_by_grp, num_rlzs_by_grp):
     """
-    :param rup_array: a composite array with fields serial, n_occ and grp_id
+    :param rup_array: a composite array with fields rup_id, n_occ and grp_id
     :param samples_by_grp: a dictionary grp_id -> samples
     :param num_rlzs_by_grp: a dictionary grp_id -> num_rlzs
     """
@@ -505,7 +639,7 @@ def get_eids(rup_array, samples_by_grp, num_rlzs_by_grp):
         samples = samples_by_grp[grp_id]
         num_rlzs = num_rlzs_by_grp[grp_id]
         num_events = rup['n_occ'] if samples > 1 else rup['n_occ'] * num_rlzs
-        eids = TWO32 * U64(rup['serial']) + numpy.arange(num_events, dtype=U64)
+        eids = numpy.arange(num_events, dtype=U32)
         all_eids.append(eids)
     return numpy.concatenate(all_eids)
 
@@ -516,20 +650,23 @@ class EBRupture(object):
     object, containing an array of site indices affected by the rupture,
     as well as the IDs of the corresponding seismic events.
     """
-    def __init__(self, rupture, srcidx, grp_id, n_occ, samples=1):
-        assert rupture.serial  # sanity check
+    def __init__(self, rupture, srcidx, grp_id, n_occ, samples=1, id=None):
+        # NB: when reading an exported ruptures.xml the rup_id will be 0
+        # for the first rupture; it used to be the seed instead
+        assert rupture.rup_id >= 0  # sanity check
         self.rupture = rupture
         self.srcidx = srcidx
         self.grp_id = grp_id
         self.n_occ = n_occ
         self.samples = samples
+        self.id = id  # id of the rupture on the DataStore, to be overridden
 
     @property
-    def serial(self):
+    def rup_id(self):
         """
         Serial number of the rupture
         """
-        return self.rupture.serial
+        return self.rupture.rup_id
 
     def get_eids_by_rlz(self, rlzs_by_gsim):
         """
@@ -543,67 +680,33 @@ class EBRupture(object):
         if self.samples == 1:  # full enumeration or akin to it
             for rlzs in rlzs_by_gsim.values():
                 for rlz in rlzs:
-                    dic[rlz] = numpy.arange(j, j + self.n_occ, dtype=U64) + (
-                        TWO32 * U64(self.serial))
+                    dic[rlz] = numpy.arange(j, j + self.n_occ, dtype=U32)
                     j += self.n_occ
         else:  # associated eids to the realizations
             rlzs = numpy.concatenate(list(rlzs_by_gsim.values()))
             assert len(rlzs) == self.samples, (len(rlzs), self.samples)
             histo = general.random_histogram(
-                self.n_occ, self.samples, self.serial)
+                self.n_occ, self.samples, self.rup_id)
             for rlz, n in zip(rlzs, histo):
-                dic[rlz] = numpy.arange(j, j + n, dtype=U64) + (
-                    TWO32 * U64(self.serial))
+                dic[rlz] = numpy.arange(j, j + n, dtype=U32)
                 j += n
         return dic
 
-    def get_events(self, rlzs_by_gsim):
-        """
-        :returns: an array of events with fields eid, rlz
-        """
-        all_eids, rlzs = [], []
-        for rlz, eids in self.get_eids_by_rlz(rlzs_by_gsim).items():
-            all_eids.extend(eids)
-            rlzs.extend([rlz] * len(eids))
-        return numpy.fromiter(zip(all_eids, rlzs), events_dt)
-
     def get_eids(self, num_rlzs):
         """
         :param num_rlzs: the number of realizations for the given group
         :returns: an array of event IDs
         """
         num_events = self.n_occ if self.samples > 1 else self.n_occ * num_rlzs
-        return TWO32 * U64(self.serial) + numpy.arange(num_events, dtype=U64)
+        return numpy.arange(num_events, dtype=U32)
 
-    def get_events_by_ses(self, events, num_ses):
-        """
-        :returns: a dictionary ses index -> events array
-        """
-        numpy.random.seed(self.serial)
-        sess = numpy.random.choice(num_ses, len(events)) + 1
-        events_by_ses = collections.defaultdict(list)
-        for ses, event in zip(sess, events):
-            events_by_ses[ses].append(event)
-        for ses in events_by_ses:
-            events_by_ses[ses] = numpy.array(events_by_ses[ses])
-        return events_by_ses
-
-    def get_ses_by_eid(self, rlzs_by_gsim, num_ses):
-        events = self.get_events(rlzs_by_gsim)
-        numpy.random.seed(self.serial)
-        sess = numpy.random.choice(num_ses, len(events)) + 1
-        return dict(zip(events['id'], sess))
-
-    def export(self, mesh, rlzs_by_gsim, num_ses):
+    def export(self, events_by_ses):
         """
         Yield :class:`Rupture` objects, with all the
         attributes set, suitable for export in XML format.
         """
         rupture = self.rupture
-        events = self.get_events(rlzs_by_gsim)
-        events_by_ses = self.get_events_by_ses(events, num_ses)
-        new = ExportedRupture(self.serial, events_by_ses)
-        new.mesh = mesh[()]
+        new = ExportedRupture(self.id, self.n_occ, events_by_ses)
         if isinstance(rupture.surface, geo.ComplexFaultSurface):
             new.typology = 'complexFaultsurface'
         elif isinstance(rupture.surface, geo.SimpleFaultSurface):
@@ -642,4 +745,44 @@ class EBRupture(object):
 
     def __repr__(self):
         return '<%s %d[%d]>' % (
-            self.__class__.__name__, self.serial, self.n_occ)
+            self.__class__.__name__, self.rup_id, self.n_occ)
+
+
+class RuptureProxy(object):
+    """
+    A proxy for a rupture record.
+
+    :param rec: a record with the rupture parameters
+    :param nsites: approx number of sites affected by the rupture
+    :param samples: how many times the rupture is sampled
+    """
+    def __init__(self, rec, nsites=None, samples=1):
+        self.rec = rec
+        self.nsites = nsites
+        self.samples = samples
+
+    @property
+    def weight(self):
+        """
+        :returns:
+            heuristic weight for the underlying rupture, depending on the
+            number of occurrences, number of samples and number of sites
+        """
+        return self.samples * self['n_occ'] * (
+            100 if self.nsites is None else max(self.nsites, 100))
+
+    def __getitem__(self, name):
+        return self.rec[name]
+
+    # NB: requires the .geom attribute to be set
+    def to_ebr(self, trt, samples):
+        """
+        :returns: EBRupture instance associated to the underlying rupture
+        """
+        # not implemented: rupture_slip_direction
+        rupture = _get_rupture(self.rec, self.geom, trt)
+        ebr = EBRupture(rupture, self.rec['srcidx'], self.rec['grp_id'],
+                        self.rec['n_occ'], samples)
+        ebr.id = self.rec['id']
+        ebr.e0 = self.rec['e0']
+        return ebr
