@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2015-2019 GEM Foundation
+# Copyright (C) 2015-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -18,12 +18,12 @@
 import operator
 import collections
 import pickle
-import time
+import copy
 import logging
 import numpy
 
 from openquake.baselib import hdf5
-from openquake.baselib.general import groupby
+from openquake.baselib.general import groupby, block_splitter
 from openquake.baselib.node import context, striptag, Node
 from openquake.hazardlib import geo, mfd, pmf, source, tom
 from openquake.hazardlib import valid, InvalidFile
@@ -31,13 +31,58 @@ from openquake.hazardlib.tom import PoissonTOM
 from openquake.hazardlib.source import NonParametricSeismicSource
 
 U32 = numpy.uint32
-U64 = numpy.uint64
 F32 = numpy.float32
+EPSILON = 1E-12
 source_dt = numpy.dtype([('srcidx', U32), ('num_ruptures', U32),
                          ('pik', hdf5.vuint8)])
 
 
-class SourceGroup(collections.Sequence):
+def extract_dupl(values):
+    """
+    :param values: a sequence of values
+    :returns: the duplicated values
+    """
+    c = collections.Counter(values)
+    return [value for value, counts in c.items() if counts > 1]
+
+
+def fix_dupl(dist, fname=None, lineno=None):
+    """
+    Fix the distribution if it contains identical values or raise an error.
+
+    :param dist:
+        a list of pairs [(prob, value)...] for a hypocenter or nodal plane dist
+    :param fname:
+        the file which is being read; if it is None, it means you are writing
+        the distribution: in that case raise an error for duplicated values
+    param lineno:
+        the line number of the file which is being read (None in writing mode)
+    """
+    n = len(dist)
+    values = collections.defaultdict(float)  # dict value -> probability
+    # value can be a scalar (hypocenter depth) or a triple
+    # (strike, dip, rake) for a nodal plane distribution
+    got = []
+    for prob, value in dist:
+        values[value] += prob
+        got.append(value)
+    if len(values) < n:
+        if fname is None:  # when called from the sourcewriter
+            raise ValueError('There are repeated values in %s' % got)
+        else:
+            logging.warning('There were repeated values %s in %s:%s',
+                            extract_dupl(got), fname, lineno)
+            assert abs(sum(values.values()) - 1) < EPSILON  # sanity check
+            newdist = sorted([(p, v) for v, p in values.items()])
+            if isinstance(newdist[0][1], tuple):  # nodal planes
+                newdist = [(p, geo.nodalplane.NodalPlane(*v))
+                           for p, v in newdist]
+            # run hazardlib/tests/data/context/job.ini to check this;
+            # you will get [(0.2, 6.0), (0.2, 8.0), (0.2, 10.0), (0.4, 2.0)]
+            dist[:] = newdist
+
+
+class SourceGroup(collections.abc.Sequence):
     """
     A container for the following parameters:
 
@@ -63,12 +108,6 @@ class SourceGroup(collections.Sequence):
     :param id:
         an optional numeric ID (default 0) set by the engine and used
         when serializing SourceModels to HDF5
-    :param eff_ruptures:
-        the number of ruptures contained in the group; if -1,
-        the number is unknown and has to be computed by using
-        get_set_num_ruptures
-    :param tot_ruptures:
-        the potential maximum number of ruptures contained in the group
     :param temporal_occurrence_model:
         A temporal occurrence model controlling the source group occurrence
     :param cluster:
@@ -76,6 +115,8 @@ class SourceGroup(collections.Sequence):
         to what used by the USGS for the New Madrid in the 2008 National
         Hazard Model.
     """
+    changes = 0  # set in apply_uncertainty
+
     @classmethod
     def collect(cls, sources):
         """
@@ -98,9 +139,8 @@ class SourceGroup(collections.Sequence):
 
     def __init__(self, trt, sources=None, name=None, src_interdep='indep',
                  rup_interdep='indep', grp_probability=None,
-                 min_mag={'default': 0}, max_mag=None, id=0, eff_ruptures=-1,
-                 tot_ruptures=0, temporal_occurrence_model=None,
-                 cluster=False):
+                 min_mag={'default': 0}, max_mag=None,
+                 temporal_occurrence_model=None, cluster=False):
         # checks
         self.trt = trt
         self.sources = []
@@ -111,13 +151,10 @@ class SourceGroup(collections.Sequence):
         self.grp_probability = grp_probability
         self.min_mag = min_mag
         self.max_mag = max_mag
-        self.id = id
-        self.tot_ruptures = tot_ruptures  # updated in .update(src)
         if sources:
             for src in sorted(sources, key=operator.attrgetter('source_id')):
                 self.update(src)
-        self.source_model = None  # to be set later, in CompositionInfo
-        self.eff_ruptures = eff_ruptures  # set later by get_rlzs_assoc
+        self.source_model = None  # to be set later, in FullLogicTree
         self.temporal_occurrence_model = temporal_occurrence_model
         self.cluster = cluster
         # check weights in case of mutually exclusive ruptures
@@ -135,6 +172,13 @@ class SourceGroup(collections.Sequence):
         return (self.cluster or self.src_interdep == 'mutex' or
                 self.rup_interdep == 'mutex')
 
+    @property
+    def weight(self):
+        """
+        :returns: total weight of the underlying sources
+        """
+        return sum(src.weight for src in self)
+
     def _check_init_variables(self, src_list, name,
                               src_interdep, rup_interdep):
         if src_interdep not in ('indep', 'mutex'):
@@ -169,6 +213,8 @@ class SourceGroup(collections.Sequence):
             src.tectonic_region_type, self.trt)
         if not src.min_mag:  # if not set already
             src.min_mag = self.min_mag.get(self.trt) or self.min_mag['default']
+            if not src.get_mags():  # filtered out
+                return
         # checking mutex ruptures
         if (not isinstance(src, NonParametricSeismicSource) and
                 self.rup_interdep == 'mutex'):
@@ -176,20 +222,30 @@ class SourceGroup(collections.Sequence):
             msg += "modelled using non-parametric sources"
             raise ValueError(msg)
 
-        nr = get_set_num_ruptures(src)
-        if nr == 0:  # the minimum_magnitude filters all ruptures
-            return
-        self.tot_ruptures += nr
         self.sources.append(src)
         _, max_mag = src.get_min_max_mag()
         prev_max_mag = self.max_mag
         if prev_max_mag is None or max_mag > prev_max_mag:
             self.max_mag = max_mag
 
+    def split(self, maxweight):
+        """
+        Split the group in subgroups with weight <= maxweight, unless it
+        it atomic.
+        """
+        if self.atomic:
+            return [self]
+        out = []
+        for block in block_splitter(
+                self, maxweight, operator.attrgetter('weight')):
+            sg = copy.copy(self)
+            sg.sources = block
+            out.append(sg)
+        return out
+
     def __repr__(self):
-        return '<%s #%d %s, %d source(s), %d effective rupture(s)>' % (
-            self.__class__.__name__, self.id, self.trt,
-            len(self.sources), self.eff_ruptures)
+        return '<%s %s, %d source(s)>' % (
+            self.__class__.__name__, self.trt, len(self.sources))
 
     def __lt__(self, other):
         """
@@ -219,10 +275,8 @@ class SourceGroup(collections.Sequence):
             lst.append((src.id, src.num_ruptures,
                         numpy.frombuffer(buf, numpy.uint8)))
         attrs = dict(
-            id=self.id,
             trt=self.trt,
             name=self.name or '',
-            eff_ruptures=self.eff_ruptures,
             src_interdep=self.src_interdep,
             rup_interdep=self.rup_interdep,
             grp_probability=self.grp_probability or '')
@@ -235,34 +289,6 @@ class SourceGroup(collections.Sequence):
             self.sources.append(pickle.loads(memoryview(row['pik'])))
 
 
-def get_set_num_ruptures(src):
-    """
-    Extract the number of ruptures and set it
-    """
-    if not src.num_ruptures:
-        t0 = time.time()
-        src.num_ruptures = src.count_ruptures()
-        dt = time.time() - t0
-        clsname = src.__class__.__name__
-        if dt > 10:
-            if 'Area' in clsname:
-                logging.warning(
-                    '%s.count_ruptures took %d seconds, perhaps the '
-                    'area discretization is too small', src, dt)
-            elif 'ComplexFault' in clsname:
-                logging.warning(
-                    '%s.count_ruptures took %d seconds, perhaps the '
-                    'complex_fault_mesh_spacing is too small', src, dt)
-            elif 'SimpleFault' in clsname:
-                logging.warning(
-                    '%s.count_ruptures took %d seconds, perhaps the '
-                    'rupture_mesh_spacing is too small', src, dt)
-            else:
-                # multiPointSource
-                logging.warning('count_ruptures %s took %d seconds', src, dt)
-    return src.num_ruptures
-
-
 def split_coords_2d(seq):
     """
     :param seq: a flat list with lons and lats
@@ -325,8 +351,7 @@ class RuptureConverter(object):
 
         :param node: a node representing a rupture
         """
-        convert = getattr(self, 'convert_' + striptag(node.tag))
-        return convert(node)
+        return getattr(self, 'convert_' + striptag(node.tag))(node)
 
     def geo_line(self, edge):
         """
@@ -434,11 +459,11 @@ class RuptureConverter(object):
         """
         mag, rake, hypocenter = self.get_mag_rake_hypo(node)
         with context(self.fname, node):
-            surfaces = [node.complexFaultGeometry]
+            [surface] = node.getnodes('complexFaultGeometry')
         rupt = source.rupture.BaseRupture(
             mag=mag, rake=rake, tectonic_region_type=None,
             hypocenter=hypocenter,
-            surface=self.convert_surfaces(surfaces))
+            surface=self.convert_surfaces([surface]))
         return rupt
 
     def convert_singlePlaneRupture(self, node):
@@ -500,7 +525,7 @@ class RuptureConverter(object):
             coll[grp_id] = ebrs = []
             for node in grpnode:
                 rup = self.convert_node(node)
-                rup.serial = int(node['id'])
+                rup.rup_id = int(node['id'])
                 sesnodes = node.stochasticEventSets
                 n = 0  # number of events
                 for sesnode in sesnodes:
@@ -515,11 +540,12 @@ class SourceConverter(RuptureConverter):
     """
     Convert sources from valid nodes into Hazardlib objects.
     """
-    def __init__(self, investigation_time=50., rupture_mesh_spacing=10.,
+    def __init__(self, investigation_time=50., rupture_mesh_spacing=5.,
                  complex_fault_mesh_spacing=None, width_of_mfd_bin=1.0,
                  area_source_discretization=None,
                  minimum_magnitude={'default': 0},
-                 spinning_floating=True, source_id=None):
+                 spinning_floating=True, source_id=None,
+                 discard_trts=''):
         self.investigation_time = investigation_time
         self.area_source_discretization = area_source_discretization
         self.minimum_magnitude = minimum_magnitude
@@ -529,6 +555,23 @@ class SourceConverter(RuptureConverter):
         self.width_of_mfd_bin = width_of_mfd_bin
         self.spinning_floating = spinning_floating
         self.source_id = source_id
+        self.discard_trts = discard_trts
+
+    def convert_node(self, node):
+        """
+        Convert the given source node into a hazardlib source, depending
+        on the node tag.
+
+        :param node: a node representing a source or a SourceGroup
+        """
+        trt = node.attrib.get('tectonicRegion')
+        if trt and trt in self.discard_trts:
+            return
+        obj = getattr(self, 'convert_' + striptag(node.tag))(node)
+        source_id = getattr(obj, 'source_id', '')
+        if self.source_id and source_id and source_id not in self.source_id:
+            return
+        return obj
 
     def get_tom(self, node):
         """
@@ -574,22 +617,13 @@ class SourceConverter(RuptureConverter):
                     magnitudes=~mfd_node.magnitudes,
                     occurrence_rates=~mfd_node.occurRates)
             elif mfd_node.tag.endswith('YoungsCoppersmithMFD'):
-                if "totalMomentRate" in mfd_node.attrib.keys():
-                    # Return Youngs & Coppersmith from the total moment rate
-                    return mfd.YoungsCoppersmith1985MFD.from_total_moment_rate(
-                        min_mag=mfd_node["minMag"], b_val=mfd_node["bValue"],
-                        char_mag=mfd_node["characteristicMag"],
-                        total_moment_rate=mfd_node["totalMomentRate"],
-                        bin_width=mfd_node["binWidth"])
-                elif "characteristicRate" in mfd_node.attrib.keys():
-                    # Return Youngs & Coppersmith from the total moment rate
-                    return mfd.YoungsCoppersmith1985MFD.\
-                        from_characteristic_rate(
-                            min_mag=mfd_node["minMag"],
-                            b_val=mfd_node["bValue"],
-                            char_mag=mfd_node["characteristicMag"],
-                            char_rate=mfd_node["characteristicRate"],
-                            bin_width=mfd_node["binWidth"])
+                return mfd.YoungsCoppersmith1985MFD(
+                    min_mag=mfd_node["minMag"],
+                    b_val=mfd_node["bValue"],
+                    char_mag=mfd_node["characteristicMag"],
+                    char_rate=mfd_node.get("characteristicRate"),
+                    total_moment_rate=mfd_node.get("totalMomentRate"),
+                    bin_width=mfd_node["binWidth"])
             elif mfd_node.tag.endswith('multiMFD'):
                 return mfd.multi_mfd.MultiMFD.from_node(
                     mfd_node, self.width_of_mfd_bin)
@@ -602,11 +636,14 @@ class SourceConverter(RuptureConverter):
         :returns: a :class:`openquake.hazardlib.geo.NodalPlane` instance
         """
         with context(self.fname, node):
+            npnode = node.nodalPlaneDist
             npdist = []
-            for np in node.nodalPlaneDist:
+            for np in npnode:
                 prob, strike, dip, rake = (
                     np['probability'], np['strike'], np['dip'], np['rake'])
                 npdist.append((prob, geo.NodalPlane(strike, dip, rake)))
+        with context(self.fname, npnode):
+            fix_dupl(npdist, self.fname, npnode.lineno)
             if not self.spinning_floating:
                 npdist = [(1, npdist[0][1])]  # consider the first nodal plane
             return pmf.PMF(npdist)
@@ -620,11 +657,13 @@ class SourceConverter(RuptureConverter):
         :returns: a :class:`openquake.hazardlib.pmf.PMF` instance
         """
         with context(self.fname, node):
-            hcdist = [(hd['probability'], hd['depth'])
-                      for hd in node.hypoDepthDist]
+            hdnode = node.hypoDepthDist
+            hddist = [(hd['probability'], hd['depth']) for hd in hdnode]
+        with context(self.fname, hdnode):
+            fix_dupl(hddist, self.fname, hdnode.lineno)
             if not self.spinning_floating:  # consider the first hypocenter
-                hcdist = [(1, hcdist[0][1])]
-            return pmf.PMF(hcdist)
+                hddist = [(1, hddist[0][1])]
+            return pmf.PMF(hddist)
 
     def convert_areaSource(self, node):
         """
@@ -858,9 +897,9 @@ class SourceConverter(RuptureConverter):
                 assert hasattr(sg, 'occurrence_rate')
         #
         for src_node in node:
-            if self.source_id and self.source_id != src_node['id']:
-                continue  # filter by source_id
             src = self.convert_node(src_node)
+            if src is None:  # filtered out by source_id
+                continue
             # transmit the group attributes to the underlying source
             for attr, value in grp_attrs.items():
                 if attr == 'tectonicRegion':
@@ -1010,20 +1049,3 @@ def update_source_model(sm_node, fname):
         others.sort(key=lambda src: (src.tag, src['id']))
         i, sources = _pointsources2multipoints(psrcs, i)
         group.nodes = sources + others
-
-
-def to_python(fname, converter):
-    """
-    Convert a source model .hdf5 file into a :class:`SourceModel` object
-    """
-    with hdf5.File(fname, 'r') as f:
-        source_model = f['/']
-    for sg in source_model:
-        for src in sg:
-            if hasattr(src, 'mfd'):
-                # multipoint source
-                # src.tom = converter.tom
-                kwargs = getattr(src.mfd, 'kwargs', {})
-                if 'bin_width' not in kwargs:
-                    kwargs['bin_width'] = [converter.width_of_mfd_bin]
-    return source_model
