@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2019 GEM Foundation
+# Copyright (C) 2012-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -26,12 +26,9 @@ import math
 import numpy
 import scipy.stats
 
-from openquake.baselib.slots import with_slots
-
 registry = {}
 
 
-@with_slots
 class BaseTOM(metaclass=abc.ABCMeta):
     """
     Base class for temporal occurrence model.
@@ -41,8 +38,6 @@ class BaseTOM(metaclass=abc.ABCMeta):
     :raises ValueError:
         If ``time_span`` is not positive.
     """
-    _slots_ = ['time_span', 'occurrence_rate']
-
     @classmethod
     def __init_subclass__(cls):
         registry[cls.__name__] = cls
@@ -90,7 +85,6 @@ class BaseTOM(metaclass=abc.ABCMeta):
         given in the constructor.
         """
 
-@with_slots
 class FatedTOM(BaseTOM):
 
     def __init__(self, time_span, occurrence_rate=None):
@@ -101,18 +95,18 @@ class FatedTOM(BaseTOM):
         return 1
 
     def get_probability_n_occurrences(self, occurrence_rate, num):
-        if num != 1: 
+        if num != 1:
             return 0
         else:
             return 1
-        
+
     def sample_number_of_occurrences(self, seeds=None):
         return 1
 
     def get_probability_no_exceedance(self, occurrence_rate, poes):
         return 1-poes
 
-@with_slots
+
 class PoissonTOM(BaseTOM):
     """
     Poissonian temporal occurrence model.
@@ -182,12 +176,11 @@ class PoissonTOM(BaseTOM):
         :param occurrence_rate:
             The average number of events per year.
         :param poes:
-            2D numpy array containing conditional probabilities that the 
+            2D numpy array containing conditional probabilities that the
             rupture occurrence causes a ground shaking value exceeding a
             ground motion level at a site. First dimension represent sites,
             second dimension intensity measure levels. ``poes`` can be obtained
-            calling the :meth:`method
-            <openquake.hazardlib.gsim.base.GroundShakingIntensityModel.get_poes>`.
+            calling the :func:`func <openquake.hazardlib.gsim.base.get_poes>`.
         :return:
             2D numpy array containing probabilities of no exceedance. First
             dimension represents sites, second dimensions intensity measure
