@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2013-2019 GEM Foundation
+# Copyright (C) 2013-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -20,6 +20,7 @@
 Validation library for the engine, the desktop tools, and anything else
 """
 
+import os
 import re
 import ast
 import logging
@@ -29,7 +30,7 @@ import numpy
 from openquake.baselib.general import distinct
 from openquake.baselib import hdf5
 from openquake.hazardlib import imt, scalerel, gsim, pmf, site
-from openquake.hazardlib.gsim import registry
+from openquake.hazardlib.gsim.base import registry, gsim_aliases
 from openquake.hazardlib.calc import disagg
 from openquake.hazardlib.calc.filters import IntegrationDistance
 
@@ -63,6 +64,7 @@ class FromFile(object):
     """
     DEFINED_FOR_INTENSITY_MEASURE_TYPES = set()
     REQUIRES_SITES_PARAMETERS = set()
+    REQUIRES_DISTANCES = set()
     kwargs = {}
 
     def init(self):
@@ -72,17 +74,41 @@ class FromFile(object):
         return '[FromFile]'
 
 
-# more tests are in tests/valid_test.py
-def gsim(value):
+def to_toml(uncertainty):
     """
-    Convert a string in TOML format into a GSIM instance
+    Converts an uncertainty node into a TOML string
+    """
+    if hasattr(uncertainty, 'attrib'):  # is a node
+        text = uncertainty.text.strip()
+        kvs = uncertainty.attrib.items()
+    else:  # is a string
+        text = uncertainty.strip()
+        kvs = []
+    text = gsim_aliases.get(text, text)  # use the gsim alias if any
+    if not text.startswith('['):  # a bare GSIM name was passed
+        text = '[%s]' % text
+    for k, v in kvs:
+        try:
+            v = ast.literal_eval(v)
+        except (SyntaxError, ValueError):
+            v = repr(v)
+        text += '\n%s = %s' % (k, v)
+    return text
 
-    >>> gsim('[BooreAtkinson2011]')
+
+# more tests are in tests/valid_test.py
+def gsim(value, basedir=''):
+    """
+    Convert a string into a GSIM instance
+
+    >>> gsim('BooreAtkinson2011')
     [BooreAtkinson2011]
     """
-    if not value.startswith('['):  # assume the GSIM name
-        value = '[%s]' % value
+    value = to_toml(value)  # convert to TOML
     [(gsim_name, kwargs)] = toml.loads(value).items()
+    for k, v in kwargs.items():
+        if k.endswith(('_file', '_table')):
+            kwargs[k] = os.path.normpath(os.path.join(basedir, v))
     minimum_distance = float(kwargs.pop('minimum_distance', 0))
     if gsim_name == 'FromFile':
         return FromFile()
@@ -90,7 +116,11 @@ def gsim(value):
         gsim_class = registry[gsim_name]
     except KeyError:
         raise ValueError('Unknown GSIM: %s' % gsim_name)
-    gs = gsim_class(**kwargs)
+    if basedir:
+        gs = gsim_class(**kwargs)
+    else:
+        gs = object.__new__(gsim_class)
+        gs.kwargs = kwargs
     gs._toml = '\n'.join(line.strip() for line in value.splitlines())
     gs.minimum_distance = minimum_distance
     return gs
@@ -341,14 +371,12 @@ def namelist(value):
     ['a1', 'b_2', '_c']
 
     >>> namelist('a1 b_2 1c')
-    Traceback (most recent call last):
-        ...
-    ValueError: List of names containing an invalid name: 1c
+    ['a1', 'b_2', '1c']
     """
     names = value.replace(',', ' ').split()
     for n in names:
         try:
-            name(n)
+            source_id(n)
         except ValueError:
             raise ValueError('List of names containing an invalid name:'
                              ' %s' % n)
@@ -538,14 +566,14 @@ def positivefloats(value):
     return floats
 
 
-def floats32(value):
+def floats(value):
     """
     :param value:
         string of whitespace separated floats
     :returns:
-        an array of 32 bit floats
+        a list of floats
     """
-    return numpy.float32(value.split())
+    return list(map(float, value.split()))
 
 
 _BOOL_DICT = {
@@ -580,9 +608,19 @@ def boolean(value):
         raise ValueError('Not a boolean: %s' % value)
 
 
-range01 = FloatRange(0, 1)
+def range01(value):
+    """
+    :param value: a string convertible to a float in the range 0..1
+    """
+    val = value.lower()
+    if val == 'true':
+        return 1.
+    elif val == 'false':
+        return 0.
+    return FloatRange(0, 1)(val)
+
+
 probability = FloatRange(0, 1)
-probability.__name__ = 'probability'
 
 
 def probabilities(value, rows=0, cols=0):
@@ -653,8 +691,10 @@ def intensity_measure_type(value):
 def intensity_measure_types(value):
     """
     :param value: input string
-    :returns: non-empty list of Intensity Measure Type objects
+    :returns: non-empty list of ordered Intensity Measure Type objects
 
+    >>> intensity_measure_types('')
+    []
     >>> intensity_measure_types('PGA')
     ['PGA']
     >>> intensity_measure_types('PGA, SA(1.00)')
@@ -663,20 +703,18 @@ def intensity_measure_types(value):
     Traceback (most recent call last):
       ...
     ValueError: Duplicated IMTs in SA(0.1), SA(0.10)
-    >>> intensity_measure_types('SA(1), PGA')
-    Traceback (most recent call last):
-    ...
-    ValueError: The IMTs are not sorted by period: SA(1), PGA
+    >>> intensity_measure_types('PGV, SA(1), PGA')
+    ['PGA', 'PGV', 'SA(1.0)']
     """
+    if not value:
+        return []
     imts = []
     for chunk in value.split(','):
         imts.append(imt.from_string(chunk.strip()))
     sorted_imts = sorted(imts, key=lambda im: getattr(im, 'period', 1))
     if len(distinct(imts)) < len(imts):
         raise ValueError('Duplicated IMTs in %s' % value)
-    if sorted_imts != imts:
-        raise ValueError('The IMTs are not sorted by period: %s' % value)
-    return [str(imt) for imt in imts]
+    return [str(imt) for imt in sorted_imts]
 
 
 def check_levels(imls, imt, min_iml=1E-10):
@@ -723,7 +761,7 @@ def intensity_measure_types_and_levels(value):
     {'SA(0.1)': [0.1, 0.2]}
     """
     dic = dictionary(value)
-    for imt_str, imls in dic.items():
+    for imt_str, imls in list(dic.items()):
         norm_imt = str(imt.from_string(imt_str))
         if norm_imt != imt_str:
             dic[norm_imt] = imls
@@ -768,6 +806,24 @@ def logscale(x_min, x_max, n):
     return numpy.exp(delta * numpy.arange(n) / (n - 1)) * x_min
 
 
+def sqrscale(x_min, x_max, n):
+    """
+    :param x_min: minumum value
+    :param x_max: maximum value
+    :param n: number of steps
+    :returns: an array of n values from x_min to x_max in a quadratic scale
+    """
+    if not (isinstance(n, int) and n > 0):
+        raise ValueError('n must be a positive integer, got %s' % n)
+    if x_min < 0:
+        raise ValueError('x_min must be positive, got %s' % x_min)
+    if x_max <= x_min:
+        raise ValueError('x_max (%s) must be bigger than x_min (%s)' %
+                         (x_max, x_min))
+    delta = numpy.sqrt(x_max - x_min) / (n - 1)
+    return x_min + (delta * numpy.arange(n))**2
+
+
 def dictionary(value):
     """
     :param value:
@@ -823,7 +879,7 @@ def floatdict(value):
     value = ast.literal_eval(value)
     if isinstance(value, (int, float, list)):
         return {'default': value}
-    dic = {'default': value[next(iter(value))]}
+    dic = {'default': max(value.values())}
     dic.update(value)
     return dic
 
@@ -835,13 +891,77 @@ def maximum_distance(value):
     :returns:
         a IntegrationDistance mapping
     """
-    dic = floatdict(value)
-    for trt, magdists in dic.items():
-        if isinstance(magdists, list):  # could be a scalar otherwise
-            magdists.sort()  # make sure the list is sorted by magnitude
-            for mag, dist in magdists:  # validate the magnitudes
-                magnitude(mag)
-    return IntegrationDistance(dic)
+    return IntegrationDistance(MagDist.new(value).max())
+
+
+def unique_sorted(items):
+    """
+    Check that the items are unique and sorted
+    """
+    if len(set(items)) < len(items):
+        raise ValueError('Found duplicates in %s' % items)
+    elif items != sorted(items):
+        raise ValueError('%s is not ordered' % items)
+    return items
+
+
+class MagDist(dict):
+    """
+    A dictionary trt -> [(mag, dist), ...]
+    """
+    @classmethod
+    def new(cls, value):
+        """
+        :param value: string to be converted
+        :returns: MagDist dictionary
+
+        >>> md = MagDist.new('50')
+        >>> md
+        {'default': [(1, 50), (10, 50)]}
+        >>> md.max()
+        {'default': 50}
+        >>> md.interp(dict(default=['5.0', '5.1', '5.2']))
+        {'default': {'5.0': 50.0, '5.1': 50.0, '5.2': 50.0}}
+        """
+        items_by_trt = floatdict(value.replace('?', '-1'))
+        self = cls()
+        for trt, items in items_by_trt.items():
+            if isinstance(items, list):
+                self[trt] = unique_sorted(items)
+                for mag, dist in self[trt]:
+                    magnitude(mag)  # check valid magnitude
+            else:  # assume scalar distance
+                assert items == -1 or items >= 0, items
+                self[trt] = [(1, items), (10, items)]
+        return self
+
+    def interp(self, mags_by_trt):
+        """
+        :param mags_by_trt: a dictionary trt -> magnitudes as strings
+        :returns: a dictionary trt->mag->dist
+        """
+        dic = {}
+        for trt, mags in mags_by_trt.items():
+            xs, ys = zip(*self[trt])
+            if len(mags) == 1:
+                ms = [numpy.float64(mags)]
+            else:
+                ms = numpy.float64(mags)
+            dists = numpy.interp(ms, xs, ys)
+            dic[trt] = dict(zip(mags, dists))
+        return dic
+
+    def max(self):
+        """
+        :returns: a dictionary trt -> maxdist
+        """
+        return {trt: self[trt][-1][1] for trt in self}
+
+    def suggested(self):
+        """
+        :returns: True if there is a * for any TRT
+        """
+        return any(self[trt][-1][1] == -1 for trt in self)
 
 
 # ########################### SOURCES/RUPTURES ############################# #
@@ -1034,6 +1154,15 @@ def simple_slice(value):
     return (start, stop)
 
 
+def uncertainty_model(value):
+    """
+    Format whitespace in XML nodes of kind uncertaintyModel
+    """
+    if value.lstrip().startswith('['):  # TOML, do not mess with newlines
+        return value.strip()
+    return ' '.join(value.split())  # remove newlines too
+
+
 # used for the exposure validation
 cost_type = Choice('structural', 'nonstructural', 'contents',
                    'business_interruption')
@@ -1145,23 +1274,22 @@ class ParamSet(hdf5.LiteralAttrs, metaclass=MetaParamSet):
     <MyParams a='2', b=7.2>
     """
     params = {}
+    KNOWN_INPUTS = {}
 
     @classmethod
     def check(cls, dic):
         """
-        Convert a dictionary name->string into a dictionary name->value
-        by converting the string. If the name does not correspond to a
-        known parameter, just ignore it and print a warning.
+        Check if a dictionary name->string can be converted into a dictionary
+        name->value. If the name does not correspond to a known parameter,
+        print a warning.
         """
-        res = {}
         for name, text in dic.items():
             try:
                 p = getattr(cls, name)
             except AttributeError:
                 logging.warning('Ignored unknown parameter %s', name)
             else:
-                res[name] = p.validator(text)
-        return res
+                p.validator(text)
 
     @classmethod
     def from_(cls, dic):
@@ -1192,7 +1320,9 @@ class ParamSet(hdf5.LiteralAttrs, metaclass=MetaParamSet):
             try:
                 convert = getattr(self.__class__, name).validator
             except AttributeError:
-                logging.warning("The parameter '%s' is unknown, ignoring" % name)
+                if name not in self.KNOWN_INPUTS:
+                    logging.warning(
+                        "The parameter '%s' is unknown, ignoring" % name)
                 continue
             try:
                 value = convert(val)
@@ -1230,9 +1360,9 @@ class RjbEquivalent(object):
     """
     def __init__(self, filename):
         with hdf5.File(filename, 'r') as f:
-            self.repi = f['default/repi'].value  # shape D
-            self.mags = f['default/mags'].value  # shape M
-            self.reqv = f['default/reqv'].value  # shape D x M
+            self.repi = f['default/repi'][()]  # shape D
+            self.mags = f['default/mags'][()]  # shape M
+            self.reqv = f['default/reqv'][()]  # shape D x M
 
     def get(self, repi, mag):
         """
