@@ -4,7 +4,7 @@
 #
 # LICENSE
 #
-# Copyright (C) 2010-2019 GEM Foundation, G. Weatherill, M. Pagani,
+# Copyright (C) 2010-2020 GEM Foundation, G. Weatherill, M. Pagani,
 # D. Monelli.
 #
 # The Hazard Modeller's Toolkit is free software: you can redistribute
@@ -194,11 +194,11 @@ class GCMTMomentTensor(object):
         """
         Switches the reference frame to NED
         """
-        if self.ref_frame is 'USE':
+        if self.ref_frame == 'USE':
             # Rotate
             return utils.use_to_ned(self.tensor), \
                 utils.use_to_ned(self.tensor_sigma)
-        elif self.ref_frame is 'NED':
+        elif self.ref_frame == 'NED':
             # Alreadt NED
             return self.tensor, self.tensor_sigma
         else:
@@ -209,11 +209,11 @@ class GCMTMomentTensor(object):
         """
         Returns a tensor in the USE reference frame
         """
-        if self.ref_frame is 'NED':
+        if self.ref_frame == 'NED':
             # Rotate
             return utils.ned_to_use(self.tensor), \
                 utils.ned_to_use(self.tensor_sigma)
-        elif self.ref_frame is 'USE':
+        elif self.ref_frame == 'USE':
             # Already USE
             return self.tensor, self.tensor_sigma
         else:
@@ -229,7 +229,7 @@ class GCMTMomentTensor(object):
 
     def eigendecompose(self, normalise=False):
         """
-        Performs and eigendecomposition of the tensor and orders into 
+        Performs and eigendecomposition of the tensor and orders into
         descending eigenvalues
         """
         self.eigenvalues, self.eigenvectors = utils.eigendecompose(self.tensor,
@@ -247,21 +247,22 @@ class GCMTMomentTensor(object):
         # Tensor
         _, evect = utils.eigendecompose(self.tensor)
         # Rotation matrix
-        _, rot_vec = utils.eigendecompose(np.matrix([[0., 0., -1],
-                                                     [0., 0., 0.],
-                                                     [-1., 0., 0.]]))
-        rotation_matrix = (np.matrix(evect * rot_vec.T)).T
+        _, rot_vec = utils.eigendecompose(np.array([[0., 0., -1],
+                                                    [0., 0., 0.],
+                                                    [-1., 0., 0.]]))
+        rotation_matrix = (evect @ rot_vec.T).T
         if np.linalg.det(rotation_matrix) < 0.:
-            rotation_matrix *= -1.
-        flip_dc = np.matrix([[0., 0., -1.],
-                             [0., -1., 0.],
-                             [-1., 0., 0.]])
+            rotation_matrix @= -1.
+        flip_dc = np.array([[0., 0., -1.],
+                            [0., -1., 0.],
+                            [-1., 0., 0.]])
         rotation_matrices = sorted(
-            [rotation_matrix, flip_dc * rotation_matrix],
+            [rotation_matrix, flip_dc @ rotation_matrix],
             cmp=cmp_mat)
         nodal_planes = GCMTNodalPlanes()
         dip, strike, rake = [(180. / pi) * angle
-                             for angle in utils.matrix_to_euler(rotation_matrices[0])]
+                             for angle in utils.matrix_to_euler(
+                                     rotation_matrices[0])]
         # 1st Nodal Plane
         nodal_planes.nodal_plane_1 = {'strike': strike % 360,
                                       'dip': dip,
@@ -269,7 +270,8 @@ class GCMTMomentTensor(object):
 
         # 2nd Nodal Plane
         dip, strike, rake = [(180. / pi) * angle
-                             for angle in utils.matrix_to_euler(rotation_matrices[1])]
+                             for angle in utils.matrix_to_euler(
+                                     rotation_matrices[1])]
         nodal_planes.nodal_plane_2 = {'strike': strike % 360.,
                                       'dip': dip,
                                       'rake': -rake}
@@ -277,7 +279,7 @@ class GCMTMomentTensor(object):
 
     def get_principal_axes(self):
         """
-        Uses the eigendecomposition to extract the principal axes from the 
+        Uses the eigendecomposition to extract the principal axes from the
         moment tensor - returning an instance of the GCMTPrincipalAxes class
         """
         # Perform eigendecomposition - returns in order P, B, T
