@@ -4,7 +4,7 @@
 #
 # LICENSE
 #
-# Copyright (C) 2010-2019 GEM Foundation, G. Weatherill, M. Pagani,
+# Copyright (C) 2010-2020 GEM Foundation, G. Weatherill, M. Pagani,
 # D. Monelli.
 #
 # The Hazard Modeller's Toolkit is free software: you can redistribute
@@ -164,7 +164,7 @@ for reg_type in BIRD_GLOBAL_PARAMETERS:
     reg = BIRD_GLOBAL_PARAMETERS[reg_type]
 
     reg['corner_moment'] = moment_function(reg['corner_mag'])
-    if reg_type is not 'IPL':
+    if reg_type != 'IPL':
         reg['CMT_pure_event_rate'] = reg['CMT_EVENTS'] / CMT_DURATION_S
         reg['length'] = 1000.0 * reg['length']
         reg['velocity'] = (reg['velocity'] * 0.001) / SECS_PER_YEAR
@@ -244,7 +244,8 @@ class Shift(object):
         # If a regionalisation parameter file is defined then read
         # regionalisation from there - otherwise use Bird regionalisation
         if region_parameter_file:
-            self.regionalisation = yaml.load(open(region_parameter_file, 'rt'))
+            self.regionalisation = yaml.safe_load(
+                open(region_parameter_file, 'rt'))
         else:
             self.regionalisation = BIRD_GLOBAL_PARAMETERS
 
