@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4#
 #
-# Copyright (C) 2014-2019 GEM Foundation
+# Copyright (C) 2014-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -124,7 +124,7 @@ def get_vulnerability_functions_05(node, fname):
                 scientific.VulnerabilityFunctionWithPMF(
                     vf_id, imt, imls, numpy.array(loss_ratios),
                     all_probs))
-            # the seed will be set by readinput.get_risk_model
+            # the seed will be set by readinput.get_crmodel
         else:
             with context(fname, vfun):
                 loss_ratios = ~vfun.meanLRs
@@ -162,11 +162,12 @@ def ffconvert(fname, limit_states, ff, min_iml=1E-10):
     with context(fname, ff):
         ffs = ff[1:]
         imls = ff.imls
-    nodamage = imls.attrib.get('noDamageLimit')
+    # NB: noDamageLimit=None is now treated as noDamageLimit=0
+    nodamage = imls.attrib.get('noDamageLimit', 0)
     if nodamage == 0:
-        # use a cutoff to avoid log(0) in GMPE.to_distribution_values
-        logging.warning('Found a noDamageLimit=0 in %s, line %s, '
-                        'using %g instead', fname, ff.lineno, min_iml)
+        # use a cutoff to avoid log(0) in to_distribution_values
+        logging.debug('Using noDamageLimit=%g in %s, line %s', min_iml,
+                      fname, ff.lineno)
         nodamage = min_iml
     with context(fname, imls):
         attrs = dict(format=ff['format'],
@@ -182,7 +183,7 @@ def ffconvert(fname, limit_states, ff, min_iml=1E-10):
     if ff['format'] == 'continuous':
         minIML = float(imls['minIML'])
         if minIML == 0:
-            # use a cutoff to avoid log(0) in GMPE.to_distribution_values
+            # use a cutoff to avoid log(0) in to_distribution_values
             logging.warning('Found minIML=0 in %s, line %s, using %g instead',
                             fname, imls.lineno, min_iml)
             minIML = min_iml
@@ -255,7 +256,7 @@ def get_consequence_model(node, fname):
         node['assetCategory']  # make sure it is there
         node['lossCategory']  # make sure it is there
         cfs = node[2:]
-    functions = {}
+    coeffs = {}
     for cf in cfs:
         with context(fname, cf):
             params = []
@@ -269,12 +270,11 @@ def get_consequence_model(node, fname):
                         raise ValueError("Expected '%s', got '%s'" %
                                          (ls, param['ls']))
                     params.append((param['mean'], param['stddev']))
-            functions[cf['id']] = scientific.ConsequenceFunction(
-                cf['id'], cf['dist'], params)
+            coeffs[cf['id']] = [p[0] for p in params]
     attrs = node.attrib.copy()
     attrs.update(description=description, limitStates=limitStates)
     cmodel = scientific.ConsequenceModel(**attrs)
-    cmodel.update(functions)
+    cmodel.update(coeffs)
     return cmodel
 
 
@@ -302,8 +302,8 @@ def convert_fragility_model_04(node, fname, fmcounter=itertools.count(1)):
     new.append((Node('limitStates', {}, ' '.join(limit_states))))
     for ffs in node[2:]:
         IML = ffs.IML
-        # NB: noDamageLimit = None is different than zero
-        nodamage = ffs.attrib.get('noDamageLimit')
+        # NB: noDamageLimit=None is now treated as noDamageLimit=0
+        nodamage = ffs.attrib.get('noDamageLimit', 0)
         ff = Node('fragilityFunction', {'format': fmt})
         ff['id'] = ~ffs.taxonomy
         ff['shape'] = convert_type[ffs.attrib.get('type', 'lognormal')]
@@ -312,8 +312,7 @@ def convert_fragility_model_04(node, fname, fmcounter=itertools.count(1)):
                 attr = dict(imt=IML['IMT'],
                             minIML=IML['minIML'],
                             maxIML=IML['maxIML'])
-                if nodamage is not None:
-                    attr['noDamageLimit'] = nodamage
+                attr['noDamageLimit'] = nodamage
                 ff.append(Node('imls', attr))
             for ffc in ffs[2:]:
                 with context(fname, ffc):
@@ -326,7 +325,6 @@ def convert_fragility_model_04(node, fname, fmcounter=itertools.count(1)):
             with context(fname, IML):
                 imls = ' '.join(map(str, (~IML)[1]))
                 attr = dict(imt=IML['IMT'])
-            if nodamage is not None:
                 attr['noDamageLimit'] = nodamage
             ff.append(Node('imls', attr, imls))
             for ffd in ffs[2:]:
@@ -400,7 +398,7 @@ def update_validators():
         'occupants': valid.positivefloat,
         'value': valid.positivefloat,
         'retrofitted': valid.positivefloat,
-        'number': valid.compose(valid.positivefloat, valid.nonzero),
+        'number': valid.compose(valid.nonzero, valid.positivefloat),
         'vulnerabilitySetID': str,  # any ASCII string is fine
         'vulnerabilityFunctionID': str,  # any ASCII string is fine
         'lossCategory': valid.utf8,  # a description field
