@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2019 GEM Foundation
+# Copyright (C) 2012-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -31,9 +31,10 @@ import numpy
 from numpy.testing import assert_equal
 from scipy import interpolate, stats, random
 
-from openquake.baselib.general import CallableDict
+from openquake.baselib.general import CallableDict, cached_property
 from openquake.hazardlib.stats import compute_stats2
 
+F64 = numpy.float64
 F32 = numpy.float32
 U32 = numpy.uint32
 
@@ -76,13 +77,17 @@ def fine_graining(points, steps):
 
 
 class VulnerabilityFunction(object):
-    dtype = numpy.dtype([('iml', F32), ('loss_ratio', F32), ('cov', F32)])
+    dtype = numpy.dtype([('iml', F64), ('loss_ratio', F64), ('cov', F64)])
+    seed = None  # to be overridden
 
     def __init__(self, vf_id, imt, imls, mean_loss_ratios, covs=None,
                  distribution="LN"):
         """
         A wrapper around a probabilistic distribution function
-        (currently only the log normal distribution is supported).
+        (currently, the Log-normal ("LN") and Beta ("BT")
+        distributions are supported amongst the continuous probability
+        distributions. For specifying a discrete probability
+        distribution refer to the class VulnerabilityFunctionWithPMF.
         It is meant to be pickeable to allow distributed computation.
         The only important method is `.__call__`, which applies
         the vulnerability function to a given set of ground motion
@@ -122,15 +127,19 @@ class VulnerabilityFunction(object):
                 msg = ("It is not valid to define a loss ratio = 0.0 with a "
                        "corresponding coeff. of variation > 0.0")
                 raise ValueError(msg)
+            if distribution == 'BT':
+                if lr > 1:
+                    raise ValueError('The meanLRs must be <= 1, got %s' % lr)
+                elif cov ** 2 > 1 / lr - 1:
+                    # see https://github.com/gem/oq-engine/issues/4841
+                    raise ValueError(
+                        'The coefficient of variation %s > %s is too large '
+                        'in %s' % (cov, numpy.sqrt(1 / lr - 1), self))
 
         self.distribution_name = distribution
 
-        # to be set in .init(), called also by __setstate__
-        (self.stddevs, self._mlr_i1d, self._covs_i1d,
-         self.distribution) = None, None, None, None
-        self.init()
-
     def init(self):
+        # called by CompositeRiskModel and by __setstate__
         self.stddevs = self.covs * self.mean_loss_ratios
         self._mlr_i1d = interpolate.interp1d(self.imls, self.mean_loss_ratios)
         self._covs_i1d = interpolate.interp1d(self.imls, self.covs)
@@ -143,6 +152,8 @@ class VulnerabilityFunction(object):
             self.distribution = DegenerateDistribution()
         self.distribution.epsilons = (numpy.array(epsilons)
                                       if epsilons is not None else None)
+        assert self.seed is not None, self
+        numpy.random.seed(self.seed)  # set by CompositeRiskModel.init
 
     def interpolate(self, gmvs):
         """
@@ -174,7 +185,7 @@ class VulnerabilityFunction(object):
         :returns:
            array of E' loss ratios
         """
-        if epsilons is None:
+        if self.distribution_name == 'LN' and epsilons is None:
             return means
         self.set_distribution(epsilons)
         res = self.distribution.sample(means, covs, means * covs, idxs)
@@ -256,7 +267,7 @@ class VulnerabilityFunction(object):
 
     def __getstate__(self):
         return (self.id, self.imt, self.imls, self.mean_loss_ratios,
-                self.covs, self.distribution_name)
+                self.covs, self.distribution_name, self.seed)
 
     def __setstate__(self, state):
         self.id = state[0]
@@ -265,6 +276,7 @@ class VulnerabilityFunction(object):
         self.mean_loss_ratios = state[3]
         self.covs = state[4]
         self.distribution_name = state[5]
+        self.seed = state[6]
         self.init()
 
     def _check_vulnerability_data(self, imls, loss_ratios, covs, distribution):
@@ -276,7 +288,7 @@ class VulnerabilityFunction(object):
         assert covs is None or all(x >= 0.0 for x in covs)
         assert distribution in ["LN", "BT"]
 
-    @lru_cache(100)
+    @lru_cache()
     def loss_ratio_exceedance_matrix(self, loss_ratios):
         """
         Compute the LREM (Loss Ratio Exceedance Matrix).
@@ -291,7 +303,7 @@ class VulnerabilityFunction(object):
                     loss_ratio, mean_loss_ratio, stddev)
         return lrem
 
-    @lru_cache(100)
+    @lru_cache()
     def mean_imls(self):
         """
         Compute the mean IMLs (Intensity Measure Level)
@@ -339,14 +351,15 @@ class VulnerabilityFunctionWithPMF(VulnerabilityFunction):
     :param ratios: an array of mean ratios (M)
     :param probs: a matrix of probabilities of shape (M, L)
     """
-    def __init__(self, vf_id, imt, imls, loss_ratios, probs, seed=42):
+    seed = None
+
+    def __init__(self, vf_id, imt, imls, loss_ratios, probs):
         self.id = vf_id
         self.imt = imt
         self._check_vulnerability_data(imls, loss_ratios, probs)
         self.imls = imls
         self.loss_ratios = loss_ratios
         self.probs = probs
-        self.seed = seed
         self.distribution_name = "PM"
 
         # to be set in .init(), called also by __setstate__
@@ -364,7 +377,7 @@ class VulnerabilityFunctionWithPMF(VulnerabilityFunction):
     def set_distribution(self, epsilons=None):
         self.distribution = DISTRIBUTIONS[self.distribution_name]()
         self.distribution.epsilons = epsilons
-        self.distribution.seed = self.seed
+        self.distribution.seed = self.seed  # needed only for PM
 
     def __getstate__(self):
         return (self.id, self.imt, self.imls, self.loss_ratios,
@@ -421,7 +434,7 @@ class VulnerabilityFunctionWithPMF(VulnerabilityFunction):
         self.set_distribution(epsilons)
         return self.distribution.sample(self.loss_ratios, probs)
 
-    @lru_cache(100)
+    @lru_cache()
     def loss_ratio_exceedance_matrix(self, loss_ratios):
         """
         Compute the LREM (Loss Ratio Exceedance Matrix).
@@ -622,20 +635,6 @@ class FragilityFunctionList(list):
         return '<FragilityFunctionList %s>' % ', '.join(kvs)
 
 
-class ConsequenceFunction(object):
-    def __init__(self, id, dist, params):
-        self.id = id
-        self.dist = dist
-        self.params = numpy.array(params)
-
-    def __toh5__(self):
-        return self.params, dict(id=self.id, dist=self.dist)
-
-    def __fromh5__(self, params, attrs):
-        self.params = params
-        vars(self).update(attrs)
-
-
 class ConsequenceModel(dict):
     """
     Dictionary of consequence functions. You can access each
@@ -844,7 +843,8 @@ class BetaDistribution(Distribution):
     def sample(self, means, _covs, stddevs, _idxs=None):
         alpha = self._alpha(means, stddevs)
         beta = self._beta(means, stddevs)
-        return numpy.random.beta(alpha, beta, size=None)
+        res = numpy.random.beta(alpha, beta, size=None)
+        return res
 
     def survival(self, loss_ratio, mean, stddev):
         return stats.beta.sf(loss_ratio,
@@ -959,8 +959,7 @@ def classical_damage(
         numpy.putmask(imls, imls > max_val, max_val)
         poes = interpolate.interp1d(hazard_imls, hazard_poes)(imls)
     else:
-        imls = (hazard_imls if fragility_functions.format == 'continuous'
-                else fragility_functions.imls)
+        imls = hazard_imls
         poes = numpy.array(hazard_poes)
     afe = annual_frequency_of_exceedence(poes, investigation_time)
     annual_frequency_of_occurrence = pairwise_diff(
@@ -1274,7 +1273,7 @@ def return_periods(eff_time, num_losses):
     >>> return_periods(1, 1)
     Traceback (most recent call last):
        ...
-    AssertionError: eff_time too small: 1
+    ValueError: eff_time too small: 1
     >>> return_periods(2, 2)
     array([1, 2], dtype=uint32)
     >>> return_periods(2, 10)
@@ -1285,8 +1284,10 @@ def return_periods(eff_time, num_losses):
     array([   1,    2,    5,   10,   20,   50,  100,  200,  500, 1000],
           dtype=uint32)
     """
-    assert eff_time >= 2, 'eff_time too small: %s' % eff_time
-    assert num_losses >= 2, 'num_losses too small: %s' % num_losses
+    if eff_time < 2:
+        raise ValueError('eff_time too small: %s' % eff_time)
+    if num_losses < 2:
+        raise ValueError('num_losses too small: %s' % num_losses)
     min_time = eff_time / num_losses
     period = 1
     periods = []
@@ -1313,24 +1314,25 @@ def losses_by_period(losses, return_periods, num_events=None, eff_time=None):
 
     NB: the return periods must be ordered integers >= 1. The interpolated
     losses are defined inside the interval min_time < time < eff_time
-    where min_time = eff_time /num_events. Outside the interval they
-    have NaN values. Here is an example:
+    where min_time = eff_time /num_events. On the right of the interval they
+    have NaN values and on the left zero values. Here is an example:
 
     >>> losses = [3, 2, 3.5, 4, 3, 23, 11, 2, 1, 4, 5, 7, 8, 9, 13]
     >>> losses_by_period(losses, [1, 2, 5, 10, 20, 50, 100], 20)
-    array([ nan,  nan,  0. ,  3.5,  8. , 13. , 23. ])
+    array([ 0. ,  0. ,  0. ,  3.5,  8. , 13. , 23. ])
 
     If num_events is not passed, it is inferred from the number of losses;
     if eff_time is not passed, it is inferred from the longest return period.
     """
+    P = len(return_periods)
     if len(losses) == 0:  # zero-curve
-        return numpy.zeros(len(return_periods))
+        return numpy.zeros(P)
     if num_events is None:
         num_events = len(losses)
     elif num_events < len(losses):
         raise ValueError(
-            'There are not enough events (%d) to compute the loss curve '
-            'from %d losses' % (num_events, len(losses)))
+            'There are not enough events (%d) to compute the loss curve'
+            % num_events)
     if eff_time is None:
         eff_time = return_periods[-1]
     losses = numpy.sort(losses)
@@ -1339,9 +1341,13 @@ def losses_by_period(losses, return_periods, num_events=None, eff_time=None):
         losses = numpy.concatenate(
             [numpy.zeros(num_zeros, losses.dtype), losses])
     periods = eff_time / numpy.arange(num_events, 0., -1)
-    rperiods = [rp if periods[0] <= rp <= periods[-1] else numpy.nan
-                for rp in return_periods]
-    curve = numpy.interp(numpy.log(rperiods), numpy.log(periods), losses)
+    num_left = sum(1 for rp in return_periods if rp < periods[0])
+    num_right = sum(1 for rp in return_periods if rp > periods[-1])
+    rperiods = [rp for rp in return_periods if periods[0] <= rp <= periods[-1]]
+    curve = numpy.zeros(len(return_periods))
+    c = numpy.interp(numpy.log(rperiods), numpy.log(periods), losses)
+    curve[num_left:P-num_right] = c
+    curve[P-num_right:] = numpy.nan
     return curve
 
 
@@ -1364,7 +1370,11 @@ class LossCurvesMapsBuilder(object):
         self.weights = weights
         self.num_events = num_events
         self.eff_time = eff_time
-        self.poes = 1. - numpy.exp(- risk_investigation_time / return_periods)
+        if return_periods.sum() == 0:
+            self.poes = 1
+        else:
+            self.poes = 1. - numpy.exp(
+                - risk_investigation_time / return_periods)
 
     def pair(self, array, stats):
         """
@@ -1391,89 +1401,152 @@ class LossCurvesMapsBuilder(object):
         L = len(self.loss_dt.names)
         array = numpy.zeros((P, R, L), F32)
         for r in losses_by_event:
-            num_events = self.num_events[r]
+            num_events = self.num_events.get(r, 0)
             losses = losses_by_event[r]
             for l, lt in enumerate(self.loss_dt.names):
                 ls = losses[:, l].flatten()  # flatten only in ucerf
                 # NB: do not use squeeze or the gmf_ebrisk tests will break
-                lbp = losses_by_period(
-                    ls, self.return_periods, num_events, self.eff_time)
+                try:
+                    lbp = losses_by_period(
+                        ls, self.return_periods, num_events, self.eff_time)
+                except ValueError as exc:
+                    raise exc.__class__('%s for %s, rlz=' % (exc, lt, r))
                 array[:, r, l] = lbp
         return self.pair(array, stats)
 
-    def build_pair(self, losses, stats):
-        """
-        :param losses: a list of lists with R elements
-        :returns: two arrays of shape (P, R) and (P, S) respectively
-        """
-        P, R = len(self.return_periods), len(self.weights)
-        assert len(losses) == R, len(losses)
-        array = numpy.zeros((P, R), F32)
-        for r, ls in enumerate(losses):
-            ne = self.num_events.get(r, 0)
-            if ne:
-                array[:, r] = losses_by_period(
-                    ls, self.return_periods, ne, self.eff_time)
-        return self.pair(array, stats)
-
     # used in event_based_risk
     def build_curve(self, asset_value, loss_ratios, rlzi):
         return asset_value * losses_by_period(
             loss_ratios, self.return_periods,
-            self.num_events[rlzi], self.eff_time)
+            self.num_events.get(rlzi, 0), self.eff_time)
 
     # used in event_based_risk
-    def build_maps(self, losses, clp, stats=()):
+    def build_maps(self, curves, clp, stats=()):
         """
-        :param losses: an array of shape (A, R, P)
+        :param curves: a composite array of shape (A, R, P)
         :param clp: a list of C conditional loss poes
         :param stats: list of pairs [(statname, statfunc), ...]
         :returns: an array of loss_maps of shape (A, R, C, LI)
         """
-        shp = losses.shape[:2] + (len(clp), len(losses.dtype))  # (A, R, C, LI)
+        shp = curves.shape[:2] + (len(clp), len(curves.dtype))  # (A, R, C, LI)
         array = numpy.zeros(shp, F32)
-        for lti, lt in enumerate(losses.dtype.names):
-            for a, losses_ in enumerate(losses[lt]):
-                for r, ls in enumerate(losses_):
+        for lti, lt in enumerate(curves.dtype.names):
+            for a, curves_ in enumerate(curves[lt]):
+                for r, ls in enumerate(curves_):
                     for c, poe in enumerate(clp):
                         clratio = conditional_loss_ratio(ls, self.poes, poe)
                         array[a, r, c, lti] = clratio
         return self.pair(array, stats)
 
     # used in ebrisk
-    def build_loss_maps(self, losses, clp, stats=()):
-        """
-        :param losses: an array of shape R, E
-        :param clp: a list of C conditional loss poes
-        :param stats: list of pairs [(statname, statfunc), ...]
-        :returns: two arrays of shape (C, R) and (C, S)
-        """
-        array = numpy.zeros((len(clp), len(losses)), F32)
-        for r, ls in enumerate(losses):
-            if len(ls) < 2:
-                continue
-            for c, poe in enumerate(clp):
-                array[c, r] = conditional_loss_ratio(ls, self.poes, poe)
-        return self.pair(array, stats)
-
-    # used in ebrisk
-    def build_curves_maps(self, loss_arrays, rlzi):
+    def build_curves(self, loss_arrays, rlzi):
         if len(loss_arrays) == 0:
-            return (), ()
+            return ()
         shp = loss_arrays[0].shape  # (L, T...)
         P = len(self.return_periods)
         curves = numpy.zeros((P,) + shp, F32)
-        C = len(self.conditional_loss_poes)
-        maps = numpy.zeros((C,) + shp, F32)
-        num_events = self.num_events[rlzi]
+        num_events = self.num_events.get(rlzi, 0)
         acc = collections.defaultdict(list)
         for loss_array in loss_arrays:
             for idx, loss in numpy.ndenumerate(loss_array):
                 acc[idx].append(loss)
         for idx, losses in acc.items():
-            curves[(slice(None),) + idx] = lbp = losses_by_period(
+            curves[(slice(None),) + idx] = losses_by_period(
                 losses, self.return_periods, num_events, self.eff_time)
-            for p, poe in enumerate(self.conditional_loss_poes):
-                maps[(p,) + idx] = conditional_loss_ratio(
-                    lbp, self.poes, poe)
-        return curves, maps
+        return curves
+
+    def gen_curves_by_rlz(self, losses_by_event, ses_ratio):
+        """
+        :param losses_by_event: a dataframe
+        :param ses_ratio: ses ratio
+        :yield: triples (rlzi, curves, losses)
+        """
+        for rlzi, losses_df in losses_by_event.groupby('rlzi'):
+            losses = numpy.array(losses_df)
+            yield (rlzi, self.build_curves(losses, rlzi),
+                   losses.sum(axis=0) * ses_ratio)
+
+
+class LossesByAsset(object):
+    """
+    A class to compute losses by asset.
+
+    :param assetcol: an AssetCollection instance
+    :param policy_name: the name of the policy field (can be empty)
+    :param policy_dict: dict loss_type -> array(deduct, limit) (can be empty)
+    """
+    alt = None  # set by the ebrisk calculator
+    losses_by_E = None  # set by the ebrisk calculator
+
+    @cached_property
+    def losses_by_A(self):
+        """
+        :returns: an array of shape (A, L)
+        """
+        return numpy.zeros((self.A, len(self.loss_names)), F32)
+
+    def __init__(self, assetcol, loss_names, policy_name='', policy_dict={}):
+        self.A = len(assetcol)
+        self.policy_name = policy_name
+        self.policy_dict = policy_dict
+        self.loss_names = loss_names
+        self.lni = {ln: i for i, ln in enumerate(loss_names)}
+
+    def gen_losses(self, out):
+        """
+        :yields: pairs (loss_name_index, losses array of shape (A, E))
+        """
+        for lt in out.loss_types:
+            lratios = out[lt]  # shape (A, E)
+            losses = numpy.zeros_like(lratios)
+            avalues = (out.assets['occupants_None'] if lt == 'occupants'
+                       else out.assets['value-' + lt])
+            for a, avalue in enumerate(avalues):
+                losses[a] = avalue * lratios[a]
+            yield self.lni[lt], losses  # shape (A, E)
+            if lt in self.policy_dict:
+                ins_losses = numpy.zeros_like(lratios)
+                for a, asset in enumerate(out.assets):
+                    ded, lim = self.policy_dict[lt][asset[self.policy_name]]
+                    ins_losses[a] = insured_losses(
+                        losses[a], ded * avalues[a], lim * avalues[a])
+                yield self.lni[lt + '_ins'], ins_losses
+
+    def aggregate(self, out, eidx, minimum_loss, tagidxs, ws):
+        """
+        Populate .losses_by_A, .losses_by_E and .alt
+        """
+        numlosses = numpy.zeros(2, int)
+        for lni, losses in self.gen_losses(out):
+            if ws is not None:  # compute avg_losses, really fast
+                aids = out.assets['ordinal']
+                self.losses_by_A[aids, lni] += losses @ ws
+            self.losses_by_E[eidx, lni] += losses.sum(axis=0)
+            if tagidxs is not None:
+                # this is the slow part, depending on minimum_loss
+                for a, asset in enumerate(out.assets):
+                    idx = ','.join(map(str, tagidxs[a]))
+                    kept = 0
+                    for loss, eid in zip(losses[a], out.eids):
+                        if loss >= minimum_loss[lni]:
+                            self.alt[idx][eid][lni] += loss
+                            kept += 1
+                    numlosses += numpy.array([kept, len(losses[a])])
+        return numlosses
+
+
+# ####################### Consequences ##################################### #
+
+consequence = CallableDict()
+
+
+@consequence.add('losses')
+def economic_losses(coeffs, asset, dmgdist, loss_type):
+    """
+    :param coeffs: coefficients per damage state
+    :param asset: asset record
+    :param dmgdist: an array of probabilies of shape (E, D - 1)
+    :param loss_type: loss type string
+    :returns: array of economic losses of length E
+    """
+    return dmgdist @ coeffs * asset['value-' + loss_type]
