@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2016-2019 GEM Foundation
+# Copyright (C) 2016-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -24,10 +24,11 @@ import logging
 import threading
 import subprocess
 
-from openquake.baselib import config, sap, zeromq as z, workerpool as w
+from openquake.baselib import config, zeromq as z, workerpool as w
 from openquake.baselib.general import socket_ready, detach_process
 from openquake.baselib.parallel import safely_call
 from openquake.commonlib import logs
+from openquake.engine import __version__
 from openquake.server.db import actions
 from openquake.server import dbapi
 from openquake.server import __file__ as server_path
@@ -52,13 +53,14 @@ class DbServer(object):
     """
     def __init__(self, db, address, num_workers=5):
         self.db = db
-        self.master_host = address[0]
         self.frontend = 'tcp://%s:%s' % address
         self.backend = 'inproc://dbworkers'
         self.num_workers = num_workers
         self.pid = os.getpid()
-        self.master = w.WorkerMaster(config.dbserver.host,
-                                     **config.zworkers)
+        if ZMQ:
+            self.zmaster = w.WorkerMaster(**config.zworkers)
+        else:
+            self.zmaster = None
 
     def dworker(self, sock):
         # a database worker responding to commands
@@ -68,11 +70,16 @@ class DbServer(object):
                 if cmd == 'getpid':
                     sock.send(self.pid)
                     continue
+                elif cmd.startswith('zmq_') and self.zmaster:
+                    msg = getattr(self.zmaster, cmd[4:])()
+                    logging.info(msg)
+                    sock.send(msg)
+                    continue
                 try:
                     func = getattr(actions, cmd)
-                except AttributeError:  # missing action
+                except AttributeError:  # SQL string
                     sock.send(safely_call(self.db, (cmd,) + args))
-                else:
+                else:  # action
                     sock.send(safely_call(func, (self.db,) + args))
 
     def start(self):
@@ -90,25 +97,18 @@ class DbServer(object):
         logging.warning('DB server started with %s on %s, pid %d',
                         sys.executable, self.frontend, self.pid)
         if ZMQ:
-            # start task_in->task_out streamer thread
-            c = config.zworkers
-            threading.Thread(
-                target=w._streamer,
-                args=(self.master_host, c.task_in_port, c.task_out_port)
-            ).start()
-            logging.warning('Task streamer started from %s -> %s',
-                            c.task_in_port, c.task_out_port)
-
-            # start zworkers and wait a bit for them
-            msg = self.master.start()
-            logging.warning(msg)
-            time.sleep(1)
-
+            # start task_in->task_server streamer thread
+            threading.Thread(target=w._streamer, daemon=True).start()
+            logging.warning('Task streamer started on port %d',
+                            int(config.zworkers.ctrl_port) + 1)
+            if 'git' not in __version__:
+                # in production installations start the zworkers
+                self.zmaster.start()
         # start frontend->backend proxy for the database workers
         try:
             z.zmq.proxy(z.bind(self.frontend, z.zmq.ROUTER),
                         z.bind(self.backend, z.zmq.DEALER))
-        except (KeyboardInterrupt, z.zmq.ZMQError):
+        except (KeyboardInterrupt, z.zmq.ContextTerminated):
             for sock in dworkers:
                 sock.running = False
                 sock.zsocket.close()
@@ -119,7 +119,7 @@ class DbServer(object):
     def stop(self):
         """Stop the DbServer and the zworkers if any"""
         if ZMQ:
-            logging.warning(self.master.stop())
+            self.zmaster.stop()
             z.context.term()
         self.db.close()
 
@@ -138,7 +138,7 @@ def get_status(address=None):
     :param address: pair (hostname, port)
     :returns: 'running' or 'not-running'
     """
-    address = address or (config.dbserver.host, DBSERVER_PORT)
+    address = address or (config.dbserver.listen, DBSERVER_PORT)
     return 'running' if socket_ready(address) else 'not-running'
 
 
@@ -167,8 +167,8 @@ def ensure_on():
         # multiprocessing.Process(target=run_server).start() and apparently
         # it works, but then run-demos.sh hangs after the end of the first
         # calculation, but only if the DbServer is started by oq engine (!?)
-        subprocess.Popen([sys.executable, '-m', 'openquake.server.dbserver',
-                          '-l', 'INFO'])
+        subprocess.Popen([sys.executable, '-m', 'openquake.commands',
+                          'dbserver', 'start'])
 
         # wait for the dbserver to start
         waiting_seconds = 30
@@ -180,13 +180,14 @@ def ensure_on():
             waiting_seconds -= 1
 
 
-@sap.script
-def run_server(dbpath=os.path.expanduser(config.dbserver.file),
-               dbhostport=None, loglevel='WARN'):
+def run_server(dbhostport=None, loglevel='WARN', foreground=False):
     """
     Run the DbServer on the given database file and port. If not given,
     use the settings in openquake.cfg.
     """
+    # configure the logging first of all
+    logging.basicConfig(level=getattr(logging, loglevel.upper()))
+
     if dbhostport:  # assume a string of the form "dbhost:port"
         dbhost, port = dbhostport.split(':')
         addr = (dbhost, int(port))
@@ -194,7 +195,7 @@ def run_server(dbpath=os.path.expanduser(config.dbserver.file),
         addr = (config.dbserver.listen, DBSERVER_PORT)
 
     # create the db directory if needed
-    dirname = os.path.dirname(dbpath)
+    dirname = os.path.dirname(os.path.expanduser(config.dbserver.file))
     if not os.path.exists(dirname):
         os.makedirs(dirname)
 
@@ -208,19 +209,10 @@ def run_server(dbpath=os.path.expanduser(config.dbserver.file),
     # reset any computation left in the 'executing' state
     actions.reset_is_running(db)
 
-    # configure logging and start the server
-    logging.basicConfig(level=getattr(logging, loglevel))
-    DbServer(db, addr).start()  # expects to be killed with CTRL-C
-
-
-run_server.arg('dbpath', 'dbpath')
-run_server.arg('dbhostport', 'dbhost:port')
-run_server.opt('loglevel', 'WARN or INFO')
-
-if __name__ == '__main__':
-    if hasattr(os, 'fork') and not config.dbserver.multi_user:
+    # start the dbserver
+    if hasattr(os, 'fork') and not (config.dbserver.multi_user or foreground):
         # needed for https://github.com/gem/oq-engine/issues/3211
         # but only if multi_user = False, otherwise init/supervisor
         # will loose control of the process
         detach_process()
-    run_server.callfunc()
+    DbServer(db, addr).start()  # expects to be killed with CTRL-C
