@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2014-2019 GEM Foundation
+# Copyright (C) 2014-2020 GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -105,7 +105,9 @@ MIDDLEWARE = (
     'django.middleware.csrf.CsrfViewMiddleware',
 )
 
+# Authentication is not enabled by default
 LOCKDOWN = False
+# Allow all users to see other users outputs by default
 ACL_ON = False
 
 # Add additional paths (as regular expressions) that don't require
@@ -203,8 +205,6 @@ except ImportError:
 
 if LOCKDOWN:
 
-    ACL_ON = True
-
     AUTHENTICATION_BACKENDS += (
         'django.contrib.auth.backends.ModelBackend',
         # 'dpam.backends.PAMBackend',
